#!/usr/bin/python
import numpy as np
import os
from shutil import copy2, copyfile
import subprocess as sb
import sys

def nparFromMod(model):
    if model=='relativistic':
        nparam = 10
        nlocal=6
    elif model=='kepler':
        nparam = 7
        nlocal=6
    elif model=='keplerRedshift':
        nparam = 8
        nlocal=6
    elif model=='5thforce':
        nparam=9
        nlocal=6
    return nparam,nlocal


def createDir(n):
    dir = os.path.dirname(n)
    if not os.path.isdir(dir):
        os.makedirs(dir)

def readOutput(name):
    fout = open(name,'r')
    isGlobal=False
    sig=[]
    for l in fout:
        if isGlobal:
            ls=l.split()
            if len(ls)<1:
                break
            if ls[1]=='-NF-' :
                sig.append(-1)
            else:
                sig.append(float(ls[1]))
        elif '== Global' in l:
            isGlobal=True
    fout.close()
    return sig


def main(inFile,idxParam,deltma,N):
    fin=open(inFile,'r')
    lines=fin.readlines()
    fin.close()
    folder=lines[0].rstrip()
    nameSimu=lines[1].rstrip()
    createDir(folder+'/bla')
    
    model = lines[3].split()[0].rstrip()
    print '    Model: ',model
    nparam,nlocal=nparFromMod(model)
    npriors = int(lines[4].split()[0])
    nstars = int(lines[nparam+5+npriors].split()[0])
    print '        Num of global parameters: ',nparam
    print '        Num of local parameters: ',nlocal
    print '        Num of stars: ',nstars
    print '        Num of priors: ',npriors

    if idxParam<=nparam:
        isGlobal = True
        idxL = 4+npriors+idxParam
    
    else:
        isGlobal=False
        idxS = (idxParam-1-nparam)/nlocal
        idxP = idxParam-nparam-idxS*nlocal
        idxL = 9+npriors+nparam+idxS*(nlocal+4)+idxP

    parOpt = float(lines[idxL].split()[0])
    parMin = parOpt - 5*delta
    parMax = parOpt + 5*delta
    par    = np.linspace(parMin,parMax,2*N+1)
    isParFitted = lines[idxL].split()[1]


    foutput = open(folder+'/'+nameSimu+'.err','w')
    for i in range(len(par)):
        fn= folder+'/'+inFile+'_'+str(i)
        
        lines[1] = nameSimu+'_'+str(i)+' \n'
        lines[idxL]=str(par[i])+'   '+isParFitted+' \n'
        fout = open(fn,'w')
        fout.writelines(lines)
        fout.close()
        sb.check_call(['./main',fn])
        sig = readOutput(folder+'/'+nameSimu+'_'+str(i)+'.res')
        tmp = nparam*' %14.6f'
        foutput.write(' %6i  %14.6f   %14.6f '%(i+1,par[i]-parOpt,par[i]) + tmp%tuple(sig)+' \n')
    foutput.close()

idxParam = 11
inFile   = 'input_error'
delta = 30/365.25
Nsimu = 100.

if len (sys.argv)>1:
    inFile=sys.argv[1]
if len (sys.argv)>2:
    idxParam = int(sys.argv[2])
if len (sys.argv)>3:
    delta = float(sys.argv[3])

main(inFile,idxParam,delta,Nsimu)

