#!/usr/bin/python
import numpy as np
import os
from shutil import copy2, copyfile
import subprocess as sb
import sys

def readInputCadence(fn):
    print ' READING input from Cadence from file ',fn
    fin=open(fn,'r')
    lines=fin.readlines()
    fin.close()
    outfolder = lines[0].rstrip()
    outname = lines[1].rstrip()
    niter = int(lines[2].split()[0])
    nintervalsAstro = int(lines[3].split()[0])
    nintervalsRV = int(lines[4+nintervalsAstro].split()[0])
    nintervals = nintervalsAstro+nintervalsRV
    model = lines[7+nintervals].split()[0].rstrip()
    npriors = int(lines[8+nintervals].split()[0])
    
    print 'Model: ',model
    nparam,nlocal=nparFromMod(model)
    

    nstars = int(lines[nparam+9+npriors+nintervals].split()[0])

    nameList  = []
    astroList = []
    RVList    = []
        
    print 'Output folder: ', outfolder
    print 'Output name: ', outname
    print 'Number of iterations: ',niter
    print 'Number of stars: ',nstars
    print 'Number of priors: ',npriors
    print 'Number of astro intervals: ',nintervalsAstro
    print 'Number of RV intervals: ',nintervalsRV
    
    
    
    for i in range(1,nstars+1):
        nameList.append(lines[10+npriors+nintervals+nparam+(i-1)*(nlocal+5)].rstrip())
        astroList.append(lines[11+npriors+nintervals+nparam+(i-1)*(nlocal+5)].rstrip())
        RVList.append(lines[12+npriors+nintervals+nparam+(i-1)*(nlocal+5)].rstrip())
        print '  - Stars ',i
        print '    Name: ',nameList[-1]
        print '    Astro file: ',astroList[-1]
        print '    RV file: ',RVList[-1]
        print ' '

    inputFile=['res_'+outname]
    inputFile.append(outname+'_')
    inputFile.append('0')
    inputFile.append(lines[7+nintervals].rstrip())#model
    inputFile.append(lines[8+nintervals].rstrip())#npriors
    if npriors>0:
        for k in range(npriors):
            inputFile.append(lines[9+nintervals+k].rstrip())


    for k in range(1+nparam):
        inputFile.append(lines[9+npriors+nintervals+k].rstrip())
    for k in range(nstars):
        inputFile.append(nameList[k].rstrip())
        if astroList[k]=='None':
            inputFile.append('None')
        else:
            inputFile.append(astroList[k]+'_')
        if RVList[k]=='None':
            inputFile.append('None')
        else:
            inputFile.append(RVList[k]+'_')
        inputFile.append(lines[13+npriors+nintervals+nparam+k*(nlocal+5)].rstrip())

        for i in range(nlocal):
            inputFile.append(lines[15+npriors+nintervals+i+nparam+k*(nlocal+5)].rstrip())
        
    
    
    return outfolder,outname,niter,nstars,npriors,nameList,astroList,RVList,inputFile

def readOutputCadence(outF,outN,niter):
    print 'READING output from cadence tool from file ',outF+'/'+outN+'.res'
    fin = open(outF+'/'+outN+'.res','r')
    inTab = False
    tab = []
    for l in fin:
        if not l.split():
            continue
        if l.split()[0]=='iteration':
            inTab=True
            counter = 0
            continue
        if inTab:
            ls =l.split()
            tab.append ([ float(ls[4]) ,int(ls[0]), ls[1],ls[2],float(ls[6])/1000.,float(ls[5])/1000.,ls[7], float(ls[3])])#divide by 1000 to come back to as from mas
    tabN = np.array(tab)
    arg  = np.argsort(tabN[:,0])
    return tabN[arg,:]

def createData(outF,nameF,niter,nstars,npriors,nameList,astroList,RVList,tabIter,covInput):
    print 'CREATING data files and input files for covariance analyses'

    nparam,nlocal=nparFromMod(covInput[3].split()[0])


    for i in range(nstars):
        if astroList[i]!='None':
            createDir(outF+'/'+astroList[i])
            copyfile(astroList[i],outF+'/'+astroList[i]+'_0')

        if RVList[i]!='None':
            createDir(outF+'/' + RVList[i])
            copyfile(RVList[i],outF+'/' + RVList[i]+'_0')
    
  
    writeInputCov(outF+'/'+nameF+'_cov',0,covInput,nparam,nlocal,nstars,npriors)

    for j in range(1,niter+1):
        idxStar = findStars(nameList,tabIter[j-1,2]) #np.argwhere(nameList==str(tabIter[0,2]))
    
        for i in range(nstars):
            print i,nstars,idxStar,tabIter[j-1,3]
            if astroList[i]!='None':
                copyfile(outF+'/'+astroList[i]+'_'+str(j-1) , outF+'/'+astroList[i]+'_'+str(j))
        
            if RVList[i]!='None':
                copyfile(outF+'/'+RVList[i]+'_'+str(j-1) , outF+'/'+RVList[i]+'_'+str(j))
    
        if tabIter[j-1,3]=='rv':
            if RVList[idxStar]=='None':
                createDir(outF+'/data' +nameList[idxStar]+'.rv_1')
            
                fout = open(outF+'/data/'+nameList[idxStar]+'.rv_'+str(j),'w')
                fout.write(str(tabIter[j-1,7])+'   0.      '+tabIter[j-1,6]+'   '+tabIter[j-1,0])
                fout.close()
            
                covInput[nparam+7+idxStar*nlocal]=outF+'/data/'+nameList[idxStar]+'.rv_'
            else:
                fout = open(outF+'/'+RVList[idxStar]+'_'+str(j),'a')
                fout.write('\n'+str(tabIter[j-1,7])+'   0.       '+tabIter[j-1,6]+'   '+tabIter[j-1,0])
                fout.close()

        if tabIter[j-1,3]=='astro':
            if astroList[idxStar]=='None':
                createDir(outF+'/data' +nameList[idxStar]+'.points_1')

                fout = open(outF+'/data/'+nameList[idxStar]+'.points_'+str(j),'w')
                fout.write(str(tabIter[j-1,7])+'   0.    0.   '+tabIter[j-1,4]+'   '+tabIter[j-1,5]+'   '+tabIter[j-1,0])
                fout.close()
            
                covInput[nparam+6+idxStar*nlocal]=outF+'/data/'+nameList[idxStar]+'.points_'
            else:
                fout = open(outF+'/'+astroList[idxStar]+'_'+str(j),'a')
                fout.write('\n'+str(tabIter[j-1,7])+'   0.    0.   '+tabIter[j-1,4]+'   '+tabIter[j-1,5]+'   '+tabIter[j-1,0])
                fout.close()
        writeInputCov(outF+'/'+nameF+'_cov',j,covInput,nparam,nlocal,nstars,npriors)


def writeInputCov(nf,idx,ci,npar,nl,ns,npriors):
    fout = open(nf+'_'+str(idx),'w')
    fout.write(ci[0]+'\n')
    fout.write(ci[1]+str(idx)+'\n')
    fout.write(ci[2]+'\n')
    fout.write(ci[3]+'\n')
    fout.write(ci[4]+'\n') #npriors


    if npriors>0:
        for j in range(npriors):
            fout.write(ci[5+j]+' \n')
    for j in range(npar+1):
        fout.write(ci[5+npriors+j]+' \n')

    for j in range(ns):
        fout.write(ci[6+npriors+npar+j*(nl+4)] + ' \n')
        fout.write(ci[7+npriors+npar+j*(nl+4)] + str(idx) + ' \n')
        fout.write(ci[8+npriors+npar+j*(nl+4)] + str(idx) + ' \n')
        for k in range(nl+1):
            fout.write(ci[9+k+npriors+npar+j*(nl+4)] + ' \n')


def nparFromMod(model):
    if model=='relativistic':
        nparam = 10
        nlocal=6
    elif model=='kepler':
        nparam = 7
        nlocal=6
    elif model=='keplerRedshift':
        nparam = 8
        nlocal=6
    elif model=='5thforce':
        nparam=9
        nlocal=6
    return nparam,nlocal

def createDir(n):
    dir = os.path.dirname(n)
    if not os.path.isdir(dir):
        os.makedirs(dir)


def findStars(list,name):
    counter=0
    for l in list:
        if l==name:
            return counter
        counter=counter+1

def runCov(outFolder,outname,niter):
    copy2('main',outFolder+'/main')
    os.chdir(outFolder)

    for i in range(niter+1):
        sb.call(['./main',outname+'_cov_'+str(i)])



if len (sys.argv)==2:
    outputCadenceFile=sys.argv[1]
else:
    outputCadenceFile = 'input_cadence'
print 'Input file: ', outputCadenceFile


outfolder,outname,niter,ns,npriors,starsName,astroFiles,RVFiles,covInput = readInputCadence(outputCadenceFile)
tabIter = readOutputCadence(outfolder,outname,niter)

createData(outfolder,outname,niter,ns,npriors,starsName,astroFiles,RVFiles,tabIter,covInput)
runCov(outfolder,outname,niter)

