import os
import subprocess as sb
import glob
import numpy as np
import sys

import inputs as ins
sys.path.append('/u/arezu/Documents/GCG/GRtests/confusion/')
from constants import  nparFromMod, stop


def name_GR_test(__model, __lines, __nextMparams):
	'''
	to name the test that we are running by reading the main input file
	and get the global params
	idea: this function is written for the GR tests. This functions looks at the global parameters listed below and see if they're fitted or not.
			global prameters: ExtM[5], Redshift[3], GR[3], Q[3], Spin[3]
			in the input file line:	5+npriors - 5+npriors+nglobal :: global parameters (ex: GM, x0, y0, vx0, vy0, R0, extM., redshift, GR, Quadrupole, Spin)
	parameters:
			__model: string, the name of the model
			__lines: the lines containing the global parameters in the main input file
	return:
			__tname: a string. it contains the names of the parameters we're fitting. example "Redshift-ExtM-"
			__nglob: number of global params
			__nloc: number of local params
			gparams: NUMPY global params
			gfits: NUMPY if they're fit or not (int)
			extmparams: NUMPY the two extra params of the extended mass, r and \gamma
			gparnames: string, names of the gparams
	'''
	__tname=''
	__nglob, __nloc = nparFromMod(__model)

	gparams=np.zeros(__nglob)#=nglobal
	gfits=np.zeros((__nglob), dtype=np.int)# 1 or 0
	extmparams=np.zeros(__nextMparams)# 2 elements
	gparnames=([None])*__nglob
	gparunits=([None])*__nglob


	for j in range(__nglob):
		gparams[j] = float(__lines[j].split()[0])# the values float
		if j<7:
			gfits[j] = int(__lines[j].split()[1])#GM, pos/vel, R0
			gparnames[j] = __lines[j].split()[3]
			gparunits[j] = __lines[j].split()[4]
		elif j==7:	#extM
			if __model !='keplerAdvance':
				gfits[j] = int(__lines[j].split()[3])#int
				extmparams = [float(__lines[j].split()[1]), float(__lines[j].split()[2])]#extM extra params
				gparnames[j] = __lines[j].split()[5] #name
				gparunits[j] = __lines[j].split()[6] #unit
				if int(__lines[j].split()[3])==1:	#extM
					__tname = __tname + __lines[j].split()[5]+'-'
			elif __model == 'keplerAdvance':
				gfits[j] = int(__lines[j].split()[1])#int
				gparnames[j] = __lines[j].split()[3] #name
				gparunits[j] = __lines[j].split()[4] #unit
				if int(__lines[j].split()[1])==1:
					__tname = __tname + __lines[j].split()[3] + '-'

		elif(j>7):#this is only to get the test name and since we only care about GR tests (for now, no priors involved yet) we only care about j=>7
			gfits[j] = int(__lines[j].split()[1])
			gparnames[j] = __lines[j].split()[3]
			gparunits[j] = __lines[j].split()[4]
			if int(__lines[j].split()[1])==1:
				__tname= __tname  + __lines[j].split()[3] + '-'
	if __model=='keplerAdvance':
		__tname = __tname + 'dw' + '-'

	#BHpos/velfixed
	if(gfits[1] and gfits[2] and gfits[3] and gfits[4]):
		__BHfixed=False
	else:
		__BHfixed=True

	return __tname[:-1], __nglob, __nloc, np.array(gparams), np.array(gfits), np.array(extmparams), gparnames, gparunits,  __BHfixed

def read_local_params(loclines):
	'''
	read local params
	'''
	locpars=[]
	locunits=[]
	for line in loclines:
		locpars.append(line.split()[3])
		locunits.append(line.split()[4])

	return	locpars, locunits


def read_res_file(__resfile, __nglobal, __nlocal, __nstars):
	'''
	reads a .res file
	parameters:
			__resfile: the .res file address
			__nglobal: nglobal
			__nlocal: nlocal
			__nstars: nstars
	return:
			guncert: NUMPY array of the global param uncert, if('-NF-'):=zero
			guncert: NUMPY array of the local param uncert, if('-NF-'):=zero (len=nlocal*nstars)
	'''
	resf = open(__resfile, 'r')
	lines = list(line for line in (l.strip() for l in resf) if line) # NO EMPTY LINE
	resf.close()

	
	# global uncert
	guncert=np.zeros(__nglobal)# if unfit = zero
	for j in range(len(lines)):
		if lines[j].rstrip() == 'Resulting uncertainties:':# this one+2 is the first global param
			globline=j+2# first global param
			break
		else:
			pass#print 'no uncertainties found ... !'
			
	c=0
	for i in range(globline, globline+ __nglobal):
		if(lines[i].split()[1] != '-NF-'):
			guncert[c] = float(lines[i].split()[1])
		c=c+1
	



	# local uncert
	c=0
	luncert=np.zeros(__nstars* __nlocal)
	for j in range( __nstars):
		for i in range( __nlocal):
			if( lines[globline+ __nglobal+2 + j*( __nlocal+1) + i ].split()[1] !='-NF-'):
				luncert[c] =  float(lines[globline+ __nglobal+2 + j*(__nlocal+1) + i ].split()[1])
			c=c+1

	
	# correlations

	return guncert, luncert



class covariance_analysis:
	'''
	class covariance_analysis:	keeps the information of a test from start to finish: (from .in and .res)
	list of functions:
					__init__(self, the input file)
					read_res(self, .res file)
					read_sensitivity_res(self, .res filezzzz)
	'''
	
	def __init__(self, inputfile, nextMparams, outdir):
		'''
		initialize a test
		parameters:
					the input file
					structure of the input file:
						0 :: folder
						1 :: file
						2 :: 0/1 decimal year !compute the next uncertainty
						3 :: model (get the nglobal and nlocal from the model_function)
						4 :: number of priors
						5 :: if(number of priors)
						5+npriors - 5+npriors+nglobal :: global parameters (ex: GM, x0, y0, vx0, vy0, R0, extM.(0,1,2), redshift, GR, Quadrupole, Spin)

						5+npriors+nglobal :: nstars (the number of stars)  
						6+npriors+nglobal :: star.name
						7+npriors+nglobal :: star.pointsfile
						8+npriors+nglobal :: star.RVfile
						nstars(*10+npriors+nglobal - 10+npriors+nglobal+nlocal) :: star local parameters (Period, T0, ecc, inclination, small omega, big omega)

		self:
			name: simply just the  name of the file
			foldername: folder name
			filename: file name
			testname: foldername minus the number of stars and the rest
			BHfixed: true(0) or false(1)
			compnext: zero or one, to compute the uncert on the next observation
			comp_next_time: the time of the next observation
			npriors: int(n priors)
			model: model
			nglobal: n global
			nlocal: n local
			globalparams: NUMPY global params
			globalfits: NUMPY one or zero int, whether or not it is fitted
			extMparams: NUMPY the two external params of the extended mass
			nstars: nstars int
			stars_name: list of the stars
			stars_points_address: the addresses of the points files
			stars+RV_address: the addresses of the RV files
			res_address: the immediate .res address
			res_sensitivity_dir: directory of all the .res files from the sensitivity analysis
		if none: values=zero.
		'''
		self.name = inputfile.split('/')[-1].split('.')[0]
		self.resDir = outdir
		
		print '--- Covariance analysis on the input file: ', self.name
		inf = open(inputfile, 'r')
		lines = list(line for line in (l.strip() for l in inf) if line) # NO EMPTY LINE
		inf.close()

		# first lines
		self.foldername = lines[0].rstrip()
		self.filename = lines[1].rstrip()
		self.fittype = int(lines[2].split()[0])#0 and 1
		
		#sen steps
		self.begin_sensitivity = float(lines[3].split()[0])#
		self.end_sensitivity = float(lines[3].split()[1])# 
		self.increment_sensitivity = float(lines[3].split()[2])# 
		self.years=np.arange( self.begin_sensitivity ,	self.end_sensitivity+self.increment_sensitivity  ,	self.increment_sensitivity )
		
		# priors
		self.npriors = int(lines[5].split()[0]) # it will expand once we add some priors. right now I am assuming that we have zero priors becasue I'm not reading anything

		# model and global params
		self.model = lines[4].split()[0]
		self.testname, self.nglobal, self.nlocal, self.globalparams, self.globalfits, self.extMparams, self.globparnames, self.globparunits, self.BHfixed \
									= name_GR_test(self.model, lines[6+self.npriors:], nextMparams)# you can add an array of the name of the global params

		self.par2testlist=self.testname.split('-')
		if 'Redshift' in self.par2testlist:
			self.par2testlist += ['vz0']

		# stars
		self.nstars = int(lines[6+self.npriors +self.nglobal].split()[0])
		s_names=[None]* self.nstars
		s_points_addr =[None]* self.nstars
		s_RV_addr =[None]* self.nstars

		# star names and addresses
		for j in range(self.nstars):
			s_names[j] = str(lines[7+self.npriors +self.nglobal +j*(self.nlocal+3) ].split()[0])
			s_points_addr[j] = str(lines[8+self.npriors +self.nglobal +j*(self.nlocal+3) ].split()[0])#even if it is 'None' it still takes it
			s_RV_addr[j] = str(lines[9+self.npriors +self.nglobal +j*(self.nlocal+3) ].split()[0])
		self.localparamnames, self.localunits = read_local_params(lines[10+self.npriors +self.nglobal +j*(self.nlocal+3): 10+self.npriors +self.nglobal +j*(self.nlocal+3)+self.nlocal   ])

		self.stars_names = s_names
		self.stars_points_address = s_points_addr
		self.stars_RV_address = s_RV_addr


		# signatures
		self.signature = self.name.split('-'+str(self.nstars)+'-')[-1]#subtract the rest of the names in the filename to get the signature: like tot or whatever else
		self.prefix = self.name.split('-'+str(self.nstars)+'-')[0]#subtract the rest of the names in the filename to get the signature: like tot or whatever else
		
		# .sen addresses and dirs
		self.sen_address = self.foldername+'/'+self.filename+'.sen'
		self.senall_address = self.foldername+'/'+self.filename+'_all.sen'
		self.usedstars = self.foldername+'/'+self.filename+'_stars.sen'
		self.ranking_files=[self.resDir+'ranking/'+p+'_'+self.prefix+'_ranking.dat' for p in self.par2testlist]		 #outdir+p+'_'+prefix+'__ranking.dat'
		self.best_ranked_files=[self.resDir+'ranking/'+p+'_'+self.prefix+'_bests.dat' for p in self.par2testlist]		 #outdir+p+'_'+prefix+'__ranking.dat'
		self.ufrac_files=[self.resDir+'ranking/'+p+'_'+self.prefix+'_ufrac.dat' for p in self.par2testlist]		 #outdir+p+'_'+prefix+'__ranking.dat'
		self.uncert_files=[self.resDir+'ranking/'+p+'_'+self.prefix+'_uncert.dat' for p in self.par2testlist]		 #outdir+p+'_'+prefix+'__ranking.dat'
		
		# others
		self.neverused=None



	def read_sen_all_file(self):
		'''
		read sen_all.sen
		'''
		print '--- -- Reading the %s file' %str(self.senall_address)
		if os.path.exists(self.senall_address):
			saf = np.loadtxt(self.senall_address)
			self.senyears_all =  saf[:,0]
			self.nstarsused_all =  saf[:,1]
			if len(self.senyears_all)!=len(self.years):
				stop('rerun %s' % self.name)


			# I am commenting this just because I dont need it at the moment
			'''
			#gpar
			globuncert = [None]*self.nglobal
			for g in range(self.nglobal):
				globuncert[g] = saf[:,g+2]
			self.globuncert_all = globuncert
			#locpars
			starslocuncert= [None]*self.nstars
			if self.fittype==0:
				for s in range(self.nstars):
					loctmp=[None]*self.nlocal
					for l in range(self.nlocal):
						loctmp[l] = saf[:,2+self.nglobal + s*self.nlocal +l]
					starslocuncert[s]= loctmp
				self.starslocuncert_all = starslocuncert
			'''
				
		else:
			stop( '%s does not exists' %self.senall_address)
		return






	"""
	def read_sen_file(self, senallfile):
		'''
		read sen.sen
		'''

		print '--- -- Reading the .sen file'
		if os.path.exists(senallfile):
			saf = np.loadtxt(senallfile)
			self.senyears =  saf[:,0]
			self.nstarsused =  saf[:,1]
			#gpar
			globuncert = [None]*self.nglobal
			for g in range(self.nglobal):
				globuncert[g] = saf[:,g+2]
			self.globuncert = globuncert
			#locpars
			starslocuncert= [None]*self.nstars
			if self.fittype==0:
				for s in range(self.nstars):
					loctmp=[None]*self.nlocal
					for l in range(self.nlocal):
						loctmp[l] = saf[:,2+self.nglobal + s*self.nlocal +l]
					starslocuncert[s]= loctmp
				self.starslocuncert = starslocuncert
				
		else:
			print 'File does not exists'
		return

	def read_res(self,resfile):
		'''
		reads the immediate result from the input
		inputs:
			resfile: the address of the .res file
		self.parameters:
			self.global_uncertainties: NUMPY list of the global uncertainties. if(-NF-):=float(zero)
			self.local_uncertainties: NUMPY list of the local uncertainties. if(-NF-):=float(zero), len == nlocal*nstars
		'''
		print '--- Reading the immediate results from the input file'

		self.global_uncertainties_res = np.zeros(self.nglobal)# if unfit = zero
		self.local_uncertainties_res = np.zeros(self.nstars*self.nlocal)
		if os.path.exists(resfile):
			self.global_uncertainties_res, self.local_uncertainties_res =  read_res_file(resfile, self.nglobal, self.nlocal, self.nstars)
		else:
			print resfile, 'FILE does not exists'



		return


	def read_sensitivity_res(self, __nsen):
		'''
		read the .res files from sensitivity analysis
		inputs:
			__nsen: number of sensitivity steps
		self.parameters:
			self.sen_global_uncertainties: NUMPY list of the global uncertainties. if(-NF-):=float(zero), reshape(__nsen, nglobal)
			self.sen_local_uncertainties: NUMPY list of the local uncertainties. if(-NF-):=float(zero), reshape(__nsen, nlocal*nstars)
		'''
		print '--- Reading the .res files from the sensitivity analysis'
		self.sen_global_uncertainties_res = np.zeros(self.nglobal*__nsen).reshape((__nsen, self.nglobal))# if unfit = zero
		self.sen_local_uncertainties_res = np.zeros(self.nstars*self.nlocal*__nsen).reshape((__nsen, self.nlocal*self.nstars))

		if os.path.isdir(self.res_sensitivity_dir):
			for n in range(__nsen):
				sen_res = self.res_sensitivity_dir+self.filename+'-'+str(n)+'.res'
				if os.path.exists(sen_res):
					self.sen_global_uncertainties_res[n], self.sen_local_uncertainties_res[n] =  read_res_file(sen_res, self.nglobal, self.nlocal, self.nstars)

				else:
					print sen_res, 'FILE does not exists'
		else:
			print __nsen, self.res_sensitivity_dir, 'FOLDER does not exists'


		return
	"""