import numpy as np
from matplotlib import pyplot as plt
import matplotlib.patches as mpatches
from math import sqrt, pi
import matplotlib.cm as cm
import glob
import sys
sys.path.append('/u/arezu/Documents/GCG/GRtests/confusion/')
from constants import tosort, tosort_inverted,cc,stop, cc, set_colors_default_case, get_random_color


def set_color(__fixed):
	if(__fixed):
		__c='red'
	else:
		__c='blue'
	return __c


def make_legend(__objects):
	'''
	creates the legend for a plot
	parameters:
				__objects: the objects (stars). we need their name and color
	return:
				recs: the shape and the size of the patches in the legend
				names: the name of the stars
	'''
	recs=[]
	names=[]
	for i in range(len(__objects)):
		if __objects[i].BHfixed:
			recs.append(mpatches.Rectangle((0,0),0.5,0.5,fc= set_color(__objects[i].BHfixed)))
			names.append('BH fixed')
		else:
			recs.append(mpatches.Rectangle((0,0),0.5,0.5,fc= set_color(__objects[i].BHfixed)))
			names.append('BH not fixed')

	names = np.array(names)
	recs = np.array(recs)
	tmpnames =  list(set(names))
	recstmp=[]
	for tmpname in tmpnames:
		idx, = np.where(names==tmpname)
		recstmp.append(recs[idx[0]])
	recs=np.array(recstmp)
	return recs, names

def assign_linestyle(__covt):
	'''
	if BHfixed or not
	'''
	if __covt.BHfixed:
		ls='dashed'
	else:
		ls='solid'
	return ls


def calculate_delta_r(dxdy_array):
	'''
	calculate the distance
	parameters: 
				dxdy_array: a 2D array of dx and dy
	return:
			deltar: the distance. it is a positive float
	'''
	deltar=0
	for var in dxdy_array:
		deltar = deltar + var**2
	return sqrt(deltar)
#-----------------------------------------------------------------------------------------------------

def plot_domega(__plotname, covtests, plotsdirectory, lparname, __parn, __numstars,thetest, __lunit):
	'''
	domega
	'''
	print '--- -- Plot the domega'
	fig = plt.figure('lpar'+lparname)
	__font= {'weight':'bold'}
	for covt in covtests:
		for s in range(covt.nstars):
			__color=set_color(covt.BHfixed)
			__linestyle = set_linestyle(covt)
			if __linestyle=='solid':
				tmpyr=[]
				tmppar=[]
				for var, yr in zip(covt.starslocuncert[s][__parn], covt.senyears):
					if var != -1.:
						tmpyr.append(yr)
						tmppar.append(var)
				if len(tmpyr):
					if covt.stars_names[s]=='S0-2' and covt.BHfixed==True:
						__color='black'
					elif covt.stars_names[s]=='S0-2' and covt.BHfixed==False:
						__color='grey'
					dwplot = plt.semilogy(tmpyr,tmppar, color=__color, ls=__linestyle)
					#dwplot = plt.annotate( tmpstars[-1], xy=(yr[-1], var[-1]), xytext=(60, 10),  textcoords='offset points', ha='right', va='bottom', color=__color, fontsize=6,\
					#			#bbox=dict(boxstyle='round,pad=0.5', fc='yellow', alpha=0.5), \
					#			arrowprops=dict(arrowstyle = '->', connectionstyle='arc3,rad=0', color='grey')\
					#			)

	theshapes, thenames = make_legend(covtests)
	plt.legend(theshapes, thenames, loc='best', bbox_to_anchor=(1.135,1.1), scatterpoints=1, borderpad=0.3, labelspacing=0.1)

	__title = thetest#+', - ('+str(__numstars)+' stars)'
	plt.title(__title, **__font)
	plt.xlabel('time [yr]', **__font)
	plt.ylabel(lparname+' ['+__lunit+']', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	plt.ylim(ymax=1./100.)
	#plt.gca().set_aspect('equal', adjustable='box')
	plt.savefig(plotsdirectory+'/'+__plotname+'-'+lparname+'.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	return


def plot_nstars_grow(covt, plotsdirectory):
	'''
	nstar grow
	'''
	print '--- -- Plot the number of stars used in the analysis'
	fig = plt.figure('nstargrow')
	__font= {'weight':'bold'}

	__color=set_color(covt.BHfixed)
	gplot = plt.plot(covt.senyears, covt.nstarsused, color=__color)


	__title = '('+str(covt.nstars)+' stars)'
	plt.title(__title, **__font)
	plt.xlabel('time [yr]', **__font)
	plt.ylabel('Number of stars= orbits.dat + simulated', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	#plt.gca().set_aspect('equal', adjustable='box')
	plt.savefig(plotsdirectory+'/numstars.png', bbox_inches='tight', dpi=600)
	plt.close(fig)
	return

def plot_single_par_SNR_all(__plotname, covtests, plotsdirectory, gparname, __parn, __numstars,thetest, __gparval):
	'''
	plot each param
	'''

	fig = plt.figure('gpar'+(gparname))
	__font= {'weight':'bold'}
	for covt in covtests:
		if covt.globuncert_all[__parn].all() !=-1.0:
			__ls = assign_linestyle(covt)
			gplot = plt.plot(covt.senyears_all, np.divide(__gparval,covt.globuncert_all[__parn]), label=covt.signature, ls=__ls)

	plt.legend(loc='best', bbox_to_anchor=(1.135,1.1), scatterpoints=1, borderpad=0.3, labelspacing=0.1)

	__title = thetest+' -- dashed = BHfixed'
	plt.title(__title, **__font)
	plt.xlabel('time [yr]', **__font)
	plt.ylabel(gparname+ ' SNR', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	#plt.gca().set_aspect('equal', adjustable='box')
	plt.savefig(plotsdirectory+'/all_SNR-'+__plotname+'-'+gparname+'.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	return


def plot_single_par_SNR(__plotname, covtests, plotsdirectory, gparname, __parn, __numstars,thetest, __gparval):
	'''
	plot each param
	'''

	fig = plt.figure('gpar'+(gparname))
	__font= {'weight':'bold'}
	for covt in covtests:
		if covt.globuncert[__parn].all() !=-1.0:
			__ls = assign_linestyle(covt)
			gplot = plt.plot(covt.senyears, np.divide(__gparval,covt.globuncert[__parn]), label=covt.signature, ls=__ls)
				
	plt.legend(loc='best', bbox_to_anchor=(1.135,1.1), scatterpoints=1, borderpad=0.3, labelspacing=0.1)

	__title = thetest+' -- dashed = BHfixed'
	plt.title(__title, **__font)
	plt.xlabel('time [yr]', **__font)
	plt.ylabel(gparname+ ' SNR', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	#plt.gca().set_aspect('equal', adjustable='box')
	plt.savefig(plotsdirectory+'/SNR-'+__plotname+'-'+gparname+'.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	return


def plot_single_par_all(__plotname, covtests, plotsdirectory, gparname, __parn, __numstars,thetest, __gunit):
	'''
	plot each param
	'''
	fig = plt.figure('gpar'+gparname)
	__font= {'weight':'bold'}

	for covt in covtests:
		if covt.globuncert_all[__parn].all() !=-1.:
			__ls = assign_linestyle(covt)
			if gparname != 'ExtM':
				gplot = plt.plot(covt.senyears_all, covt.globuncert_all[__parn], label=covt.signature ,ls=__ls)
			elif gparname == 'ExtM':
				gplot = plt.semilogy(covt.senyears_all, covt.globuncert_all[__parn], label=covt.signature ,ls=__ls)
				

	plt.legend(loc='best', bbox_to_anchor=(1.135,1.1), scatterpoints=1, borderpad=0.3, labelspacing=0.1)

	__title = thetest+' -- dashed = BHfixed'
	plt.title(__title, **__font)
	plt.xlabel('time [yr]', **__font)
	plt.ylabel(gparname+' ['+__gunit+']', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	#plt.gca().set_aspect('equal', adjustable='box')
	plt.savefig(plotsdirectory+'/all_'+__plotname+'-'+gparname+'.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	return



def plot_single_par(__plotname, covtests, plotsdirectory, gparname, __parn, __numstars,thetest, __gunit):
	'''
	plot each param
	'''
	fig = plt.figure('gpar'+gparname)
	__font= {'weight':'bold'}

	for covt in covtests:
		if covt.globuncert[__parn].all() !=-1.:
			__ls = assign_linestyle(covt)
			if gparname != 'ExtM':
				gplot = plt.plot(covt.senyears, covt.globuncert[__parn], label=covt.signature, ls=__ls)
			elif gparname == 'ExtM':
				gplot = plt.semilogy(covt.senyears, covt.globuncert[__parn], label=covt.signature, ls=__ls)
				

	plt.legend(loc='best', bbox_to_anchor=(1.135,1.1), scatterpoints=1, borderpad=0.3, labelspacing=0.1)

	__title = thetest+' -- dashed = BHfixed'
	plt.title(__title, **__font)
	plt.xlabel('time [yr]', **__font)
	plt.ylabel(gparname+' ['+__gunit+']', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	#plt.gca().set_aspect('equal', adjustable='box')
	plt.savefig(plotsdirectory+'/'+__plotname+'-'+gparname+'.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	return



def plot_params_stack(__stackname, __tests_obj, __plotsdir):
	'''

	self.testname, self.nglobal, self.nlocal, self.globalparams, self.globalfits, self.extMparams, self.globparnames 
	following the idea that in each stack the fitted params are the same
	'''
	
	globparnames = __tests_obj[0].globparnames# idea: all in one folder follow the same pattern. all are one test
	globparfits = __tests_obj[0].globalfits
	globparams = __tests_obj[0].globalparams
	globunits = __tests_obj[0].globparunits
	numberstars =  __tests_obj[0].nstars
	__thetest = __tests_obj[0].testname
	__model = __tests_obj[0].model

	print '--- -- Plot the params and SNRs'
	for gpname, gfit, i, gparval, gunit in zip(globparnames, globparfits, range(len(globparnames)), globparams, globunits):#self.sen_global_uncertainties_res[n]
		if gfit:
			plot_single_par(__stackname, __tests_obj, __plotsdir, gpname, i, numberstars, __thetest, gunit)
			plot_single_par_all(__stackname, __tests_obj, __plotsdir, gpname, i, numberstars, __thetest, gunit)
			plot_single_par_SNR(__stackname, __tests_obj, __plotsdir, gpname, i, numberstars, __thetest, gparval)
			plot_single_par_SNR_all(__stackname, __tests_obj, __plotsdir, gpname, i, numberstars, __thetest, gparval)
	#domega:
	if __model== 'keplerAdvance':
		locparnames = __tests_obj[0].localparamnames
		locunits = __tests_obj[0].localunits
		#plot_domega(__stackname, __tests_obj, __plotsdir, locparnames[-1], -1, numberstars, __thetest, locunits[-1])	# we are interested in the last local par which is domega
	return



def plot_param_sen(__covtests_obj, __plotsdirectory):
	'''                                                                                                                                                     
	plot global params.                 
	parameters:
			__covtests_obj: list of the tests as an objects
			plotsdirectory: the plot directory
                                                               
	'''
	print '--- Plotting the global params uncertainties and SNRs and ... in time'

	#nstars used
	plot_nstars_grow(__covtests_obj[0], __plotsdirectory)
	
	#test list
	tests_name_list=[]
	for covt in __covtests_obj:# a file for each tests
		tests_name_list.append(covt.testname)#list of folders AKA categories (string)				
	tests_name_list = np.array(list(set(tests_name_list))) # this is the list of the  files
	testname_tests_objs = [[] for i in range(len(tests_name_list))]
	for covt in __covtests_obj:
		idx, = np.where( tests_name_list == str(covt.testname))
		testname_tests_objs[idx[0]].append(covt)
	
	#simply just each param for each test
	for tname, tests_obj in zip(tests_name_list, testname_tests_objs):
		plot_params_stack(tname, tests_obj, __plotsdirectory)


	return
	
	
#-----------------------------------------------------------------------------------------------------
def plot_period_data( __plotsdirectory):
	'''
	plot period vs npoints or nRVs or astrouncert or RV uncert
	'''

	print '--- plot the statistics'
	infile = './cov_inputs/0x0-0y0-0vx0-0vy0-dw-122-tot.in'
	inf = open(infile, 'r')
	lines = list(line for line in (l.strip() for l in inf) if line) # NO EMPTY LINE
	inf.close()
	nstars= int(lines[14])

	starnames=[]
	periods=[]
	npoints=[]
	nRVs=[]
	starstat=[]
	lastpoints_uncert=[]
	lastRV_uncert=[]
	for i in range(nstars):
		starnames.append(lines[15 + i*11 + 0])
		if starnames[-1][0]=='s':
			starstat.append('sim')
		elif starnames[-1][0]=='S':
			starstat.append('orb')
		periods.append(float(lines[15 + i*11 +4].split()[0]))
		pointsfile = lines[15 + i*11 + 1]
		RVfile = lines[15 + i*11 + 2]
		n=0
		if pointsfile !='None':
			inf = open(pointsfile, 'r')
			plines = list(pline for pline in (pl.strip() for pl in inf) if pline) # NO EMPTY LINE
			inf.close()
			for pline in plines:
				if float(pline.split()[0])>2016.:
					n=n+1
			lastpoints_uncert.append(float(plines[-1].split()[3]))
		else:
			lastpoints_uncert.append(-1.)
		npoints.append(n)

		n=0
		if RVfile !='None':
			inf = open(RVfile, 'r')
			rlines = list(rline for rline in (rl.strip() for rl in inf) if rline) # NO EMPTY LINE
			inf.close()
			for rline in rlines:
				if float(rline.split()[0])>2016.:
					n=n+1
			lastRV_uncert.append(float(rlines[-1].split()[2]))
		else:
			lastRV_uncert.append(-1.)
		nRVs.append(n)

	#npoints ---------------------------------------------------------------------------------------------
	fig = plt.figure('periodpoints')
	__font= {'weight':'bold'}

	for star, stat, per, np in zip(starnames,starstat,periods,npoints):
		if stat=='sim':
			__c='magenta'
		elif stat=='orb':
			__c='blue'
		else:
			print star
		gplot = plt.scatter(per, np, color=__c)
	#plt.legend([mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('magenta')),mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('blue'))],['sim','orb'])
	__title = 'Period vs number of points post 2016.'+' -- magenta=sim, blue=orb'
	plt.title(__title, **__font)
	plt.xscale('log')
	plt.xlabel('Period [yr]', **__font)
	plt.ylabel('number of points', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	plt.savefig(__plotsdirectory+'/period_npoints.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	# nrv ---------------------------------------------------------------------------------------------
	fig = plt.figure('periodRVs')
	__font= {'weight':'bold'}

	for star, stat, per, nrv in zip(starnames,starstat,periods,nRVs):
		if stat=='sim':
			__c='magenta'
		elif stat=='orb':
			__c='blue'
		else:
			print star
		gplot = plt.scatter(per, nrv, color=__c)
	#plt.legend([mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('magenta')),mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('blue'))],['sim','orb'])
	__title = 'Period vs number of RVs post 2016.'+' -- magenta=sim, blue=orb'
	plt.title(__title, **__font)
	plt.xscale('log')
	plt.xlabel('Period [yr]', **__font)
	plt.ylabel('number of RVs', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	plt.savefig(__plotsdirectory+'/period_nRVs.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	#uncert points ---------------------------------------------------------------------------------------------
	fig = plt.figure('perioduncertpoints')
	__font= {'weight':'bold'}

	for star, stat, per, puncert in zip(starnames,starstat,periods,lastpoints_uncert):
		if puncert !=-1.:
			if stat=='sim':
				__c='magenta'
			elif stat=='orb':
				__c='blue'
			else:
				print star
			gplot = plt.scatter(per, puncert*cc.as2mas, color=__c)
	#plt.legend([mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('magenta')),mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('blue'))],['sim','orb'])
	__title = 'Period vs last epoch astro uncert.'+' -- magenta=sim, blue=orb'
	plt.title(__title, **__font)
	plt.xscale('log')
	plt.yscale('log')
	plt.xlabel('Period [yr]', **__font)
	plt.ylabel('astro uncert. [mas]', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	plt.savefig(__plotsdirectory+'/period_astrouncert.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	
	#uncert points ---------------------------------------------------------------------------------------------
	fig = plt.figure('perioduncertpoints')
	__font= {'weight':'bold'}

	for star, stat, per, rvuncert in zip(starnames,starstat,periods,lastRV_uncert):
		if rvuncert !=-1.:
			if stat=='sim' or stat=='ads':
				__c='magenta'
			elif stat=='orb':
				__c='blue'
			else:
				print star
			gplot = plt.scatter(per, rvuncert, color=__c)
	#plt.legend([mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('magenta')),mpatches.Rectangle((0,0),0.5,0.5,fc= set_color('blue'))],['sim','orb'])
	__title = 'Period vs last epoch RV uncert'+' -- magenta=sim, blue=orb'
	plt.title(__title, **__font)
	plt.xscale('log')
	plt.yscale('log')
	plt.xlabel('Period [yr]', **__font)
	plt.ylabel('RV uncert. [km/s]', **__font)
	plt.xticks(**__font)
	plt.yticks(**__font)
	plt.savefig(__plotsdirectory+'/period_RVuncert.png', bbox_inches='tight', dpi=600)
	plt.close(fig)

	return	




#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------------


def plot_ranking_ufrac_uncert(instances, plotsdirectory, stars, sdict, outDir,closefile, rankmode):
	'''
	plot ranking
	'''
	
	def plot_them(file2plot_name, ylab):
		# closest approaches
		inf = open(closefile, 'r')
		lines = list(line for line in (lines.strip() for lines in inf) if line) # NO EMPTY LINE
		closestars=[l.split()[0] for l in lines]
		inf.close()
		
		print '--- plot the %s for each test' %file2plot_name
		celebs=['S0-2', 'S0-38', 'S0-102', 'S0-20', 'S0-16', 'S0-5', 'S0-1' ]
	
		ref_instances = [f for f in instances if ('tot' in f.signature)]
	
		for inst in ref_instances:#the tot  ones. you dont need to read all of them. tot has the name of the ranking files
				
			inf = open(inst.usedstars, 'r')
			lastused = list(line for line in (lines.strip() for lines in inf) if line)[-1][2:] # NO EMPTY LINE
			inf.close()
			neverused =  [s for s in inst.stars_names if s not in lastused]

			#plotting
			for filename in inst.__dict__[file2plot_name]:
				file=np.genfromtxt(filename, dtype='str', comments='#')
	

				fig = plt.figure('perioduncertpoints')
				years=np.arange(inst.begin_sensitivity,inst.end_sensitivity+inst.increment_sensitivity, inst.increment_sensitivity)
	
				nmax_rank=len([line for line in file])
				for i,line in enumerate(file):
					
					if line[0] not in neverused and line[0]!='tot':
						yrs=[years[i] for i,l in enumerate(line[1:].astype(float)) if l!=-1]
						ranks=[l for l in line[1:].astype(float) if l!=-1]
						if len(ranks)>0:
							thelw=1.
							thelabel=str(line[0])
							if line[0] in celebs:
								thelw=2.
							
							if line[0]=='tot':
								thecolor='black'
							elif line[0] in sdict:
								thecolor=sdict[line[0]].pencolor
							else:
								thecolor= get_random_color()
							plt.plot(yrs,ranks,label=thelabel,lw=thelw,color=thecolor)
							plt.annotate(line[0] , xy=(yrs[-1], ranks[-1].astype(float)), xytext=(0.99, 0.10), \
										textcoords='offset points', ha='right', va='bottom', color=thecolor, fontsize=6,\
										#bbox=dict(boxstyle='round,pad=0.5', fc='yellow', alpha=0.5), \
										#arrowprops=dict(arrowstyle = '->', connectionstyle='arc3,rad=0', color='grey')\
							)
			
							#T0
							if line[0] in closestars and len(lines[closestars.index(line[0])])>1:
								for item in np.array(lines[closestars.index(line[0])].split()[1:]).astype(float):
									idx=np.where(item>=yrs)[0]
									if len(idx)>0 and yrs[idx[-1]]!=yrs[-1]:
										plt.scatter(yrs[idx[-1]], ranks[yrs.index(yrs[idx[-1]])], color=thecolor)
							
				
							
				#plt.xlabel('These never enter the analysis: \n'+str(neverused))
				plt.ylabel(ylab)
				plt.xlabel('Time [yr]')
				#plt.xlim([2018.,2019.])
				if 'rank' in file2plot_name:
					plt.ylim([nmax_rank,0.])
				#plt.legend(loc='best', bbox_to_anchor=(1.30,1.0), scatterpoints=1, borderpad=0.3, labelspacing=0.1)
				name=filename.split('/')[-1].split('.')[0]
				plt.title('ranking on %s param, with %s test' %(name.split('_')[0],name.split('_')[1]))
	
				plt.savefig(plotsdirectory+name+'.png', bbox_inches='tight', dpi=600)
	
				if 'rank' in file2plot_name:
					plt.ylim([nmax_rank,0.9999])
				plt.yscale('log')
				plt.savefig(plotsdirectory+name+'_log.png', bbox_inches='tight', dpi=600)
	
				plt.close(fig)


	#### start
	print '--- plot the ranking and the ufrac of the stars for each test'
	set_colors_default_case(stars)
	
	plot_them('ranking_files','ranking')
	if 'drop1' in rankmode[1]:
		plot_them( 'ufrac_files','fractional uncertainty: uncert_(all-1)/uncert_allstars')

	return
	
#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------------
	
		
	
def plot_ranking_varying_params(objects, stars, sdict, outDir, plotsdirectory, here, if_varying_test, varying_items,common_signature):
	'''
	'''
	print '-- plot the uncertainties by varying the parameters'
	common_stars=[o.signature for o in objects if common_signature in o.signature]


	for par in varying_items:


		star_obj_vary=[o for o in objects if o.signature==par+common_signature]#par_common_star_test

		for i,sov in enumerate(star_obj_vary):
			# get years and parcolnum and parcolnames
			parcol = [sov.globparnames.index(g) for g in sov.globparnames if g in sov.testname]
			partestnames = [g for g in sov.globparnames if g in sov.par2testlist]#be careful if you have more tests
			partestidx = [i for i,g in enumerate(sov.globparnames) if g in sov.par2testlist]
			years=np.genfromtxt(sov.senall_address, dtype='str')[:,0].astype(float)
			colors=cm.rainbow(np.linspace(0,1,len(years)))
			
			print par+common_signature, parcol,partestnames, partestidx, sov.name, sov.par2testlist, sov.testname
			if len(sov.par2testlist)>1:
				stop('fix plot_ranking_varying_params')
			
			filename= sov.testname+'_'+sov.prefix+'_'+par+'_uncert.dat'
			res_varying_file = np.genfromtxt(sov.resDir+'ranking/'+filename, comments='#', dtype='str')
			
			if par in ['mo','my']:
				parval=[sdict[s].mag for s in res_varying_file[:,0]]
			else:
				parval=[sdict[s].__dict__[par] for s in res_varying_file[:,0]]


			
			#figure
			fig = plt.figure('perioduncertpoints')
			for j,y in enumerate(years):
				parres=res_varying_file[:,j+1].astype(float)
				
				parval,parres=tosort(parval,parres)
				plt.scatter(parval,parres, label=str(y),s=1, color=colors[j])
				plt.plot(parval,parres, label=str(y),lw=0.6, color=colors[j], alpha=0.4)
				'''plt.annotate(str(y) , xy=(parval[-1], parres[-1].astype(float)), xytext=(0.60, 0.10), \
							textcoords='offset points', ha='right', va='bottom', color=colors[j], fontsize=4,\
							)'''

			plt.xlabel(par+' parameter')
			plt.ylabel(sov.testname+' uncert')
			#plt.ylim([-1,32])
			#plt.legend(loc='best', bbox_to_anchor=(1.30,1.0), scatterpoints=1, borderpad=0.3, labelspacing=0.1)
			suffix=''
			if par=='mo':
				suffix='(old)'
			elif par=='my':
				suffix='(yng)'

			plt.title(filename.split('.')[0]+suffix+' -- from blue to red')
			
			plt.savefig(plotsdirectory+filename.split('.')[0]+'.png', bbox_inches='tight', dpi=600)
			plt.yscale('log')
			plt.savefig(plotsdirectory+filename.split('.')[0]+'_log.png', bbox_inches='tight', dpi=600)
			plt.close(fig)

	
	
	
#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------------
def plot_measurable_rates(instances, stars, plotsdirectory, stardic, testDir, timeslice):
	'''
	like ecc vs semi--major
	'''
	totinstances=[inst for inst in instances if 'tot' in inst.signature]
	celebs=['S0-2', 'S0-38', 'S0-102', 'S0-20', 'S0-16', 'S0-5', 'S0-1' ]
	orbstars=[star for star in stars if (star.status=='orb' or star.status=='gil') ]




	for inst in totinstances:
		years=np.arange(inst.begin_sensitivity,inst.end_sensitivity+inst.increment_sensitivity, inst.increment_sensitivity)
		tidx=np.where(years<=timeslice)[0][-1]+1

		for filename in inst.ranking_files:
			file=np.genfromtxt(filename,dtype='str')
			
			
			si =  [[s,int(file[i,tidx])] for i,s in enumerate(file[:,0]) if (int(file[i,tidx])!=-1 and s!='tot')]
			
			instars = np.array(si)[:,0]
			ranks=np.array(si)[:,1].astype(int)
			ranks,instars= tosort_inverted(ranks,instars)
			colors=cm.rainbow(np.linspace(0,1,len(instars)))			



			fig = plt.figure('perioduncertpoints')
			#plt.scatter
			for s,c in zip(instars,colors):
				plt.scatter(stardic[s].e, stardic[s].P, color=c, label=s)

			#plt.functions
			Periods=[0.01, 0.1, 1., 10.]
			ecc=np.linspace(0.,1.,100)
			GM=4.
			for Period in Periods:
				aAU=(GM* Period**2 * constants.GMSUN_4pi2)**(1.0/3.0)
				omegadot= 6.*GM*constants.GMSUN/constants.CAUYR**2/aAU/(1-ecc**2)*2*pi/Period
				plt.plot(ecc,omegadot, lw=1., color='grey', label='P='+str(Period))

			
			
			
			
			plt.legend(loc='best', bbox_to_anchor=(1.30,1.0), scatterpoints=1, borderpad=0.3, labelspacing=0.1)
			plt.yscale('log')
			plt.ylim([0., 1.e4])
			name=filename.split('/')[-1].split('.')[0]
			plt.savefig(plotsdirectory+name+'_'+str(int(timeslice))+'.png', bbox_inches='tight', dpi=600)
			plt.close(fig)
	
	return
	
	
	
def plot_covarage_and_such(stars, outDir, plotsdirectory, lastobs):
	'''
	'''
	print '--- plotting the percentage coverage and such of each of the orbiting star'

	stars = [s for s in stars if (s.status=='orb')]
	colors=cm.rainbow(np.linspace(0,1,len(stars)))
	for i,star in enumerate(stars):
		star.pencolor=colors[i]
	steps=np.arange(2016.,lastobs,0.2)
	theheader=''
	for t in steps:
		theheader=theheader+'\t'+str(t)
	
	#covarage
	tab=[]
	fig=plt.figure('T0 frequency')#dastro fig
	for star in stars:
		covarage=[(t-1995.)/star.P*100. for t in steps]
		tab.append([star.name]+covarage)
		plt.plot(steps,covarage, label=star.name, lw=1., color=star.pencolor)
	
	plt.title('astro covarage')
	plt.legend(loc='best', bbox_to_anchor=(1.30,1.0), scatterpoints=1, borderpad=0.3, labelspacing=0.1)
	plt.xlabel('times [yr]')
	plt.ylabel('covarage percentage')
	#plt.grid()
	plt.savefig(plotsdirectory+'covarages.png', bbox_inches='tight', dpi=600)
	plt.close(fig)
	np.savetxt(outDir+'covarage.dat',tab, delimiter='\t', fmt='%s', header=theheader)
	

	#2*nastro+1rv = number of measurements
	tab=[]
	fig=plt.figure('T0 frequency')#dastro fig
	for star in stars:
		ntotal=[2*len(np.where(star.astro_epochs_2cov <=t)[0])+len(np.where(star.RV_epochs_2cov <=t)[0]) for t in steps]
		tab.append([star.name]+ntotal)
		plt.plot(steps,ntotal, label=star.name, lw=1., color=star.pencolor)
	
	plt.title('number of measurements 2*astro+rv')
	plt.legend(loc='best', bbox_to_anchor=(1.30,1.0), scatterpoints=1, borderpad=0.3, labelspacing=0.1)
	plt.xlabel('times [yr]')
	plt.ylabel('2*astro + rv')
	#plt.grid()
	plt.savefig(plotsdirectory+'number_measurement.png', bbox_inches='tight', dpi=600)
	plt.close(fig)
	np.savetxt(outDir+'nmeasurement_total.dat',tab, delimiter='\t', fmt='%s', header=theheader)
	
	
	return
