import numpy as np
import os
import glob
import sys
from constants import createDir
from shutil import copy2, copyfile
import subprocess as sb





def set_time_steps(yrstart, yrend, yrstep):
	'''
	make an np array of the time steps AKA number of sensitivity files, steps
	'''
	steps = np.arange(yrstart, yrend, yrstep)
	return steps
#-------------------------------------------------------------------

def create_data(__covtest, __senidx, __datadir, __sen_data_dir, __lasttime):
	'''
	copy over data: points and rv
	'''
	datafile_list=[]
	for star in __covtest.stars_names:
		datafiles = glob.glob( __datadir+star+'.*GEN')
		for datafile in datafiles:
			filename= datafile.split('/')[-1]
		
			df = open(datafile, 'r')
			lines = list(line for line in (l.strip() for l in df) if line) # NO EMPTY LINE
			df.close()
			file_line_temp = np.array([])
			for line in lines:
				if (float(line.split()[0]) <= __lasttime):
					file_line_temp = np.append(file_line_temp, line.rstrip())
		
		
		
			if len(file_line_temp)!=0:
				sen_filename = __sen_data_dir+filename[:-3]+'-'+str(__senidx)
				np.savetxt(sen_filename, np.transpose(file_line_temp), fmt='%s')
				datafile_list.append(sen_filename.split('/')[-2]+'/'+sen_filename.split('/')[-1])

		
	return datafile_list

def create_sen_data(__covt, senidx, __datadir, lasttime, __sensitivity_address):
	'''
	create data dir and data file
	'''
	sen_data_dir = __sensitivity_address+ __datadir
	createDir(sen_data_dir)	
	__datafile_list = create_data( __covt, senidx, './'+__datadir, sen_data_dir, lasttime)
	
	return __datafile_list

#------------------------------------------------------------------------------------------
def create_sen_inputs(__infile, __covt, senidx, __sensitivity_address, __datafiles, __datadir):
	'''
	creates the input
	parameters:
			__infile: the first input file
			__covt: the object
			senidx: idx
	'''
	inf = open(__infile, 'r')
	lines = list(line for line in (l.strip() for l in inf) if line) # NO EMPTY LINE
	inf.close()
	
	sen_foldername= __covt.filename
	sen_filename= __covt.filename+'-'+str(senidx)
	
	# sen input files
	#sen_input_lines = fill_the_in_lines(__covt, senidx, __datafiles)
	sen_input_lines = lines
	sen_input_lines[0] = sen_foldername
	sen_input_lines[1] = sen_filename

	for n in range(__covt.nstars):
		sen_input_lines[7+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ] = 'None'
		sen_input_lines[8+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ] = 'None'
		for dataf in __datafiles:
			if lines[6+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ]+'.points-'+str(senidx) == dataf.split('/')[-1]:
				sen_input_lines[7+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ] =  __datadir +\
													lines[6+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ]+'.points-'+str(senidx)
			if lines[6+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ]+'.rv-'+str(senidx) == dataf.split('/')[-1]:
				sen_input_lines[8+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ] =  __datadir+ \
													lines[6+__covt.npriors +__covt.nglobal +n*(__covt.nlocal+4) ]+'.rv-'+str(senidx)


	inputname = sen_foldername+'.in'+'-'+str(senidx)
	sen_input_addr = __sensitivity_address+ inputname
	np.savetxt(sen_input_addr, np.transpose(sen_input_lines), fmt='%s')
	return inputname


def run_sensitivity(__sec_addr, __mainfile, __senin):
	'''
	runs ./main on the input
	'''
	os.chdir(__sec_addr)
	
	sb.call(['./'+__mainfile, __senin])
	os.chdir('../..')
	return


def initialize(inputfile, covtest, __sen_steps, datadirectory, __base_addr, fortranmain):
	'''
	create inputs, and data
	copy over the make file
	'''
	
	print '--- Sensitivity analysis on the input file:', covtest.name
	sensitivity_address = covtest.foldername+'/'+covtest.filename+'/'
	createDir(sensitivity_address)


	for i in range(len(__sen_steps)):
		datafiles = create_sen_data(covtest, i, datadirectory, __sen_steps[i], sensitivity_address)

		sen_inputname = create_sen_inputs(inputfile, covtest, i, sensitivity_address, datafiles, datadirectory)

		copy2(__base_addr+fortranmain, sensitivity_address+fortranmain)#sso that it also copies over all the permissions
		
		run_sensitivity(sensitivity_address, fortranmain, sen_inputname)
	
	return