#!/usr/bin/python

# starts from data files given in  the input file and create data files cut
# following the times in timeList. (it creates one file per time keeping only
# observations < timeList[i].
# Then it creates an input file for these data and run it.
import numpy as np
import os
import sys
from shutil import copy2, copyfile
import subprocess as sb


def writeInputCov(nf,idx,ci,npar,nl,ns,npriors,folder):
    fout = open(nf+'_'+str(idx),'w')
    fout.write(ci[0])
    fout.write(ci[1].rstrip()+ '_' +str(idx)+'\n')
    fout.write(ci[2])
    fout.write(ci[3])
    fout.write(ci[4])#npriors
    if npriors>0:
        for j in range(npriors):
            fout.write(ci[5+j])

    for j in range(npar+1):
        fout.write(ci[5+npriors+j])

    for j in range(ns):
        fout.write(ci[6+npriors+npar+j*(nl+4)])
        fout.write(folder+'/'+ci[7+npriors+npar+j*(nl+4)].rstrip() + '_' + str(idx) + ' \n')
        fout.write(folder+'/'+ci[8+npriors+npar+j*(nl+4)].rstrip() + '_' + str(idx) + ' \n')
        for k in range(nl+1):
            fout.write(ci[9+npriors+k+npar+j*(nl+4)])


def nparFromMod(model):
	if model=='relativistic':
		nparam = 10
		nlocal=6
	elif model=='kepler':
		nparam = 7
		nlocal=6
	elif model=='keplerRedshift':
		nparam = 8
		nlocal=6
	elif model=='5thforce':
		nparam=9
		nlocal=6
	elif model=='BH'
		nparam=12
		nlocal=6
	return nparam,nlocal

def createDir(n):
    dir = os.path.dirname(n)
    if not os.path.isdir(dir):
        os.makedirs(dir)

def createData(name,time,i,folder):
    fin=open(name+'GEN','r')
    createDir(folder+'/'+name+'_'+str(i))
    fout=open(folder+'/'+name+'_'+str(i),'w')
    
    for l in fin:
        if float(l.split()[0])<=time:
            fout.write(l)

    fin.close()
    fout.close()

def createDataAndInput(inFile,idx,time):
    inf=open(inFile,'r')
    lines=inf.readlines()
    inf.close()
    folder=lines[0].rstrip()
    
    model = lines[3].split()[0].rstrip()
    print 'Model: ',model
    nparam,nlocal=nparFromMod(model)
    npriors = int(lines[4].split()[0])
    nstars = int(lines[nparam+5+npriors].split()[0])
    print nparam,nlocal,nstars,npriors

    nameList  = []
    astroList = []
    RVList    = []
    
    
    # creating the data file
    for i in range(1,nstars+1):
        nameList.append(lines[6+npriors+nparam+(i-1)*(nlocal+4)].rstrip())
        astroList.append(lines[7+npriors+nparam+(i-1)*(nlocal+4)].rstrip())
        RVList.append(lines[8+npriors+nparam+(i-1)*(nlocal+4)].rstrip())
        print '  - Stars ',i
        print '    Name: ',nameList[-1]
        print '    Astro file: ',astroList[-1]
        print '    RV file: ',RVList[-1]
        print ' '
        if astroList[-1]!='None':
            createData(astroList[-1],time,idx,folder)
        if RVList[-1]!='None':
            createData(RVList[-1],time,idx,folder)

    #creating the input file
    print inFile
    writeInputCov(folder+'/'+inFile,idx,lines,nparam,nlocal,nstars,npriors,folder)

    # run the covariance analysis
    sb.call(['./main',folder+'/'+inFile+'_'+str(idx)])



if len (sys.argv)==2:
    inputCovFile=sys.argv[1]
else:
    inputCovFile='input_2stars'
print 'Input file: ', inputCovFile

timeList=np.arange(2016.,2026.,0.5)
timeList=[2016.6, 2017.342,2017.343, 2017.347, 2017.349, 2017.55, 2017.552, 2017.554, 2017.557, 2017.666, 2017.67, 2018.25 ,2018.27,2018.29, 2018.46, 2018.59,2018.657,2018.67,2019]
n=len(timeList)


for i in range(n):
    print 'step ',i
    createDataAndInput(inputCovFile,i,timeList[i])

