!@AREZU : global remark case nohair to be added where needed.
module BHMOD
  use kindtype
  use starsMod, only: star
  use constantsMod, only : pi,solvekepler
	implicit none
	public :: computeObservationsBH, getNumParamBH, getNameParamBH
	private :: createBH,computenext,astro,rv,orbplane,eq_motion
	
	integer(id), private,parameter :: nglob=14 ! 6 BH pos + GM + EM + REDSHIFT + GR + 3 spin + Q 
	integer(id), private,parameter :: nloc=6
	integer(id), private,parameter :: ntot=nloc+nglob	
	integer, private, parameter     :: nparint=14 ! 6 ci + GM +GM ext+ GR + Q + 3 for k + S
	integer, private, parameter    :: neq=6*(nparint+1)

	real(rd), private, parameter   :: torigin=2000._rd
	real(rd), private, parameter   :: tstart=1992._rd
	
	logical,private, save :: nohair=.false.
		
	character(len=5), dimension(nglob), parameter, private :: nameGlob=&
				&(/"GM   ","x0   ","y0   ","vx0  ","vy0  ","vz0  ","R0   ","Ext. ","Red. ","Rel. ","S    ","alpha","delta","Q    "/)
	character(len=5), dimension(nloc), parameter, private :: nameLoc=&
				&(/"P    ","T0   ","e    ","i    ","om   ","Om   "/)
	
	type, private :: orb
		real(rd), private      :: a
		real(rd), private      :: T
		real(rd), private      :: t0
		real(rd), private      :: e
		real(rd), private      :: w
		real(rd), private      :: i
		real(rd), private      :: o
		real(rd), private      :: n	

		real(rd), private      :: Arot
		real(rd), private      :: Brot
		real(rd), private      :: Crot

		real(rd), private      :: Frot
		real(rd), private      :: Grot
		real(rd), private      :: Hrot

		real(rd), private      :: se

		real(rd), private      :: GM
		real(rd), private      :: x0
		real(rd), private      :: y0
		real(rd), private      :: vx0
		real(rd), private      :: vy0
		real(rd), private      :: vz0
		real(rd), private      :: R0
		real(rd), private      :: GRRedshift
		real(rd), private      :: GMExt
		real(rd), private      :: GR
		real(rd), private      :: gammaext
		real(rd), private      :: R0ext
		real(rd), private      :: Bext
		
		real(rd), private      :: Q ! quadrupole moment
		real(rd), private      :: chi ! norm of the spin		
		real(rd), private      :: alpha ! quadrupole moment		
		real(rd), private      :: delta ! quadrupole moment		
		real(rd), private      :: cosa
		real(rd), private      :: sina		     
		real(rd), private      :: cosd
		real(rd), private      :: sind		
		real(rd), private, dimension(3) :: k ! direction of spin
	end type orb
	
	real(rd), private, save :: GM
	real(rd), private, save :: GQ
	real(rd), private, save :: GS		
	real(rd), private, save, dimension(3) :: k	
	real(rd), private, save, dimension(3) :: GSpin	
	real(rd), private, save :: Bext
	real(rd), private, save :: gammaext
	real(rd), private, save :: GR
	real(rd), private, save :: r03g


contains

subroutine setNoHair()
  nohair = .true.
end subroutine setNoHair

subroutine createBH(parG,parL,o,gext,rext)
	use constantsMod, only: GMs_yr2_au3, pi
	real(rd), dimension(nglob), intent(in) :: parG
	real(rd), dimension(nloc), intent(in)  :: parL	
	type(orb), intent(out)                 :: o
	real(rd), intent(in)  :: rext
	real(rd), intent(in)  :: gext	
	!local paramaters
	o%t=parl(1)
	o%t0=parl(2)
	o%e=parl(3)
	o%i=parl(4)*pi/180.0_rd
	o%w=parl(5)*pi/180.0_rd
	o%o=parl(6)*pi/180.0_rd
	
	! global parameters
	o%GM=parg(1)   ! in 10^6 GM_sun 
	o%x0=parg(2)   ! in mas 
	o%y0=parg(3)   ! in mas 
	o%vx0=parg(4)  ! in mas /yr
	o%vy0=parg(5)	 ! in mas / yr
	o%vz0=parg(6)  ! in km/s
	o%R0=parg(7)   ! in kpc 
	o%GMExt=parg(8)   ! in 10^6 GM_sun 
	o%GRRedshift=parg(9)
	o%GR=parg(10)
	
	o%chi=parg(11)! reduced units
	o%alpha=parg(12)*pi/180.0_rd  
	o%delta=parg(13)*pi/180.0_rd 	
	o%Q=parg(14) ! reduced units
		
	o%cosa = cos(o%alpha)
	o%sina = sin(o%alpha)
	o%cosd = cos(o%delta)
	o%sind = sin(o%delta)	
	o%k = (/o%cosa*o%cosd, o%sina*o%cosd, o%sind/)
	
	o%R0ext=rext*648000000._rd/pi ! in AU
	o%gammaext=gext
	o%Bext=o%GMExt/o%R0ext**(3.-o%gammaext)
	
	
	
	o%n=2._rd*pi/o%T
	o%se=sqrt(1._rd-o%e**2)
	o%a=(o%GM*1.e6_rd/o%n**2*GMs_yr2_au3)**(1._rd/3._rd)! in AU

	o%Arot =  cos(o%w)*cos(o%O) - sin(o%w)*sin(o%O)*cos(o%i)!
	o%Brot =  cos(o%w)*sin(o%O) + sin(o%w)*cos(o%O)*cos(o%i)
	
	o%Frot = -sin(o%w)*cos(o%O) - cos(o%w)*sin(o%O)*cos(o%i)
	o%Grot = -sin(o%w)*sin(o%O) + cos(o%w)*cos(o%O)*cos(o%i)

	o%Crot = sin(o%w)*sin(o%i)
	o%Hrot = cos(o%w)*sin(o%i)	
	
end subroutine createBH



subroutine computeObservationsBH(pg,s,gext,rext)
	use constantsMod, only: sigNext	, timenext

	real(rd), dimension(nglob), intent(in) :: pg	
	type(star), intent(inout)  :: s
	real(rd), intent(in)  :: rext
	real(rd), intent(in)  :: gext
	type(orb) :: o
	integer(id) :: i

	real(rd) :: x,y,vx,vy
	real(rd), dimension(ntot) :: dxdp,dydp,dvxdp,dvydp
	
	call createBH(pg,s%par,o,gext,rext)
	if (s%astrob) then
		write(*,'(A)',advance='no') "*"
	  call astro(o,s)
  end if
  if (s%rvb) then
		write(*,'(A)',advance='no') "*"
    call rv(o,s)
  end if
  
  if (signext) then
    call computeNext(o,s)
		if (s%astrob) then
    	s%diffastronext(1)=s%ranext-s%ra(s%na);			s%diffastronext(2)=s%decnext-s%dec(s%na)
		else
			s%diffastronext(1)=0._rd; s%diffastronext(2)=0._rd
		end if
		if (s%rvb) then
    	s%diffRVnext = s%rvnext-s%rv(s%nrv)
		else
			s%diffRVnext = 0._rd
		end if
  end if
end subroutine computeObservationsBH

subroutine computeNext(o,s)
  use odemod,only : integ, setupstart
	use constantsMod, only: GMs_yr2_au3, AUyr2kms,c,timenext,c4,c2,c1
	type(orb), intent(in) :: o
	type(star), intent(inout) :: s
	real(rd)  :: x,y,z,vx,vy,vz,r,v2,SR_redshift,GR_redshift
	real(rd), dimension(ntot)   :: dxdp,dydp,dzdp,dvxdp,dvydp,dvzdp,drdp,dv2dp,dsrdp,dgrdp
	
	real(rd)  :: x0,y0,z0,vx0,vy0,vz0
	real(rd), dimension(ntot)   :: dx0dp,dy0dp,dz0dp,dvx0dp,dvy0dp,dvz0dp
	real(rd), dimension(neq) :: ci, yint
real(rd) :: qunit,sunit
	
	! set constant variables
	GM=o%GM*1.e6_rd*GMs_yr2_au3      ! AU**3/yr**2
	Bext=o%bext*1.e6_rd*GMs_yr2_au3  ! AU**gamma / yr**2
	GR=o%GR
	gammaext=o%gammaext
	r03g=o%r0ext**(3-o%gammaext)     !
  ! @AREZU
	k=o%k
	Qunit = (GM)**3*c4
  GQ = -o%Q*Qunit ! AU**5/yr**2
	sunit= (GM)**2*c2*c1
 	GS = o%chi * sunit !G*spin=[AU^3/yr]
  GSpin=GS*k
  
  !initical condition
	call orbPlane(o,tstart,x0,y0,z0,dx0dp,dy0dp,dz0dp,vx0,vy0,vz0,dvx0dp,dvy0dp,dvz0dp)
  ci=0._rd
  
	ci(1)=x0;ci(2)=y0;ci(3)=z0;ci(4)=vx0;ci(5)=vy0;ci(6)=vz0;
	
	ci(7)=1._rd; ci(11)=1._rd; ci(15)=1._rd;
	ci(16+3*nparint)=1; 	ci(20+3*nparint)=1; 	ci(24+3*nparint)=1; 
	
	
	call setupstart(typeO=4,NE=neq,TSTART=tstart,YSTART=ci,TEND=maxval(s%tastro),ToLp=2.e-13_rd,THRESH1=1.e-20_rd)

  !integration of te equation of motion
	call integ(eq_motion,timenext,yint)!inte

  x  = yint(1)
  y  = yint(2)
  z  = yint(3) 
  vx = yint(4)
  vy = yint(5)
  vz = yint(6)
    
  dxdp= yint(7)*dx0dp + yint(10)*dy0dp + yint(13)*dz0dp + yint(16)*dvx0dp + yint(19)*dvy0dp + yint(22)*dvz0dp
  dydp= yint(8)*dx0dp + yint(11)*dy0dp + yint(14)*dz0dp + yint(17)*dvx0dp + yint(20)*dvy0dp + yint(23)*dvz0dp
  dzdp= yint(9)*dx0dp + yint(12)*dy0dp + yint(15)*dz0dp + yint(18)*dvx0dp + yint(21)*dvy0dp + yint(24)*dvz0dp
  
  dvxdp = yint(7+3*nparint)*dx0dp + yint(10+3*nparint)*dy0dp + yint(13+3*nparint)*dz0dp + yint(16+3*nparint)*dvx0dp &
          & + yint(19+3*nparint)*dvy0dp + yint(22+3*nparint)*dvz0dp
  dvydp = yint(8+3*nparint)*dx0dp + yint(11+3*nparint)*dy0dp + yint(14+3*nparint)*dz0dp + yint(17+3*nparint)*dvx0dp &
          & + yint(20+3*nparint)*dvy0dp + yint(23+3*nparint)*dvz0dp
  dvzdp = yint(9+3*nparint)*dx0dp + yint(12+3*nparint)*dy0dp + yint(15+3*nparint)*dz0dp + yint(18+3*nparint)*dvx0dp &
          & + yint(21+3*nparint)*dvy0dp + yint(24+3*nparint)*dvz0dp
  
  
  !der wrt to GM (parameter 1)  @AREZU  a piece  needs probably to be added
  dxdp(1) = dxdp(1) + yint(25) + 3*GQ/o%GM*yint(46) + 2*GS/o%GM*yint(34)
  dydp(1) = dydp(1) + yint(26) + 3*GQ/o%GM*yint(47) + 2*GS/o%GM*yint(35)
  dzdp(1) = dzdp(1) + yint(27) + 3*GQ/o%GM*yint(48) + 2*GS/o%GM*yint(36)
  
  dvxdp(1) = dvxdp(1) + yint(25+3*nparint) + 3*GQ/o%GM*yint(46+3*nparint) + 2*GS/o%GM*yint(34+3*nparint)
  dvydp(1) = dvydp(1) + yint(26+3*nparint) + 3*GQ/o%GM*yint(47+3*nparint) + 2*GS/o%GM*yint(35+3*nparint)
  dvzdp(1) = dvzdp(1) + yint(27+3*nparint) + 3*GQ/o%GM*yint(48+3*nparint) + 2*GS/o%GM*yint(36+3*nparint)
                                           
  !der wrt to Ext. mass (parameter 8)
  dxdp(8) = dxdp(8) + yint(28)
  dydp(8) = dydp(8) + yint(29)
  dzdp(8) = dydp(8) + yint(30)    
  
  dvxdp(8) = dvxdp(8) + yint(28+3*nparint)
  dvydp(8) = dvydp(8) + yint(29+3*nparint)
  dvzdp(8) = dvzdp(8) + yint(30+3*nparint)
      
  !der wrt to GR (parameter 10)
  dxdp(10) = dxdp(10) + yint(31)
  dydp(10) = dydp(10) + yint(32)
  dzdp(10) = dzdp(10) + yint(33)
  
  dvxdp(10) = dvxdp(10) + yint(31+3*nparint)
  dvydp(10) = dvydp(10) + yint(32+3*nparint)
  dvzdp(10) = dvzdp(10) + yint(33+3*nparint)
    
	!der wrt to S  (parameter 11) @AREZU  a piece  needs probably to be added - coming back to dimensionless unit
	dxdp(11) =  yint(34)*sunit
	dydp(11) =  yint(35)*sunit
	dzdp(11) =  yint(36)*sunit

  dvxdp(11) =  yint(34+3*nparint)*sunit
  dvydp(11) =  yint(35+3*nparint)*sunit
  dvzdp(11) =  yint(36+3*nparint)*sunit

	!der wrt to alpha (par 12)
  dxdp(12) = - yint(37)*k(2) + yint(40)*k(1) 
  dydp(12) = - yint(38)*k(2) + yint(41)*k(1) 
  dzdp(12) = - yint(39)*k(2) + yint(42)*k(1)
  
  dvxdp(12) =  - yint(37+3*nparint)*k(2) + yint(40+3*nparint)*k(1) 
  dvydp(12) =  - yint(38+3*nparint)*k(2) + yint(41+3*nparint)*k(1) 
  dvzdp(12) =  - yint(39+3*nparint)*k(2) + yint(42+3*nparint)*k(1)
  
  !der wrt to delta
  dxdp(13) = - yint(37)*o%sind*o%cosa - yint(40)*o%sind*o%sina + yint(43)*o%cosd
  dydp(13) = - yint(38)*o%sind*o%cosa - yint(41)*o%sind*o%sina + yint(44)*o%cosd
  dzdp(13) = - yint(39)*o%sind*o%cosa - yint(42)*o%sind*o%sina + yint(45)*o%cosd
  
  dvxdp(13) =  - yint(37)*o%sind*o%cosa - yint(40+3*nparint)*o%sind*o%sina + yint(43+3*nparint)*o%cosd
  dvydp(13) =  - yint(38)*o%sind*o%cosa - yint(41+3*nparint)*o%sind*o%sina + yint(44+3*nparint)*o%cosd
  dvzdp(13) =  - yint(39)*o%sind*o%cosa - yint(42+3*nparint)*o%sind*o%sina + yint(45+3*nparint)*o%cosd
  
  !der wrt to Q
 	dxdp(14) =  - yint(46)*qunit
	dydp(14) =  - yint(47)*qunit
	dzdp(14) =  - yint(48)*qunit
	
	dvxdp(14) =  -  yint(46+3*nparint)*qunit
	dvydp(14) =  -  yint(47+3*nparint)*qunit
	dvzdp(14) =  -  yint(48+3*nparint)*qunit
  
  !conversion
	s%ranext = x/o%R0
	s%dradpnext(:)=dxdp/o%R0
	s%dradpnext(7) = s%dradpnext(7) - s%ranext/o%r0

	s%ranext=s%ranext - o%x0 - o%vx0*(timenext-torigin)

	s%dradpnext(2)=s%dradpnext(2)-1._rd
	s%dradpnext(4)=s%dradpnext(4)-(timenext-torigin)


	s%decnext = y/o%r0
	s%ddecdpnext(:) = dydp/o%r0
	s%ddecdpnext(7) = s%ddecdpnext(7) - s%decnext/o%r0

	s%decnext = s%decnext + o%y0 + o%vy0*(timenext-torigin)
	s%ddecdpnext(3)=	s%ddecdpnext(3) + 1._rd
	s%ddecdpnext(5)=	s%ddecdpnext(5) + (timenext-torigin)	


	s%dradpnext(nglob+4:nglob+6)=s%dradpnext(nglob+4:nglob+6)*pi/180._rd
	s%ddecdpnext(nglob+4:nglob+6)=s%ddecdpnext(nglob+4:nglob+6)*pi/180._rd
	s%dradpnext(12:13)=s%dradpnext(12:13)*pi/180._rd
	s%ddecdpnext(12:13)=s%ddecdpnext(12:13)*pi/180._rd
	
  s%rvnext = -vz*AUyr2kms + o%vz0
	s%drvdpnext(:)= -dvzdp*AUyr2kms
	s%drvdpnext(6)=s%drvdpnext(6)+1._rd
  
	! RELATIVITY
	r    = sqrt(x**2+y**2+z**2)
	v2   = vx**2+vy**2+vz**2
	SR_redshift = 0.5_rd*v2/c*AUyr2kms
	GR_redshift = o%GM*1.e6_rd/r/c *GMs_yr2_au3*AUyr2kms
	
	drdp = (x*dxdp + y * dydp+ z * dzdp)/r
	dv2dp = 2._rd*(vx*dvxdp + vy*dvydp + vz*dvzdp)
  
	dSRdp = 0.5_rd*dv2dp/c*AUyr2kms
	dGRdp = -GR_redshift*drdp/r
	dgrdp(1) =	dgrdp(1) + GR_redshift/o%GM ! der wrt to GM
  
	s%rvnext = s%rvnext + o%GRRedshift *(sr_redshift+gr_redshift)
	s%drvdpnext(:) = s%drvdpnext(:) + o%GRRedshift *(dsrdp+dgrdp)
	s%drvdpnext(9) = 	s%drvdpnext(9) + sr_redshift+gr_redshift
	
	s%drvdpnext(nglob+4:nglob+6)=s%drvdpnext(nglob+4:nglob+6)*pi/180._rd
	s%drvdpnext(12:13)=s%drvdpnext(12:13)*pi/180._rd
	if (nohair) then
    	    s%dradpnext(11) =s%dradpnext(11) +2._rd*o%chi*s%dradpnext(14)
    	    s%ddecdpnext(11)=s%ddecdpnext(11)+2._rd*o%chi*s%ddecdpnext(14)
    	    s%drvdpnext(11) =s%drvdpnext(11) +2._rd*o%chi*s%drvdpnext(14)
  end if
end subroutine computeNext


subroutine rv(o,s)
  	use odemod,only : integ, setupstart
	use constantsMod, only: GMs_yr2_au3, AUyr2kms,c,c4,c2,c1
	type(orb), intent(in) :: o
	type(star), intent(inout) :: s
	real(rd)  :: x,y,z,vx,vy,vz,r,v2,SR_redshift,GR_redshift
	real(rd), dimension(ntot)   :: dxdp,dydp,dzdp,dvxdp,dvydp,dvzdp,drdp,dv2dp,dsrdp,dgrdp
	
	real(rd)  :: x0,y0,z0,vx0,vy0,vz0
	real(rd), dimension(ntot)   :: dx0dp,dy0dp,dz0dp,dvx0dp,dvy0dp,dvz0dp
	real(rd), dimension(neq) :: ci, yint
	integer(id) :: i,j
	real(rd) :: qunit,sunit
	
	! set constant variables
	GM=o%GM*1.e6_rd*GMs_yr2_au3     !AU**3/yr**2
	Bext=o%bext*1.e6_rd*GMs_yr2_au3 !AU**gamma/yr**2
	GR=o%GR
	gammaext=o%gammaext
	r03g=o%r0ext**(3-o%gammaext)
	
	k=o%k
	Qunit = (GM)**3*c4
  GQ = -o%Q*Qunit ! AU**5/yr**2
	sunit= (GM)**2*c2*c1
 	GS = o%chi * sunit !G*spin=[AU^3/yr]
 	GSpin=GS*k
	
	!initial condition
	call orbPlane(o,tstart,x0,y0,z0,dx0dp,dy0dp,dz0dp,vx0,vy0,vz0,dvx0dp,dvy0dp,dvz0dp)
  ci=0._rd
	ci(1)=x0;ci(2)=y0;ci(3)=z0;ci(4)=vx0;ci(5)=vy0;ci(6)=vz0;
	ci(7)=1._rd; ci(11)=1._rd; ci(15)=1._rd;
	ci(16+3*nparint)=1; 	ci(20+3*nparint)=1; 	ci(24+3*nparint)=1; 
	
	call setupstart(typeO=4,NE=neq,TSTART=tstart,YSTART=ci,TEND=maxval(s%trv),ToLp=2.e-13_rd,THRESH1=1.e-20_rd)

	s%drvdp=0._rd	      	      
	do i=1,s%nrv
	  ! integration of the equation of motion
	  call integ(eq_motion,s%trv(i),yint)!integration
	  
	  x  = yint(1)
    y  = yint(2)
    z  = yint(3) 
    vx = yint(4)
    vy = yint(5)
    vz = yint(6)
    
    dxdp= yint(7)*dx0dp + yint(10)*dy0dp + yint(13)*dz0dp + yint(16)*dvx0dp + yint(19)*dvy0dp + yint(22)*dvz0dp
    dydp= yint(8)*dx0dp + yint(11)*dy0dp + yint(14)*dz0dp + yint(17)*dvx0dp + yint(20)*dvy0dp + yint(23)*dvz0dp
    dzdp= yint(9)*dx0dp + yint(12)*dy0dp + yint(15)*dz0dp + yint(18)*dvx0dp + yint(21)*dvy0dp + yint(24)*dvz0dp

    dvxdp = yint(7+3*nparint)*dx0dp + yint(10+3*nparint)*dy0dp + yint(13+3*nparint)*dz0dp + yint(16+3*nparint)*dvx0dp &
            & + yint(19+3*nparint)*dvy0dp + yint(22+3*nparint)*dvz0dp
    dvydp = yint(8+3*nparint)*dx0dp + yint(11+3*nparint)*dy0dp + yint(14+3*nparint)*dz0dp + yint(17+3*nparint)*dvx0dp &
            & + yint(20+3*nparint)*dvy0dp + yint(23+3*nparint)*dvz0dp
    dvzdp = yint(9+3*nparint)*dx0dp + yint(12+3*nparint)*dy0dp + yint(15+3*nparint)*dz0dp + yint(18+3*nparint)*dvx0dp &
            & + yint(21+3*nparint)*dvy0dp + yint(24+3*nparint)*dvz0dp


    !der wrt to GM (parameter 1)  @AREZU  a piece  needs probably to be added
    dxdp(1) = dxdp(1) + yint(25) + 3*GQ/o%GM*yint(46) + 2*GS/o%GM*yint(34)
    dydp(1) = dydp(1) + yint(26) + 3*GQ/o%GM*yint(47) + 2*GS/o%GM*yint(35)
    dzdp(1) = dzdp(1) + yint(27) + 3*GQ/o%GM*yint(48) + 2*GS/o%GM*yint(36)

    dvxdp(1) = dvxdp(1) + yint(25+3*nparint) + 3*GQ/o%GM*yint(46+3*nparint) + 2*GS/o%GM*yint(34+3*nparint)
    dvydp(1) = dvydp(1) + yint(26+3*nparint) + 3*GQ/o%GM*yint(47+3*nparint) + 2*GS/o%GM*yint(35+3*nparint)
    dvzdp(1) = dvzdp(1) + yint(27+3*nparint) + 3*GQ/o%GM*yint(48+3*nparint) + 2*GS/o%GM*yint(36+3*nparint)
        
    !der wrt to Ext. mass (parameter 8)
    dxdp(8) = dxdp(8) + yint(28)
    dydp(8) = dydp(8) + yint(29)
    dzdp(8) = dydp(8) + yint(30)    
    
    dvxdp(8) = dvxdp(8) + yint(28+3*nparint)
    dvydp(8) = dvydp(8) + yint(29+3*nparint)
    dvzdp(8) = dvzdp(8) + yint(30+3*nparint)
        
    !der wrt to GR (parameter 10)
    dxdp(10) = dxdp(10) + yint(31)
    dydp(10) = dydp(10) + yint(32)
    dzdp(10) = dzdp(10) + yint(33)

    dvxdp(10) = dvxdp(10) + yint(31+3*nparint)
    dvydp(10) = dvydp(10) + yint(32+3*nparint)
    dvzdp(10) = dvzdp(10) + yint(33+3*nparint)
    
    ! der wrt Sx 
    dxdp(11)  =  yint(37)*sunit
    dydp(11)  =  yint(38)*sunit
    dzdp(11)  =  yint(39)*sunit
    dvxdp(11) =  yint(37+3*nparint)*sunit
    dvydp(11) =  yint(38+3*nparint)*sunit
    dvzdp(11) =  yint(39+3*nparint)*sunit
    
    ! der wrt Sy 
    dxdp(12)  =  yint(40)*sunit
    dydp(12)  =  yint(41)*sunit 
    dzdp(12)  =  yint(42)*sunit    
    dvxdp(12) =  yint(40+3*nparint)*sunit
    dvydp(12) =  yint(41+3*nparint)*sunit
    dvzdp(12) =  yint(42+3*nparint)*sunit
        
    ! der wrt Sz 
    dxdp(13)  =  yint(43)*sunit
    dydp(13)  =  yint(44)*sunit
    dzdp(13)  =  yint(45)*sunit
    dvxdp(13) =  yint(43+3*nparint)*sunit
    dvydp(13) =  yint(44+3*nparint)*sunit
    dvzdp(13) =  yint(45+3*nparint)*sunit
        
!	  !der wrt to S  (parameter 11) 
!  	dxdp(11) = dxdp(11) + yint(34)*sunit
!  	dydp(11) = dydp(11) + yint(35)*sunit
!  	dzdp(11) = dzdp(11) + yint(36)*sunit
!
!    dvxdp(11) = dvxdp(11) + yint(34+3*nparint)*sunit
!    dvydp(11) = dvydp(11) + yint(35+3*nparint)*sunit
!    dvzdp(11) = dvzdp(11) + yint(36+3*nparint)*sunit
!                                              
!  	!der wrt to alpha (par 12)
!    dxdp(12) = - yint(37)*k(2) + yint(40)*k(1) 
!    dydp(12) = - yint(38)*k(2) + yint(41)*k(1) 
!    dzdp(12) = - yint(39)*k(2) + yint(42)*k(1)
!
!    dvxdp(12) =  - yint(37+3*nparint)*k(2) + yint(40+3*nparint)*k(1) 
!    dvydp(12) =  - yint(38+3*nparint)*k(2) + yint(41+3*nparint)*k(1) 
!    dvzdp(12) =  - yint(39+3*nparint)*k(2) + yint(42+3*nparint)*k(1)
!
!    !der wrt to delta
!    dxdp(13) = - yint(37)*o%sind*o%cosa - yint(40)*o%sind*o%sina + yint(43)*o%cosd
!    dydp(13) = - yint(38)*o%sind*o%cosa - yint(41)*o%sind*o%sina + yint(44)*o%cosd
!    dzdp(13) = - yint(39)*o%sind*o%cosa - yint(42)*o%sind*o%sina + yint(45)*o%cosd
!
!    dvxdp(13) =  - yint(37)*o%sind*o%cosa - yint(40+3*nparint)*o%sind*o%sina + yint(43+3*nparint)*o%cosd
!    dvydp(13) =  - yint(38)*o%sind*o%cosa - yint(41+3*nparint)*o%sind*o%sina + yint(44+3*nparint)*o%cosd
!    dvzdp(13) =  - yint(39)*o%sind*o%cosa - yint(42+3*nparint)*o%sind*o%sina + yint(45+3*nparint)*o%cosd
!
!    !der wrt to Q 
!  	dxdp(14) =  - yint(46)*qunit
!  	dydp(14) =  - yint(47)*qunit
!  	dzdp(14) =  - yint(48)*qunit
!
!  	dvxdp(14) =  -  yint(46+3*nparint)*qunit
!  	dvydp(14) =  -  yint(47+3*nparint)*qunit
!  	dvzdp(14) =  -  yint(48+3*nparint)*qunit

  
	  s%rv(i) = -vz*AUyr2kms + o%vz0 ! MINUS sign because fo the reference frame choosen i the integration of the eom

	  s%drvdp(i,:)  = -dvzdp*AUyr2kms
	  s%drvdp(i,6)  =  s%drvdp(i,6)+1._rd !vz

	
	  ! RELATIVITY
	  r    = sqrt(x**2+y**2+z**2)
	  v2   = vx**2+vy**2+vz**2
	  SR_redshift = 0.5_rd*v2/c*AUyr2kms
	  GR_redshift = o%GM*1.e6_rd/r/c *GMs_yr2_au3*AUyr2kms
	
	  drdp = (x*dxdp + y * dydp + z * dzdp)/r
	  dv2dp = 2._rd*(vx*dvxdp + vy*dvydp + vz*dvzdp)

	  dSRdp = 0.5_rd*dv2dp/c*AUyr2kms
	  dGRdp = -GR_redshift*drdp/r
	  dgrdp(1) =	dgrdp(1) + gr_redshift/o%GM ! der wrt to GM

	  s%rv(i) = s%rv(i) + o%GRRedshift *(sr_redshift+gr_redshift)
	  s%drvdp(i,:) = s%drvdp(i,:) + o%GRRedshift *(dsrdp+dgrdp)
	  s%drvdp(i,9) = 	s%drvdp(i,9) + sr_redshift+gr_redshift
	  s%drvdp(i,nglob+4:nglob+6) = s%drvdp(i,nglob+4:nglob+6)*pi/180._rd
	  s%drvdp(i,12:13) = s%drvdp(i,12:13)*pi/180._rd

  end do!i
  if (nohair) then
    s%drvdp(:,11) =s%drvdp(:,11) +2._rd*o%chi*s%drvdp(:,14)
  end if
end subroutine rv

subroutine astro(o,s)
  	use odemod,only : integ, setupstart
  	use constantsmod, only: GMs_yr2_au3,c4,c2,c1
	type(orb), intent(in) :: o
	type(star), intent(inout) :: s

	
	real(rd)  :: x0,y0,z0,vx0,vy0,vz0, x,y,z
	real(rd), dimension(ntot)   :: dx0dp,dy0dp,dz0dp,dvx0dp,dvy0dp,dvz0dp
	real(rd), dimension(ntot)   :: dxdp,dydp
	real(rd), dimension(6*(nparint+1)) :: ci, yint
	integer(id) :: i,j
	real(rd) :: qunit,sunit
	
	! set constant variables
	GM=o%GM*1.e6_rd*GMs_yr2_au3      ! AU**3/yr**2
	Bext=o%bext*1.e6_rd*GMs_yr2_au3  ! AU**gamma/yr**2
	GR=o%GR 
	gammaext=o%gammaext
	r03g=o%r0ext**(3-o%gammaext)
	
	k=o%k
	Qunit = (GM)**3*c4
  GQ = -o%Q*Qunit ! AU**5/yr**2
  sunit= (GM)**2*c2*c1
 	GS = o%chi * sunit !G*spin=[AU^3/yr]
  GSpin=GS*k
  
	!initial condition
	call orbPlane(o,tstart,x0,y0,z0,dx0dp,dy0dp,dz0dp,vx0,vy0,vz0,dvx0dp,dvy0dp,dvz0dp)
  ci=0._rd
	ci(1)=x0;ci(2)=y0;ci(3)=z0;ci(4)=vx0;ci(5)=vy0;ci(6)=vz0;
	ci(7)=1._rd; ci(11)=1._rd; ci(15)=1._rd;
	ci(16+3*nparint)=1; 	ci(20+3*nparint)=1; 	ci(24+3*nparint)=1; 

	call setupstart(typeO=4,NE=neq,TSTART=tstart,YSTART=ci,TEND=maxval(s%tastro),ToLp=2.e-13_rd,THRESH1=1.e-20_rd)

	s%dradp=0._rd
	s%ddecdp=0._rd	      	      
	do i=1,s%na
	  call integ(eq_motion,s%tastro(i),yint)!integration
  
    x=yint(1)
    y=yint(2)
    z=yint(3)
    
    dxdp= yint(7)*dx0dp + yint(10)*dy0dp + yint(13)*dz0dp + yint(16)*dvx0dp + yint(19)*dvy0dp + yint(22)*dvz0dp
    dydp= yint(8)*dx0dp + yint(11)*dy0dp + yint(14)*dz0dp + yint(17)*dvx0dp + yint(20)*dvy0dp + yint(23)*dvz0dp
    
    !der wrt to GM (parameter 1) @AREZU  a piece  needs probably to be added
    dxdp(1) = dxdp(1) + yint(25) + 3*GQ/o%GM*yint(46) + 2._rd*GS/o%GM*yint(34)
    dydp(1) = dydp(1) + yint(26) + 3*GQ/o%GM*yint(47) + 2._rd*GS/o%GM*yint(35) 

    !der wrt to Ext. mass (parameter 8)
    dxdp(8) =  yint(28)
    dydp(8) =  yint(29)
    
    !der wrt to GR (parameter 10)
    dxdp(10) = yint(31)
    dydp(10) = yint(32)

    ! der wrt Sx 
    dxdp(11)  =  yint(37)*sunit
    dydp(11)  =  yint(38)*sunit
    ! der wrt Sy 
    dxdp(12)  =  yint(40)*sunit
    dydp(12)  =  yint(41)*sunit 
    ! der wrt Sz 
    dxdp(13)  =  yint(43)*sunit
    dydp(13)  =  yint(44)*sunit
      
	  !der wrt to S  (parameter 11) 
  	!dxdp(11) =  yint(34)*sunit
  	!dydp(11) =  yint(35)*sunit

  	!der wrt to alpha (par 12)
    !dxdp(12) = - yint(37)*k(2) + yint(40)*k(1) 
    !dydp(12) = - yint(38)*k(2) + yint(41)*k(1) 

    !der wrt to delta
    !dxdp(13) = - yint(37)*o%sind*o%cosa - yint(40)*o%sind*o%sina + yint(43)*o%cosd
    !dydp(13) = - yint(38)*o%sind*o%cosa - yint(41)*o%sind*o%sina + yint(44)*o%cosd

    !der wrt to Q  
  	!dxdp(14) =  - yint(46)*qunit
  	!dydp(14) =  - yint(47)*qunit
	
    !conversion 
  	s%ra(i) = x/o%R0
  	s%dradp(i,:)= dxdp(:)/o%R0
  	s%dradp(i,7) = s%dradp(i,7) - s%ra(i)/o%r0

  	s%ra(i)=s%ra(i) - o%x0 - o%vx0*(s%tastro(i)-torigin)

  	s%dradp(i,2)=s%dradp(i,2)-1._rd
  	s%dradp(i,4)=s%dradp(i,4)-(s%tastro(i)-torigin)


  	s%dec(i) = y/o%r0
  	s%ddecdp(i,:) = dydp/o%r0
  	s%ddecdp(i,7) = s%ddecdp(i,7) - s%dec(i)/o%r0

  	s%dec(i) = s%dec(i) + o%y0 + o%vy0*(s%tastro(i)-torigin)
  	s%ddecdp(i,3)=	s%ddecdp(i,3) + 1._rd
  	s%ddecdp(i,5)=	s%ddecdp(i,5) + (s%tastro(i)-torigin)	


  	s%dradp(i,nglob+4:nglob+6)=s%dradp(i,nglob+4:nglob+6)*pi/180._rd
  	s%ddecdp(i,nglob+4:nglob+6)=s%ddecdp(i,nglob+4:nglob+6)*pi/180._rd
  	s%dradp(i,12:13)=s%dradp(i,12:13)*pi/180._rd
  	s%ddecdp(i,12:13)=s%ddecdp(i,12:13)*pi/180._rd
  end do!i
  if (nohair) then
        s%dradp(:,11)  = s%dradp(:,11) +2._rd*o%chi*s%dradp(:,14)
        s%ddecdp(:,11) = s%ddecdp(:,11) +2._rd*o%chi*s%ddecdp(:,14)
      end if
end subroutine astro

subroutine orbPlane(o,t,x3,y3,z3,dx3dp,dy3dp,dz3dp,vx3,vy3,vz3,dvx3dp,dvy3dp,dvz3dp)
	type(orb), intent(in) :: o
	real(rd), intent(in)  :: t
	real(rd), intent(out) :: x3,y3,z3,vx3,vy3,vz3
	real(rd), intent(out), dimension(ntot) :: dx3dp,dy3dp,dz3dp,dvx3dp,dvy3dp,dvz3dp
	
	real(rd) :: x,y,vx,vy
	real(rd) , dimension(ntot):: dxdp,dydp,dvxdp,dvydp

	real(rd), dimension(ntot) :: dadp, dEdp,dndp,dmdp,dtmpvdp
	real(rd) :: m,E,sE,cE,tmpv
	
	! FIRST: orbital initial conditions in the orbital plane
	dadp(:)=0._rd;	dEdp(:)=0._rd;	dndp(:)=0._rd;

    dndp(nglob+1)=-o%n/o%t
	
	dadp=-2._rd*o%a / (3._rd*o%n) *dndp
	dadp(1)=dadp(1)+ o%a/ o%GM / 3._rd
	
	m=o%n*(t-o%t0)
	

	dmdp=dndp*(t-o%t0)
	dmdp(nglob+2) =	dmdp(nglob+2) - o%n
	call solveKepler(m,o%e,E)
	sE=sin(E);cE=cos(E)
	
		
	dEdp=dmdp
	dEdp(nglob+3) = dEdp(nglob+3) + sE !der wrt to e
	dEdp=dEdp/(1._rd-o%e*cE)

	x = o%a*(ce-o%e) 
	y = o%a*o%se*se

	dxdp = (cE-o%e)*dadp -o%a*se*dedp
	dxdp(nglob+3) = dxdp(nglob+3) - o%a! der wrt e
	
	dydp = y/o%a *dadp + o%a*o%se*cE*dEdp
	dydp(nglob+3) = dydp(nglob+3) - o%a*o%e*sE/o%se
	
	tmpv = o%n*o%a/(1-o%e*ce)
	dtmpvdp = (dndp*o%a + o%n*dadp)/(1._rd-o%e*ce) - o%n*o%a*o%e*sE/(1._rd-o%e*cE)**2*dEdp
	dtmpvdp(nglob+3) = dtmpvdp(nglob+3) + o%n*o%a*cE/(1._rd-o%e*cE)**2   !der wrt e
	
	vx = -tmpv*se
	vy = o%se*tmpv*ce

	dvxdp = -dtmpvdp*se - tmpv*ce*dEdp
	dvydp = dtmpvdp*o%se*ce - tmpv*o%se*se*dEdp
	dvydp(nglob+3)=dvydp(nglob+3) - tmpv*cE*o%e/o%se!der wrt e
	
	
	! second: rotation to get the initial conditions in 3 dimensions
	! The conventions for the reference frame used: x is RA (i.e. left), y is DEC (i.e. up) and z is points towards us (i.e. the NEGATIVE of usual conventions)
    x3 = x * o%Brot + y * o%Grot
  	dx3dp = dxdp * o%Brot + dydp * o%Grot


  	dx3dp(nglob+4) = dx3dp(nglob+4) - x*cos(o%o)*sin(o%i)*sin(o%w) - y*cos(o%o)*sin(o%i)*cos(o%w) ! i
    dx3dp(nglob+5) = dx3dp(nglob+5) + x* (-sin(o%w)*sin(o%o) + cos(o%o)*cos(o%i)*cos(o%w))+ &
                                &	    y* (-sin(o%o)*cos(o%w) - cos(o%o)*cos(o%i)*sin(o%w)) ! omega 
  	dx3dp(nglob+6) = dx3dp(nglob+6)	+ x* (cos(o%w)*cos(o%o) -sin(o%o)*cos(o%i)*sin(o%w)) + &
  															&     y* (-cos(o%o)*sin(o%w)-sin(o%o)*cos(o%i)*cos(o%w))! Omega

    vx3 = vx * o%Brot + vy * o%Grot
    dvx3dp = dvxdp * o%Brot + dvydp * o%Grot


    dvx3dp(nglob+4) = dvx3dp(nglob+4) - vx*cos(o%o)*sin(o%i)*sin(o%w) - vy*cos(o%o)*sin(o%i)*cos(o%w) ! i
    dvx3dp(nglob+5) = dvx3dp(nglob+5) + vx* (-sin(o%w)*sin(o%o) + cos(o%o)*cos(o%i)*cos(o%w))+ &
                            &	    vy* (-sin(o%o)*cos(o%w) - cos(o%o)*cos(o%i)*sin(o%w)) ! omega 
    dvx3dp(nglob+6) = dvx3dp(nglob+6)	+ vx* (cos(o%w)*cos(o%o) -sin(o%o)*cos(o%i)*sin(o%w)) + &
														&     vy* (-cos(o%o)*sin(o%w)-sin(o%o)*cos(o%i)*cos(o%w))! Omega


	y3    = x* o%Arot + y * o%Frot
  	dy3dp = dxdp* o%Arot + dydp * o%Frot
  	dy3dp(nglob+4) = dy3dp(nglob+4) + x*sin(o%o)*sin(o%i)*sin(o%w) + y*sin(o%o)*sin(o%i)*cos(o%w) ! i
  	dy3dp(nglob+5) = dy3dp(nglob+5) + x* (-sin(o%w)*cos(o%o)-sin(o%o)*cos(o%i)*cos(o%w)) + &
  	    											&					y* (-cos(o%o)*cos(o%w)+sin(o%o)*cos(o%i)*sin(o%w)) ! omega
  	dy3dp(nglob+6) = dy3dp(nglob+6) + x* (-cos(o%w)*sin(o%o)-cos(o%o)*cos(o%i)*sin(o%w)) + &
  														& 	      y* (sin(o%o)*sin(o%w)-cos(o%o)*cos(o%i)*cos(o%w))!Omega

    vy3    = vx* o%Arot + vy * o%Frot
  	dvy3dp = dvxdp* o%Arot + dvydp * o%Frot
  	dvy3dp(nglob+4) = dvy3dp(nglob+4) + vx*sin(o%o)*sin(o%i)*sin(o%w) + vy*sin(o%o)*sin(o%i)*cos(o%w) ! i
  	dvy3dp(nglob+5) = dvy3dp(nglob+5) + vx* (-sin(o%w)*cos(o%o)-sin(o%o)*cos(o%i)*cos(o%w)) + &
  	    											&					vy* (-cos(o%o)*cos(o%w)+sin(o%o)*cos(o%i)*sin(o%w)) ! omega
  	dvy3dp(nglob+6) = dvy3dp(nglob+6) + vx* (-cos(o%w)*sin(o%o)-cos(o%o)*cos(o%i)*sin(o%w)) + &
  														& 	      vy* (sin(o%o)*sin(o%w)-cos(o%o)*cos(o%i)*cos(o%w))!Omega
  														
  														

    z3 =-(x * o%Crot + y * o%Hrot )

    dz3dp = (dxdp * o%Crot + dydp * o%Hrot)
    dz3dp(nglob+4)=dz3dp(nglob+4)+cos(o%i)*sin(o%w)*x+cos(o%i)*cos(o%w)*y ! i
    dz3dp(nglob+5)=dz3dp(nglob+5)+sin(o%i)*cos(o%w)*x-sin(o%i)*sin(o%w)*y ! omega
    dz3dp=-dz3dp
    
    vz3 =-( vx * o%Crot + vy * o%Hrot )

    dvz3dp = (dvxdp * o%Crot + dvydp * o%Hrot)
    dvz3dp(nglob+4)=dvz3dp(nglob+4)+cos(o%i)*sin(o%w)*vx+cos(o%i)*cos(o%w)*vy ! i
    dvz3dp(nglob+5)=dvz3dp(nglob+5)+sin(o%i)*cos(o%w)*vx-sin(o%i)*sin(o%w)*vy ! omega
    
    dvz3dp=-dvz3dp
end subroutine orbPlane


subroutine getNumParamBH(ng,nl)
	integer(id), intent(out) :: ng,nl
	ng=nglob
	nl=nloc
end subroutine getNumParamBH

subroutine getNameParamBH(ng,nl)
	character(len=5), dimension(nglob), intent(out) :: ng
	character(len=5), dimension(nloc), intent(out) :: nl
	ng=nameglob
	nl=nameloc
end subroutine getNameParamBH

subroutine eq_motion(t,x,f)
use constantsmod, only:c2,GMs_yr2_au3,c4,vect
  REAL(rd), DIMENSION(neq), INTENT(IN):: X
  REAL(rd), INTENT(IN):: T
  REAL(rd), DIMENSION(neq), INTENT(OUT):: F
  real(rd), dimension(3,3) :: A0,A1
  real(rd), dimension(3) ::rxv,kxv,ei,rxei,kxei,eixv,kxx
  integer :: i,j
  real(rd) :: r,rv,v2, tmp1,kr
  f(1:3)=x(4:6)
  
  r=sqrt(x(1)*x(1)+x(2)*x(2)+x(3)*x(3))
  v2=x(4)*x(4)+x(5)*x(5)+x(6)*x(6)
  rv=x(1)*x(4)+x(2)*x(5)+x(3)*x(6)
  kr=dot_product(k,x(1:3))!AU
	rxv=vect(x(1:3),x(4:6))! AU**2/yr
	
	kxv=vect(k,x(4:6)) ! AU/yr
	kxx=vect(k,x(1:3)) ! AU

  ! ALL UNITS ARE A.U. and year and for GM AU^3/yr^2

  ! Equation of motion
  f(4:6)=-GM*x(1:3)/r**3 & ! Newton
    & - Bext * x(1:3)*r**(-gammaext)   &! Extended mass
    & + GR*c2*GM/r**3*x(1:3)* (4._rd*GM/r-v2) + 4._rd*GR*c2*GM*x(4:6)*rv/r**3 & ! post Newtoian
!    & - GR*GM**2*c4/r**4*x(1:3)*(9._rd*GM/r-2._rd*rv**2/r**2) - GR*2._rd*GM**2*c4*rv*x(4:6)/r**4    &! 2pN monopole
    & + 2._rd*GS/r**5*(3._rd*kR*RxV-r**2*kxV)  !&  ! spin
    !& + 3.0_rd*GQ/r**5*(0.5_rd*(1._rd-5._rd*(kr/r)**2)*x(1:3)+ kr*k)  ! quadrupole

    

! ========= PARTIALS
  f(7:7+3*nparint-1)=x(7+3*nparint:neq)
  ! matrix A0 = \partial acceleraton / \partial parameters and A1 \partial acceleration / \partial velocity
  A0=0._rd;A1=0._rd
!@arezu add contribution from Q and S  
  tmp1=3._rd*GM/r**5 + gammaext*Bext*r**(-2._rd-gammaext) - 3._rd*GR*GM*c2/r**5* (4._rd*GM/r-v2)- GR*4._rd*GM*c2*GM/r**6 !&
!      & +3._rd*GR*GM**2*c4/r**6*(15._rd*GM/r-4._rd*rv**2/r**2) ! 2pn monopole 
  do i=1,3
    ei=0._rd
		ei(i)=1._rd
		rxei=vect(x(1:3),ei)
		kxei=vect(k,ei)
		eixv=vect(ei,x(4:6))
    do j=1,3
	  ! first term = Newton, second and 3rd = post Newtonian
      A0(i,j)=A0(i,j)+tmp1*x(i)*x(j) - 12._rd*GR*c2*GM*x(j)*x(3+i)*rv/r**5 + 4._rd*GR*c2*GM*x(3+i)*x(3+j)/r**3! &
  !        & + 4._rd*GR*GM**2*c4*rv*(x(i)*x(j+3)+2._rd*x(i+3)*x(j))/r**6 - 2._rd*GR*GM**2*c4*x(i+3)*x(j+3)/r**4  &! 2pn monopole
   !       & -7.5_rd*GQ/r**7*(x(i)*x(j)*(1.0_rd-7.0_rd*kr**2/r**2) + 2.0_rd*kr*(k(i)*x(j)+k(j)*x(i))) &  ! quadrupole
  !				& + 3.0_rd*GQ*k(i)*k(j)/r**5 ! quadrupole
  				
  			A0(j,i)=A0(j,i)+6.0_rd*GS/r**5*( k(i)*rxv(j)  -  5._rd*kr*rxv(j)*x(i)/r**2  + kr*eixv(j) + kxv(j)*x(i) ) !spin
  				
  				
      A1(i,j)=A1(i,j)-2._rd*GR*GM*c2/r**3*x(i)*x(j+3) + 4._rd*GR*GM*c2/r**3*x(j)*x(i+3) !&
      !    & + 4._rd*GR*GM**2*c4/r**6*rv*x(i)*x(j) - 2._rd*GR*GM**2*c4/r**4*x(i+3)*x(j) ! 2pn monopole
      A1(j,i)=A1(j,i) +2._rd*GS/r**3*(3.0_rd*kr/r**2*rxei(j) - kxei(j)) !spin 

      if (i==j) then ! here are the diagonal from the matrics A0 and A1
	    ! Newton then extended mass then GR
        a0(i,i)=a0(i,i)-GM/r**3 -Bext*r**(-gammaext) +GR*GM*c2/r**3* (4._rd*GM/r-v2) !&
   !      & -GR*GM**2*c4/r**4**(9._rd*GM/r-2._rd*rv**2/r**2)  &!2PN monopole
  !        &+GQ/r**5*(1.5_rd-7.5_rd*kr**2/r**2)   ! quadrupole
        A1(i,j)=A1(i,j)+4._rd*GR*GM*c2*rv/r**3! &
        !   & - 2._rd*GR*GM**2*c4*rv/r**4 ! 2 pn monopole

      end if
    end do !j
  end do!i
  
  do i=1,nparint  
    f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)= matmul(A0,x(7+3*i-3:7+3*i-1))  + matmul(A1,x(7+3*nparint+3*(i-1):7+3*nparint+3*i-1))

    ! AFTER this, it is the matrix B
    if (i==7) then ! GM
      f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)-x(1:3)*1.e6_rd*GMs_yr2_au3/r**3+ &
                  &  GR*c2*1.e6_rd*GMs_yr2_au3/r**3*x(1:3)* (8._rd*GM/r-v2) + 4._rd*GR*c2*1.e6_rd*GMs_yr2_au3*x(4:6)*rv/r**3 !&
        !      & -GR*c4*GM*1.e6_rd*GMs_yr2_au3/r**4*x(1:3)* (27._rd*GM/r-4._rd*rv**2/r**2) &
         !     & -4._rd*GR*c4*1.e6_rd*GMs_yr2_au3*GM/r**4*rv*x(4:6)
    end if
    
    if (i==8) then ! extended mass
      f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)&
              &-x(1:3)*r**(-gammaext)*1.e6_rd*GMs_yr2_au3/r03g
    end if!i==6
    
    if (i==9) then ! GR
      f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+GM/r**3*c2*(4._rd*GM/r-v2)*x(1:3)&
            & +4._rd*GM/r**3*c2*rv*x(4:6) ! - GM**2*c4/r**4*x(1:3)*(9._rd*GM/r-2._rd*rv**2/r**2) - 2._rd*GM**2*c4*rv*x(4:6)/r**4 
    end if
    if (i==11) then ! Sx
  			eixv=vect((/1._rd,0._rd,0._rd/),x(4:6))
  			f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ 2._rd/r**5*(3._rd*x(1)*RxV-r**2*eixv)!&!
!  				& +3._rd*GQ/r**5*(-5._rd*x(1:3)*kr*x(1)/r**2 + k*x(1)) !quadrupole
!  			f(7+3*nparint+3*(i-1))=f(7+3*nparint+3*(i-1))+3._rd*GQ/r**5*kr !quadrupole
  		end if
  		if (i==12) then ! Sy
  			eixv=vect((/0._rd,1._rd,0._rd/),x(4:6))
  			f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ 2._rd/r**5*(3._rd*x(2)*RxV-r**2*eixv)!&
!  				& +3._rd*GQ/r**5*(-5._rd*x(1:3)*kr*x(2)/r**2 + k*x(2)    )!quadrupole
!  			f(7+3*nparint+3*i-2)=f(7+3*nparint+3*i-2)+3._rd*GQ/r**5*kr!quadrupole
  		end if
  		if (i==13) then ! Sz
  			eixv=vect((/0._rd,0._rd,1._rd/),x(4:6))
  			f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ 2._rd/r**5*(3._rd*x(3)*RxV-r**2*eixv) ! &
!  					& +3._rd*GQ/r**5*(-5._rd*x(1:3)*kr*x(3)/r**2 + k*x(3))!quadrupole
 ! 				f(7+3*nparint+3*i-1)=f(7+3*nparint+3*i-1)+3._rd*GQ/r**5*kr
  		end if
  	       
	!  if (i==10) then ! spin!
	!		f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ 2._rd/r**5*(3._rd*kR*RxV-r**2*kxV)
	!	end if
	!	if (i==11) then ! kx
	!		eixv=vect((/1._rd,0._rd,0._rd/),x(4:6))
	!		f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ 2._rd*GS/r**5*(3._rd*x(1)*RxV-r**2*eixv)&!
	!			& +3._rd*GQ/r**5*(-5._rd*x(1:3)*kr*x(1)/r**2 + k*x(1))
	!		f(7+3*nparint+3*(i-1))=f(7+3*nparint+3*(i-1))+3._rd*GQ/r**5*kr
	!	end if
	!	if (i==12) then ! ky
	!		eixv=vect((/0._rd,1._rd,0._rd/),x(4:6))
	!		f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ 2._rd*GS/r**5*(3._rd*x(2)*RxV-r**2*eixv)&
	!			& +3._rd*GQ/r**5*(-5._rd*x(1:3)*kr*x(2)/r**2 + k*x(2)    )
	!		f(7+3*nparint+3*i-2)=f(7+3*nparint+3*i-2)+3._rd*GQ/r**5*kr
	!	end if
	!	if (i==13) then ! kz
	!		eixv=vect((/0._rd,0._rd,1._rd/),x(4:6))
	!		f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ 2._rd*GS/r**5*(3._rd*x(3)*RxV-r**2*eixv)  &
	!				& +3._rd*GQ/r**5*(-5._rd*x(1:3)*kr*x(3)/r**2 + k*x(3))
	!			f(7+3*nparint+3*i-1)=f(7+3*nparint+3*i-1)+3._rd*GQ/r**5*kr
	!	end if
  !	if (i==14) then ! Quadrupole
  !		f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)=f(7+3*nparint+3*(i-1):7+3*nparint+3*i-1)+ &
	!			& 3.0_rd/r**5*(0.5_rd*(1._rd-5._rd*(kr/r)**2)*x(1:3)+ kr*k)
 	! end if

  end do!i
end subroutine eq_motion
end module BHMod