module cadenceMod
  use kindtype
  use starsMod, only: nstars,stars,star
  implicit none
   

	public   ::  initCadence, initcov,iterCadence
	private :: load_Star_cad, createTemporalSteps, updateCovRV,updateCovAstro,findOptAstro, &
	  & findOptRV,findOptAstro_single,findOptRV_single,writeAllUncertainties,writeinput

	
	integer(id), private, save :: npastobs
	
	integer(id), private, save :: nfuturobsAstro
	integer(id), private, save :: nfuturobsRV
		
	integer(id), private, dimension(:), allocatable :: idxPar

	real(rd), dimension(:), private, allocatable :: futureTimeAstro
	real(rd), dimension(:), private, allocatable :: futureTimeRV	
	real(rd), private, save :: increment
	
	integer(id),private, save     :: Niteration     !< number of time periods
  integer(id),private, save     :: NtintervalsAstro    !< number of time periods
  integer(id),private, save     :: NtintervalsRV    !< number of time periods
  integer(id),private, save     :: gpOptimization !< number of gp on which to optimize

  real(rd), dimension(:,:), allocatable,private      :: tintervalsAstro !< two dim array to keep the beggining and end of a time interval
  real(rd), dimension(:,:), allocatable,private      :: tintervalsRV !< two dim array to keep the beggining and end of a time interval

	real(rd), dimension(:,:), allocatable, private    :: cov !< covariance matrix 
	real(rd), dimension(:,:), allocatable, private    :: pastGPcov !< covariance matrix 

  integer(id), private, save    :: npartofit !<

	integer(id), private, save    :: idxparOpt=-2 !<

	integer(id), parameter, private :: fout=55, ffout=56
	logical, private, allocatable,dimension(:,:) :: timeavailableAstro
	logical, private, allocatable,dimension(:,:) :: timeavailableRV
	
	logical, private, save :: informationEntropy
	real(rd), private,save :: lastPastTime=0._rd

contains
subroutine  initCadence(nameIn)
  use constantsMod, only: dirout, name,signext,timenext,priors,npriors,tofitglobal,nparamglob,nparamloc, nparam,yr2day,nglobtofit
  use modelInterface, only: setModel, computePartials
  implicit none
	character(len=*), intent(in) :: nameIn
	integer :: fittmp,nohairtmp
  integer(id) :: i,j,idxtmp,idxtmp2
  integer(id) :: itmp,istar
  logical(lgt) :: exist
	real(rd) ::J2tmp,stmp
	character(len=15)::plan
	character(len=2):: cn,tmp
	character(len=1):: as
	character(len=15) :: modelStr
	
	!
	!Reading the file
	!
	  inquire(file=nameIn, EXIST=exist) 
	  if (.NOT.exist) then
	      WRITE(*,*) "ERROR : File ", trim(nameIn)," does not exist"
	      STOP
	  endif
	  open(unit=55,file=nameIn,action='read')
	
	 read(55,'(A)') dirout
#ifdef IFORT	 
	inquire(directory=trim(dirout),exist=exist)
#else	
	inquire(file=trim(dirout),exist=exist)
#endif	

	if (.not.exist) then
		write(*,*) 'Creating folder : ', trim(dirout)
		call system(' mkdir '//trim(dirout))
	end if
	!write(*,*) exist,trim(dirout)
	
	!=============================== 2: name of the file	
	read(55,'(A)') name	
	write(*,*) 'File:', name
		
	!=============================== 3: number of itenaration	
	read(55,*) Niteration	
	write(*,*) 'Number of iteration:', Niteration

	!=============================== 4i: number of time intervals	
	read(55,*) NtintervalsAstro
	write(*,*) 'Number of astro time intervals:', NtintervalsAstro

	allocate(tintervalsAstro(2,NtintervalsAstro))
	do i=1, NtintervalsAstro
			read(55, *)  tintervalsAstro(:,i)
			write(*,*) 'Beginning and end of an astro interval:', tintervalsAstro(1,i), '--', tintervalsAstro(2,i)
	end do !i

	read(55,*) Ntintervalsrv
	write(*,*) 'Number of RV time intervals:', NtintervalsRV

	allocate(tintervalsRV(2,NtintervalsRV))
	do i=1, NtintervalsRV
		read(55, *)  tintervalsRV(:,i)
		write(*,*) 'Beginning and end of a RV interval:', tintervalsRV(1,i), '--', tintervalsRV(2,i)
	end do !i

	!=============================== 5: increment	
	read(55, *) increment
	write(*,*) 'Increment:', increment, 'days'
	increment=increment/yr2day ! in year


	call createTemporalSteps(NtintervalsAstro,tintervalsAstro,increment,nfuturobsastro,futureTimeAstro) 
	call createTemporalSteps(NtintervalsRV,tintervalsRV,increment,nfuturobsRV,futureTimeRV)
		
		if (nfuturobsRV+nfuturobsastro<Niteration) then
			write(*,*) 'Error: Number of iterations must be smaller than number of time intervals'
			stop
		end if
		
	!=============================== 6: gpOptimization
	read(55, *) gpOptimization

	
	informationEntropy =( gpOptimization<= 0) !if < 0 -> information entropy 
	if (informationEntropy)then
		write(*,*)'Optimization based on information entropy'
	else
			write(*,*) 'Optimization of a global parameter:', gpOptimization
	end if

	
	read(55,'(A)') modelStr


	read(55,*) npriors
	if (npriors>0) then
		allocate(priors(npriors))
		do i=1,npriors
			read(55,*)priors(i)%idx,priors(i)%sig
		end do !i
	end if!n priors>0
	call setModel(modelstr,55)

	npartofit=nglobtofit
	read(55,*) nstars
	nparam=nparamglob+nstars*nparamloc
	
	if(allocated(idxPar)) deallocate(idxPar)
	allocate(idxPar(nparam))
	
	if (nstars<1) then
		write(*,*) "ERROR : number of stars should be at least 1 and it is: ",nstars
		stop
	end if
	if (npriors>0) then
		do i=1,npriors
			if (priors(i)%idx>nparam) then
				write(*,*) "ERROR: index of prior ", i, " too large: ",priors(i)%idx
				stop
			end if
			if (priors(i)%sig<=0._rd) then
				write(*,*)"ERROR: non positive sigma for prior ", i
				stop
			end if
		end do!i
	end if!npriors

	if (allocated(stars)) deallocate(stars)
	allocate(stars(nstars))
	idxtmp=1
	idxtmp2=1
	do i=1,nparamglob
		if (toFitGlobal(i)) then
			idxPar(idxtmp2)=idxtmp
			idxtmp=idxtmp+1
		else
			idxPar(idxtmp2)=-1
		end if
		idxtmp2=idxtmp2+1
	end do
	npastobs=0
	do i=1, nstars
		allocate(stars(i)%par(nparamloc))
		allocate(stars(i)%tofit(nparamloc))
		
		read(55,*)stars(i)%name
		read(55,'(A)')stars(i)%astroFile
		read(55,'(A)')stars(i)%rvFile
		read(55,*)stars(i)%sigFutRa, stars(i)%sigFutDec, stars(i)%sigFutRV
		do j=1,nparamloc
					read(55,*) stars(i)%par(j),fittmp
					stars(i)%tofit(j)=(fittmp==1)
					npartofit=npartofit+fittmp
					if (stars(i)%tofit(j)) then
						idxPar(idxtmp2)=idxtmp
						idxtmp=idxtmp+1
					else
						idxPar(idxtmp2)=-1
					end if
					idxtmp2=idxtmp2+1
		end do!j
		call load_Star_cad(stars(i),i)
    npastobs=npastobs+2*stars(i)%npastastro+stars(i)%npastrv
	end do! i	
	close(55)

  call writeInput()
  call computepartials()
end subroutine  initCadence


subroutine createTemporalSteps(nint,tint,dt,nfut,futTime) 			! create temporal steps using Ntintervals, tintervals(:,:) and the increment
	integer(id), intent(in) :: nint
	real(rd), dimension(:,:), intent(in) :: tint
	real(rd), intent(in) :: dt
	integer(id), intent(out) :: nfut
	real(Rd), intent(out), dimension(:),allocatable ::futTime
  integer(id), dimension(:), allocatable      :: nSteps 

	integer(id) :: i, j, k !k is a number keeper
	
	allocate(nSteps(nint))

	nfut = 2* Nint

	do i=1, nint
		nSteps(i) = floor((tint(2,i)-tint(1,i))/dt)
		nfut = nfut + nSteps(i)
		write(*,*) 'Number of steps in time interval',i, ':', nSteps(i)
	end do !i
	

	write(*,*) 'Minimum number of steps:', MINVAL(nSteps)
	write(*,*) 'Total number of steps', nfut
	

	if (allocated(futTime)) deallocate(futTime)
	allocate(futTime(nfut))
	
	futTime(1)= tint(1,1) !beginning
	k=1
	do i=1, Nint
		do j=1, nSteps(i)
			futTime(k+j) = futTime(k+j-1) + dt !middle time steps
		end do !j
	k=nSteps(i) + 1 + k										!keep a number as a counter'k' 
	futTime(k) = tint(2,i) !end
	k = k+1


	if (i<Nint) then
		!write(*,*) nint,i,nfut
		futtime(k) = tint(1,i+1) !beggining of the next round
	end if
	end do !i
		!write(*,*)futtime
end subroutine createTemporalSteps



subroutine load_Star_cad(s,idx)
	use constantsMod, only: nparamglob,nparamloc,fromJY2MJD
	use starsmod, only:loadstar
	type(star), intent(inout) :: s
	integer(id), intent(in) :: idx
	real(rd),dimension(2) :: tmp
	real(rd) :: tmpyr
	integer(id):: nmean
	integer(id) ::i

	call loadstar(s,nfuturobsAstro,nfuturobsRV)
	s%astrob=.True.
	
	! ASTROMETRIC OBSERVATIONS		
		write(*,*) "   Number of past astrometric observations: ",s%npastastro
		write(*,*) "   Number of past + future astrometric observations: ",s%na

		if (trim(s%astroFile)=="None") then
			! NO PAST OBSERVATIONS
			if ((s%sigFutRa<0.).or.(s%sigFutDec<0.)) then
				write(*,*) 'Error in input for stars ',idx,' : one of the sig for astro is <0 while no astro file is furnished'
				stop
			end if! error
			! FUTURE OBSERVATIONS
			s%sigra=s%sigFutRa;s%sigdec=s%sigFutdec;
			s%tastro=futureTimeastro
		else
		  if (maxval(s%tastro(1:s%npastastro))>lastPastTime) lastPastTime=maxval(s%tastro(1:s%npastastro))
			! FUTURE OBSERVATIONS
			s%tastro(s%npastastro+1:)=futureTimeastro
			
			if (s%sigFutRa<0.) then
					nmean=min(5,s%npastastro)-1
					s%sigra(s%npastastro+1:)=sum(s%sigra(s%npastastro-nmean:s%npastastro))/(1._rd+nmean)
			else
				s%sigra(s%npastastro+1:)=s%sigFutRa
			end if
			if (s%sigFutDec<0.) then
					nmean=min(5,s%npastastro)-1
					s%sigdec(s%npastastro+1:)=sum(s%sigdec(s%npastastro-nmean:s%npastastro))/(1._rd+nmean)
			else
				s%sigdec(s%npastastro+1:)=s%sigFutdec
			end if		
	end if!astro
	
	! RV OBSERVATIONS
	s%rvb=.True.
	
	write(*,*) "   Number of past RV observations: ",s%npastrv
	write(*,*) "   Number of past + future RV observations: ",s%nrv
	if (trim(s%rvFile)=="None") then
		! NO PAST OBSERVATIONS
		if (s%sigFutRv<0.) then
			write(*,*) 'Error in input for stars ',idx,' : the sig for RV is <0 while no RV file is furnished'
			stop
		end if! error
		! FUTURE OBSERVATIONS
		s%sigrv=s%sigFutRv
		s%trv=futureTimeRV
	else
		! FUTURE OBSERVATIONS
		s%trv(s%npastrv+1:)=futureTimerv
		if (s%sigFutRV<0.) then
				nmean=min(5,s%npastrv)-1
				s%sigrv(s%npastrv+1:)=sum(s%sigrv(s%npastrv-nmean:s%npastrv))/(1._rd+nmean)
		else
			s%sigrv(s%npastrv+1:)=s%sigFutRv
		end if
		write(*,*) "     -- RV data loaded --"		
	end if! RV
end subroutine load_Star_cad


subroutine iterCadence()
	use constantsMod, only: nparamglob,nparamloc,tofitglobal,fromJY2MJD,dirout,name
	use modelinterface, only: nameglob,nameloc
	integer:: istar,k, j
	real(rd)    :: dsigmaxastro,dsigmaxrv
	integer(id) :: idxtimeastro,idxtimerv ,idxstarastro,idxstarrv, i,idxparloc
	logical :: exist
	write(*,*)" Start of the cadence iterations"
	write(*,'(A)',advance='no')"  Parameters to optimize: "

 	if (informationentropy) then
		write(*,*) ' information entropy'
		idxparopt=-1
	else
		if (gpOptimization <= nparamglob) then
			write(*,*) nameglob(gpOptimization)
		else
			istar=(gpOptimization-1-nparamglob)/nparamloc+1
			write(*,*) nameloc(gpOptimization-nparamglob-(istar-1)*nparamloc), ' from star ',trim(stars(istar)%name)
		end if
	idxparopt=idxpar(gpOptimization)
	end if
	
	if (allocated(timeavailableastro)) deallocate(timeavailableastro)
	allocate(timeavailableastro(nstars,nfuturobsastro))
	timeavailableastro = .true.
	
	if (allocated(timeavailablerv)) deallocate(timeavailablerv)
	allocate(timeavailablerv(nstars,nfuturobsrv))
	timeavailablerv = .true.

	write(fout,*)  "    "
	write(fout,*)  "    "
	write(fout,*)  "  ====================================================  "
	write(fout,*)  "         RESULTS  FROM THE PAST DATA  "
	write(fout,*)  "  ====================================================  "
	write(fout,*)" Resulting uncertainties: "
	write(fout,*)"  == Global parameters: =="

	do i=1, nparamglob
		if (tofitglobal(i)) then
			write(fout,'(2x,A5,2x,G10.5)') nameGlob(i), pastGPcov(idxpar(i),idxpar(i))
		else
				write(fout,'(2x,A5,2x,A)' )  nameglob(i), '-NF-'
		end if
	end do !i



	!==Output === PAST GP
!	inquire(file=trim(dirout)//'/'//trim(name)//'out',exist=exist)
	open(unit=ffout,file = trim(dirout)//'/'//trim(name)//'.dat',action='write',status='replace')


!	write(ffout,*)  "   "
!	write(ffout,*)  "   "
!	write(ffout,*)  "  ====================================================  "
!	write(ffout,*)  "         GLOBAL PARAMETERS PAST and FUTURE"
!	write(ffout,*)  "  ====================================================  "
!	write(ffout,'(2x,A)',advance='no') '  i    '
	write(ffout,'(2x,A,5x)',advance='no') 'iter'
	write(ffout,'(2x,A,5x)',advance='no') 'Star'
	write(ffout,'(2x,A,5x)',advance='no') 'Type'
	write(ffout,'(2x,A,7x)',advance='no') 'Time'
	write(ffout,'(2x,A,8x)',advance='no') 'MJD '	
	do i=1, nparamglob
		write(ffout, '(2x,A5,5x)', advance='no') nameGlob(i)
	end do !i
	write(ffout,*)

	write(ffout,'(2x,i3,7x)', advance='no' )  0
	write(ffout,'(1x,A,2x)', advance='no' )  'None'
	write(ffout,'(5x,A,4x)', advance='no' )  'None'
	write(ffout, '(G13.8,1x)', advance='no')  lastPastTIme
	write(ffout, '(G13.8,1x)', advance='no')  fromJY2MJD(lastPastTIme)
	call writeAllUncertainties(ffout)

	write(ffout,*)

  if (.not.informationentropy) 	 	write(*,*) "  Initial uncertainty on this parameter: ",sqrt(cov(idxparopt,idxparopt))


	write(fout,*)  "   "
	write(fout,*)  "   "
	write(fout,*)  "  ====================================================  "
	write(fout,*)  "         CADENCE TOOL RESULTS  "
	write(fout,*)  "  ====================================================  "

	write(fout, '(A,8x)', advance='no') 'iteration'
	write(fout, '(A,8x)', advance='no') 'Star'
	write(fout, '(A,8x)', advance='no') 'Optimal observation'
	write(fout, '(A,8x)', advance='no') 'YJD'
	write(fout, '(A,8x)', advance='no') 'MJD'
	write(fout, '(A,8x)', advance='no') 'DEC uncert.'
	write(fout, '(A,8x)', advance='no') 'ra uncert.'
	write(fout, '(A,8x)', advance='no') 'rv uncert.'
 
	if (.not.informationentropy)	then
		if (gpOptimization <= nparamGlob)then
			write(fout, '(A,8x)', advance='no') nameGlob(gpOptimization)
		else
			istar=(gpOptimization-1-nparamglob)/nparamloc+1
			write(fout, '(A,A,A8x)', advance='no') trim(nameloc(gpOptimization-nparamglob-(istar-1)*nparamloc)),' ',trim(stars(istar)%name)
		end if
	end if

		write(fout, *)
	

	
	do i=1,niteration
		write(*,*) "   -- Iteration ",i
		write(fout, '(G10.5,4x)', advance='no') i
		write(ffout,'(2x,i3,7x)', advance='no' )  i

		call findOptAstro(idxstarastro,idxtimeastro,dsigmaxastro)
		call findOptRV(idxstarrv,idxtimerv,dsigmaxrv)
		
		if (dsigmaxrv>dsigmaxastro) then !if RV improves better the uncertainty on the "desired" parameter
			write(*,*) '  Optimal observation is RV from star ',trim(stars(idxstarrv)%name),&
				&' at time ',futuretimerv(idxtimerv),&
				&' or ',fromJY2MJD(futuretimerv(idxtimerv)),' MJD with uncertainty ',&
				& stars(idxstarrv)%sigrv(stars(idxstarrv)%npastrv+idxtimerv)
				

			call updateCovRV(idxstarrv,idxtimerv)
			timeavailablerv(idxstarrv,idxtimerv)=.false.
!			timeavailableastro(:,idxtimerv)=.false.
!				do k=1,npartofit
!						write(*,'(2x,G10.5)')sqrt(cov(k ,k))
!				end do
		write(fout, '(A)', advance='no') trim(stars(idxstarrv)%name)
		write(fout, '(A,4x)', advance='no') '       rv'
		write(fout, '(G20.15,4x)', advance='no')  futuretimerv(idxtimerv)
		write(fout, '(G20.15,4x)', advance='no')  fromJY2MJD(futuretimerv(idxtimerv))
		write(fout, '(G20.15,4x)', advance='no') 0. !empty
		write(fout, '(G20.15,4x)', advance='no') 0.  !empty
		write(fout, '(G20.15,4x)', advance='no')  stars(idxstarrv)%sigrv(stars(idxstarrv)%npastrv+idxtimerv)
		
		write(ffout, '(1x,A,2x)', advance='no') trim(stars(idxstarrv)%name)
		write(ffout, '(A,4x)', advance='no') '       rv'
		write(ffout, '(G13.8,1x)', advance='no')   futuretimerv(idxtimerv)
		write(ffout, '(G13.8,1x)', advance='no')  fromJY2MJD(futuretimerv(idxtimerv))
		else !if ASTRO improves better the uncertainty on the "desired" parameter
			write(*,*) '  Optimal observation is ASTRO from star ',trim(stars(idxstarastro)%name),' at time ',futuretimeastro(idxtimeastro),&
				&' or ',fromJY2MJD(futuretimeastro(idxtimeastro)),' MJD with uncertainty ',&
				& stars(idxstarastro)%sigra(stars(idxstarastro)%npastastro+idxtimeastro), ' for RA and ', &
				& stars(idxstarastro)%sigdec(stars(idxstarastro)%npastastro+idxtimeastro),' for DEC'


				
			call updateCovAstro(idxstarastro,idxtimeastro)
			timeavailableastro(idxstarastro,idxtimeastro)=.false.
!			timeavailablerv(:,idxtimeastro)=.false.
!					do k=1,npartofit
!							write(*,'(2x,G10.5)')sqrt(cov(k ,k))
!					end do
		write(fout, '(A)', advance='no') trim(stars(idxstarastro)%name)
		write(fout, '(A,4x)', advance='no') '    astro'
		write(fout, '(G20.15,4x)', advance='no')  futuretimeastro(idxtimeastro)
		write(fout, '(G20.15,4x)', advance='no')  fromJY2MJD(futuretimeastro(idxtimeastro))
		write(fout, '(G20.15,4x)', advance='no')  stars(idxstarastro)%sigdec(stars(idxstarastro)%npastastro+idxtimeastro)
		write(fout, '(G20.15,4x)', advance='no')  stars(idxstarastro)%sigra(stars(idxstarastro)%npastastro+idxtimeastro)
		write(fout, '(G20.15,4x)', advance='no') 0. !empty
		
		write(ffout, '(1x,A,2x)', advance='no') trim(stars(idxstarastro)%name)
		write(ffout, '(A,4x)', advance='no') '    astro'
		write(ffout, '(G13.8,1x)', advance='no')   futuretimeastro(idxtimeastro)
		write(ffout, '(G13.8,1x)', advance='no') fromJY2MJD(futuretimeastro(idxtimeastro))
		end if
		if(.not.informationentropy) write(*,*) "  New uncertainty on the parameter: ",sqrt(cov(idxparopt,idxparopt))
		write(*,*)
		if(.not.informationentropy)write(fout,'(G20.15,4x)', advance='no') sqrt(cov(idxparopt,idxparopt))
		write(fout,*)

!====== GLOBAL PARAMETER AT EACH ITERATION
	call writeAllUncertainties(ffout)
	write(ffout,*)''
end do
	
close(fout)
close(ffout)



end subroutine iterCadence


subroutine updateCovASTRO(idxstar,idxtime)
	use constantsMod, only: nparamglob,nparamloc,tofitglobal,invMat_addI
	integer(id), intent(in) :: idxstar
	integer(id), intent(in) :: idxtime	

	real(rd),dimension(:,:), allocatable :: pder
	real(rd), dimension(:,:), allocatable :: PA
	integer(id) :: j
	real(rd), dimension(2,2) :: invcpc
	allocate(pder(npartofit,2),PA(npartofit,2))
	pder=0._rd;PA=0._rd;


		do j=1,nparamGlob
			if (tofitglobal(j)) then
				pder(idxPar(j),1)=stars(idxStar)%dradp(stars(idxStar)%npastastro+idxtime,j) / &
				          &  stars(idxStar)%sigra(stars(idxStar)%npastastro+idxtime)
				pder(idxPar(j),2)=stars(idxStar)%ddecdp(stars(idxStar)%npastastro+idxtime,j) / &
				          &  stars(idxStar)%sigdec(stars(idxStar)%npastastro+idxtime)
			end if
		end do!j	

		do j=1,nparamLoc
			if (stars(idxStar)%tofit(j)) then
				pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),1) =&
					&  stars(idxStar)%dradp(stars(idxStar)%npastastro+idxtime,nparamglob+j) /  &
					& stars(idxStar)%sigra(stars(idxStar)%npastastro+idxtime)
				pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),2) =&
					&  stars(idxStar)%ddecdp(stars(idxStar)%npastastro+idxtime,nparamglob+j) / &
					& stars(idxStar)%sigdec(stars(idxStar)%npastastro+idxtime)						
			end if
		end do!j
		
		PA=matmul(cov,pder)
		call invMat_addI(matmul(transpose(pder),PA),invCPC)
		cov=cov-matmul(matmul(pa,invCPC),transpose(PA))
	deallocate(pder,PA)	
end subroutine updateCovAstro

subroutine updateCovRV(idxstar,idxtime)
	use constantsMod, only: nparamglob,nparamloc,tofitglobal!,invMat_addI
	integer(id), intent(in) :: idxstar
	integer(id), intent(in) :: idxtime	

	real(rd),dimension(:,:), allocatable :: pder
	real(rd), dimension(:,:), allocatable :: PA
	integer(id) :: j
	allocate(pder(npartofit,1),PA(npartofit,1))
	pder=0._rd;PA=0._rd;
	
	do j=1,nparamGlob
		if (tofitglobal(j)) then
				pder(idxPar(j),1)=stars(idxStar)%drvdp(stars(idxStar)%npastrv+idxtime,j) / &
				  & stars(idxStar)%sigrv(stars(idxStar)%npastrv+idxtime)
		end if
	end do!j	

	do j=1,nparamLoc
		if (stars(idxStar)%tofit(j)) then
			pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),1) =&
				&  stars(idxStar)%drvdp(stars(idxStar)%npastrv+idxtime,nparamglob+j)/stars(idxStar)%sigrv(stars(idxStar)%npastrv+idxtime)
			end if
	end do!j
		
	PA=matmul(cov,pder)
	cov=cov-matmul(PA,transpose(PA))/(1._rd+dot_product(pder(:,1),PA(:,1)))
   	deallocate(pder,PA)
end subroutine updateCovRV


subroutine findOptAstro(idxStarOpt,idxTimeOpt,dsigOpt)
	integer(id), intent(out) :: idxstaropt
	integer(id), intent(out) :: idxTimeOpt
	real(rd), intent(out) :: dsigopt
	
	real(rd) :: dsigstar
	integer(id) :: i,idxopt_s
	
	idxStarOpt=0
	idxTimeOpt=0
	dsigOpt=-huge(1._rd)
	do i=1,nstars
		call findOptAstro_single(i,idxopt_s,dsigstar)
		if (dsigstar>dsigOpt) then
			dsigopt=dsigstar
			idxTimeOpt=idxopt_s
			idxStarOpt=i
		end if
	end do !i
!	write(*,*) 'BEST ASTRO ', idxStaropt,idxtimeopt, sqrt(cov(idxparopt,idxparopt)- dsigopt)
	
end subroutine findOptAstro


subroutine findOptAstro_single(idxStar,idxmax,dsigmax)
	use constantsMod, only: nparamglob,nparamloc,tofitglobal,npriors,invMat_addI
	integer(id), intent(in) :: idxStar
	integer(id), intent(out) :: idxmax
	real(rd), intent(out)    :: dsigmax
	
	integer(id) :: i,j,k
	real(rd),dimension(:,:), allocatable :: pder
	real(rd), dimension(:,:), allocatable :: PA
	real(rd) :: dsigi
	real(rd), dimension(2,2):: invCPC
	real(rd) :: siglocRA
	real(rd) :: siglocDEC
	
	allocate(pder(npartofit,2),PA(npartofit,2))
	pder=0._rd;PA=0._rd;
	idxMax=0
	dsigmax=-huge(1._rd)
	dsigi=0.

	do i=1,nfuturobsastro
		if (timeavailableastro(idxstar,i)) then
			siglocRA = stars(idxStar)%sigra(stars(idxStar)%npastastro+i)
			siglocDEC = stars(idxStar)%sigdec(stars(idxStar)%npastastro+i)
			do j=1,nparamGlob
				if (tofitglobal(j)) then
					pder(idxPar(j),1)=stars(idxStar)%dradp(stars(idxStar)%npastastro+i,j)  
					pder(idxPar(j),2)=stars(idxStar)%ddecdp(stars(idxStar)%npastastro+i,j)  
				end if
			end do!j	

			do j=1,nparamLoc
				if (stars(idxStar)%tofit(j)) then
					pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),1) = stars(idxStar)%dradp(stars(idxStar)%npastastro+i,nparamglob+j)
						pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),2) =&
							&  stars(idxStar)%ddecdp(stars(idxStar)%npastastro+i,nparamglob+j)				
					end if
			end do!j

		pder(:,1)=pder(:,1)/siglocRA
		pder(:,2)=pder(:,2)/siglocDEC		
		
		if (informationEntropy) then
			! to be done
			dsigi=0.5_rd*(log(dot_product(pder(:,1),matmul(cov,pder(:,1)))) + log(dot_product(pder(:,2),matmul(cov,pder(:,2))))) !to change
		else
			PA=matmul(cov,pder)
			
			! see page 4 of http://www.cs.tut.fi/~tabus/course/ASP/LectureNew10.pdf
			call invMat_addI(matmul(transpose(pder),PA),invCPC)
			dsigi=dot_product(matmul(PA(idxparopt,:),invCPC),PA(idxparopt,:))

		end if! information entropy

		if (dsigi>dsigmax) then
			idxMax  = i
			dsigmax = dsigi
		end if
!		write(*,*) 'astro', i, idxmax, dsigmax, dsigi
		end if! time available
	end do!i 

	deallocate(pder,PA)
end subroutine findOptAstro_single



subroutine findOptRV(idxStarOpt,idxTimeOpt,dsigOpt)
	integer(id), intent(out) :: idxstaropt
	integer(id), intent(out) :: idxTimeOpt
	real(rd), intent(out) :: dsigopt

	real(rd) :: dsigstar
	integer(id) :: i,idxopt_s
	
	idxStarOpt=0
	idxTimeOpt=0
	dsigOpt=-huge(1._rd)
	do i=1,nstars
		call findOptRV_single(i,idxopt_s,dsigstar)
		if (dsigstar>dsigOpt) then
			dsigopt=dsigstar
			idxTimeOpt=idxopt_s
			idxStarOpt=i
		end if
	end do !i
!	write(*,*) 'BEST RV    ', idxStaropt,idxtimeopt, sqrt(cov(idxparopt,idxparopt)-dsigopt)
	
end subroutine findOptRV

subroutine findOptRV_single(idxStar,idxmax,dsigmax)
	use constantsMod, only: nparamglob,nparamloc,tofitglobal,fromJY2MJD
	integer(id), intent(in) :: idxStar
	integer(id), intent(out) :: idxmax
	real(rd), intent(out)    :: dsigmax
	
	integer(id) :: i,j,k
	real(rd),dimension(:), allocatable :: pder
	real(rd), dimension(:), allocatable :: PA
	real(rd) :: sigLoc
	real(rd) :: dsigi
	
	allocate(pder(npartofit),PA(npartofit))
	pder=0._rd;PA=0._rd;
	idxMax=0
	dsigmax=-huge(1._rd)
	dsigi=0.

	
	do i=1,nfuturobsrv
		if (timeavailablerv(idxstar,i)) then
			sigloc = stars(idxStar)%sigrv(stars(idxStar)%npastrv+i)
			do j=1,nparamGlob
				if (tofitglobal(j)) then
					pder(idxPar(j))   = stars(idxStar)%drvdp(stars(idxStar)%npastrv+i,j) 
				end if
			end do!j	

			do j=1,nparamLoc
				if (stars(idxStar)%tofit(j)) then
					pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j)) =&
						&  stars(idxStar)%drvdp(stars(idxStar)%npastrv+i,nparamglob+j)
					end if
			end do!j
			pder=pder/sigloc
			if (informationEntropy) then
!				write(*,*) (futuretime(i)), sigloc,sqrt(dot_product(pder,matmul(cov,pder))*sigloc**2)
!				stop
				dsigi=0.5_rd*log(dot_product(pder,matmul(cov,pder))) !information entropy log(uncertainy for next observation / uncertainty that we put for next observations)  (the denominator is stars(idxStar)%sigrv(npastrv(idxStar)+i) )
			else
			
				PA=matmul(cov,pder)
				! see page 6-22 from https://see.stanford.edu/materials/lsoeldsee263/06-ls-app.pdf
				dsigi=PA(idxparopt)*PA(idxparopt)/(1._rd+dot_product(pder,PA))				
			end if!information entropy
			if (dsigi>dsigmax) then
				idxMax  = i
				dsigmax = dsigi
			end if !dsigmax
! 			write(*,*) 'rv', i, idxmax, dsigmax, dsigi
		end if! time available
		
	end do!i 
	deallocate(pder,PA)
end subroutine findOptRV_single

subroutine initCov()
  use constantsMod, only: nparamglob,nparamloc,tofitglobal,fromJY2MJD,priors,npriors
	use fitMod, only: fitsvd,computeSig
	integer(id) :: i,idxObs,j
	real(rd), dimension(:,:), allocatable :: pder
	real(rd), dimension(:), allocatable   :: sig,omc,par
	logical, dimension(:), allocatable :: parmask
	real(rd) :: chi2

	allocate(pder(npastobs+npriors,npartofit))
	allocate(sig(npastobs+npriors),omc(npastobs+npriors),par(npartofit),parmask(npartofit))
	omc=0._rd; parmask=.true.
	
	if (allocated(cov) ) deallocate(cov)
	allocate(cov(npartofit,npartofit))
	write(*,*)" =================================== "
	write(*,*)"  Preliminary step: loading the partials"
	idxObs=1
	do i=1,nstars
		if (stars(i)%npastastro>0)then
			do j=1,nparamGlob
				if (tofitglobal(j)) then
					pder(idxObs:idxObs+stars(i)%npastastro-1,idxPar(j))=stars(i)%dradp(1:stars(i)%npastastro,j)					
					pder(idxObs+stars(i)%npastastro:idxObs+2*stars(i)%npastastro-1,idxPar(j))=stars(i)%ddecdp(1:stars(i)%npastastro,j)
				end if
			end do!j			
			do j=1,nparamLoc
				if (stars(i)%tofit(j)) then
					pder(idxObs:idxObs+stars(i)%npastastro-1,idxPar(nparamglob+(i-1)*nparamloc+j))  &
							& =stars(i)%dradp(1:stars(i)%npastastro,nparamglob+j)
					pder(idxObs+stars(i)%npastastro:idxObs+2*stars(i)%npastastro-1,idxPar(nparamglob+(i-1)*nparamloc+j)) &
							& = stars(i)%ddecdp(1:stars(i)%npastastro,nparamglob+j)
				end if
			end do!j
			sig(idxObs:idxObs+stars(i)%npastastro-1)=stars(i)%sigra(1:stars(i)%npastastro)
			sig(idxObs+stars(i)%npastastro:idxObs+2*stars(i)%npastastro-1)=stars(i)%sigdec(1:stars(i)%npastastro)
			idxObs=idxObs+2*stars(i)%npastastro
		end if !astro
		
		if (stars(i)%npastrv>0) then
			do j=1,nparamGlob
				if (tofitglobal(j)) then

					pder(idxObs:idxObs+stars(i)%npastrv-1,idxPar(j))=stars(i)%drvdp(1:stars(i)%npastrv,j)
				end if
			end do!j			
			do j=1,nparamLoc
				if (stars(i)%tofit(j)) then
					pder(idxObs:idxObs+stars(i)%npastrv-1,idxPar(nparamglob+(i-1)*nparamloc+j))  &
							& =stars(i)%drvdp(1:stars(i)%npastrv,nparamglob+j)
				end if
			end do!j
			sig(idxObs:idxObs+stars(i)%npastrv-1)=stars(i)%sigrv(1:stars(i)%npastrv)
			idxObs=idxObs+stars(i)%npastrv
		end if!RV
	end do!i - nstars
	if (npriors>0) then
		do i=1,npriors
			pder(idxObs,priors(i)%idx) = 1._rd
			sig(idxObs)=priors(i)%sig
			idxObs=idxObs+1
		end do!i
	end if!nrpiors
	
	write(*,*)"      Inversion of the normal matrix"
	call fitsvd(pder,OMC,sig,par,parmask,cov,chi2)
	write(*,*)" Preliminary step performed successfully"
	write(*,*)" =================================== "

	if (allocated(pastGPcov) ) deallocate(pastGPcov)
	allocate(pastGPcov(npartofit,npartofit))
!===============past cov in terminal
	do i=1,npartofit
			!write(*,'(2x,G10.5)') sqrt(cov(i,i))
			pastGPcov(i,i) =  sqrt(cov(i,i))
	end do

end subroutine initCov


subroutine writeAllUncertainties(f)
	use constantsMod
	use modelinterface
	integer(id), intent(in) :: f
	integer(id) :: j, istar,idxparloc
		do j=1,nparam
			if (j<=nparamglob) then
				if (tofitglobal(j)) then
					write(f,'(1x,G10.5,1x)', advance='no') sqrt(cov(idxpar(j),idxpar(j))) 
				else
					write(f,'(3x,i2,7x)', advance='no')  -1
				end if
			else
				istar=(j-1-nparamglob)/nparamloc+1
				idxparloc=j-nparamglob-(istar-1)*nparamloc

				if (stars(istar)%tofit(idxparloc)) then
					write(f,'(1x,G10.5,1x)', advance='no') sqrt(cov(idxpar(j),idxpar(j)))  
				else
					write(f,'(3x,i2,7x)', advance='no')  -1
				end if
			end if
		end do !j
end subroutine writeAllUncertainties

subroutine writeInput()
  use constantsMod
  use modelinterface
  integer(id) :: i,istar,j
!==================writeOut
  open(unit=fout,file = trim(dirout)//'/'//trim(name)//extres,action='write',status='replace')
  	
	write(fout,*)  "  ====================================================  "
	write(fout,*)  "         INPUT    "
	write(fout,*)  "  ====================================================  "
	write(fout,*)   " Covariance analysis made using the model: 			", trim(modelname)
	write(fout,*)   "     - Global parameteres used: "
	do i=1,nparamglob
			if (toFitGlobal(i)) then
				 write(fout,'(A,A,A,G10.4,A)') "     - ",nameGlob(i)," : ",globPar(i),"   -- FITTED"
			else
				 write(fout,*) "     - ",nameGlob(i)," : ",globPar(i),"   -- NOT FITTED"
			end if
	end do!i
	write(fout,*)  " Number of stars simulated: ",nstars
	do i=1,nstars
		write(fout,*) "    ",trim(stars(i)%name)," with local parameters: "
		do j=1,nparamloc
			if (stars(i)%tofit(j)) then
				write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- FITTED"
			else
				write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- NOT FITTED"
			end if
		end do!j
		if (stars(i)%astrob) then
			write(fout,*)   '    with ',stars(i)%na,' astro data from file: ',trim(stars(i)%astrofile)
		else
			write(fout,*)   '    with NO astro data'
		end if!astrob
		if (stars(i)%rvb) then
			write(fout,*)   '    with ',stars(i)%nrv,' RV data from file: ',trim(stars(i)%rvfile)
		else
			write(fout,*)   '    with NO RV data'
		end if!astrob
		write(fout,*)""
	end do!i


	write(fout,*)' Priors :'
	if(npriors<=0) then 
		write(fout,*)'   No priors considered'
	else
		write(fout,'(i3,A)') npriors, " priors considered:"
		do i=1,npriors
			write(fout,'(A,i3,A)',advance='no') " Prior n. ", i, ' on parameter '
			if (priors(i)%idx<= nparamglob) then
				write(fout,*) nameGlob(priors(i)%idx),' with uncertainty of ', priors(i)%sig
			else
					istar=(priors(i)%idx-1-nparamglob)/nparamloc+1
					write(fout,'(A,A,A,A,G10.4)') nameloc(priors(i)%idx-nparamglob-(istar-1)*nparamloc), ' from star ',trim(stars(istar)%name),&
						&' with uncertainty of ', priors(i)%sig
			end if
		end do!i
	end if

	write(fout,*)  "  ====================================================  "
	write(fout,*)  "      CADENCE TOOL INPUT    "
	write(fout,*)  "  ====================================================  "

	write(fout,*) 'Number of iteration:							', Niteration
	write(fout,*) 'Number of astro time intervals:					', Ntintervalsastro
	write(fout,*) 'Number of RV time intervals:					', NtintervalsRV
	
	do i=1, Ntintervalsastro
			write(fout,*) 'Beginning and end of an astro interval:   ', tintervalsastro(1,i), '--', tintervalsastro(2,i)
	end do !i
	write(fout,*)''
	do i=1, NtintervalsRV
			write(fout,*) 'Beginning and end of an RV interval:   ', tintervalsrv(1,i), '--', tintervalsrv(2,i)
	end do !i

	write(fout,*) 'Increment:                                  ', increment, 'year'

	if (informationEntropy)then
			write(fout,*) 'Optimization on information entropy'
	else
		if (gpOptimization <= nparamGlob)then
			write(fout,*) 'Optimization of a global parameter:         ', nameGlob(gpOptimization)
		else
			istar=(gpOptimization-1-nparamglob)/nparamloc+1
			write(fout,*) 'Optimization on local parameter: ',nameloc(gpOptimization-nparamglob-(istar-1)*nparamloc),&
			 	&' from star ',trim(stars(istar)%name)
		end if
end if
!close(fout)
!stop

  write(*,*) 'END OF INPUT'
end subroutine writeInput
end module cadenceMod
