module dynamicMod
  use kindtype
	use starsMod, only: nstars, star, stars
	
  implicit none
   

	public   ::  initdyn, writeSimulated, writeOut, addParam
	
contains

subroutine  initDyn(nameIn)
  use constantsMod, only: dirout, name,signext,timenext,priors,npriors,tofitglobal,nparamglob,nparamloc, nparam
	use modelInterface, only: setModel, computePartials
	use starsMod, only:loadstar
!	use odemod, only:setup
  implicit none
	character(len=*), intent(in) :: nameIn

  integer(id) :: i,j, fittmp,k
  integer(id) :: itmp
  logical(lgt) :: exist

	character(len=15) :: modelStr


	!
	!Reading the file
	!
	  inquire(file=nameIn, EXIST=exist) 
	  if (.NOT.exist) then
	      WRITE(*,*) "ERROR : File ", trim(nameIn)," does not exist"
	      STOP
	  endif
	  open(unit=55,file=nameIn,action='read')
	
	 read(55,'(A)') dirout
#ifdef IFORT	 
	inquire(directory=trim(dirout),exist=exist)
#else	
	inquire(file=trim(dirout),exist=exist)
#endif	

	if (.not.exist) then
		write(*,*) 'Creating folder : ', trim(dirout)
		call system(' mkdir '//trim(dirout))
	end if
	!write(*,*) exist,trim(dirout)
	write(*,*)'Output directory: ',trim(dirout)
	 read(55,'(A)') name	
	write(*,*)'Name: ',name

	read(55,'(i1)',ADVANCE='NO')  i
	if (i==1) then
		sigNext=.true.
		read(55,*) timeNext
	else
		read(55,*)
	end if

	read(55,'(A)') modelStr

	
	read(55,*) npriors
	if (npriors>0) then
		allocate(priors(npriors))
		do i=1,npriors
			read(55,*)priors(i)%idx,priors(i)%sig
		end do !i
	end if!n priors>0
	
	call setModel(modelstr,55)

	
	read(55,'(i4)') nstars
	nparam=nparamglob+nstars*nparamloc
	if (nstars<1) then
		write(*,*) "ERROR : number of stars should be at least 1 and it is: ",nstars
		stop
	end if

	if (npriors>0) then
		do i=1,npriors
			if (priors(i)%idx>nparam) then
				write(*,*) "ERROR: index of prior ", i, " too large: ",priors(i)%idx
				stop
			end if
			if (priors(i)%sig<=0._rd) then
				write(*,*)"ERROR: non positive sigma for prior ", i
				stop
			end if
		end do!i
	end if!npriors
	
	if (allocated(stars)) deallocate(stars)
	allocate(stars(nstars))

	do i=1, nstars
		allocate(stars(i)%par(nparamloc))
		allocate(stars(i)%tofit(nparamloc))
		
		read(55,*)stars(i)%name
		read(55,'(A)')stars(i)%astroFile
		read(55,'(A)')stars(i)%rvFile
		stars(i)%ntofit=0
		do j=1,nparamloc
					read(55,*) stars(i)%par(j),fittmp
					stars(i)%tofit(j)=(fittmp==1)
					if (stars(i)%tofit(j))  stars(i)%ntofit=stars(i)%ntofit+1
		end do!j
		call loadStar(stars(i))
		allocate(stars(i)%idx_fit(stars(i)%ntofit))
    k=0
    do j=1,nparamloc
      	if (stars(i)%tofit(j)) then
      	  k=k+1
      	  stars(i)%idx_fit(k)=j
    	  end if
    end do!j
	end do! i
	
	close(55)
	write(*,*)""
	write(*,*)" Computation of the next uncertainty on observables at time: ", timenext
	write(*,*)""
end subroutine  initDyn



subroutine writeSimulated()
	use constantsMod
	integer(id) :: i,j
	
	
	! PRINT SIMULATED DATA
	do i=1,nstars
		if (stars(i)%astrob) then
			open(unit=60,file=trim(dirout)//'/'//trim(name)//'_'//trim(stars(i)%name)//extang,action='write',status='replace')
			do j=1,stars(i)%na
				write(60,*) stars(i)%tastro(j),-stars(i)%ra(j)/1000._rd,stars(i)%dec(j)/1000._rd,0., 0., fromJY2MJD(stars(i)%tastro(j))
			end do !j
			close(60)
		end if!
		if (stars(i)%rvb) then
			open(unit=61,file=trim(dirout)//'/'//trim(name)//'_'//trim(stars(i)%name)//extrv,action='write',status='replace')
			do j=1,stars(i)%nrv
				write(61,*) stars(i)%trv(j),stars(i)%rv(j),0., fromJY2MJD(stars(i)%trv(j))
			end do !j
			close(61)
		end if!
		
	end do !i
	! info to be used later
!	open(unit=60,file=file,action='write',status='replace')
end subroutine writeSimulated

subroutine writeOut(fout)
	use constantsMod
	use modelinterface
	integer(id), intent(in)  :: fout
	integer(id) :: i,j,istar
	
	open(unit=fout,file = trim(dirout)//'/'//trim(name)//extres,action='write',status='replace')
	
	write(fout,*)  "  ====================================================  "
	write(fout,*)  "         INPUT    "
	write(fout,*)  "  ====================================================  "
	write(fout,*)   " Covariance analysis made using the model: ", trim(modelname)
	write(fout,*)   "     - Global parameteres used: "
	do i=1,nparamglob
			if (toFitGlobal(i)) then
				 write(fout,'(A,A,A,G10.4,A)') "     - ",nameGlob(i)," : ",globPar(i),"   -- FITTED"
			else
				 write(fout,*) "     - ",nameGlob(i)," : ",globPar(i),"   -- NOT FITTED"
			end if
	end do!i
	write(fout,*)  " Number of stars simulated: ",nstars
	do i=1,nstars
		write(fout,*) "    ",trim(stars(i)%name)," with local parameters: "
		do j=1,nparamloc
			if (stars(i)%tofit(j)) then
				write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- FITTED"
			else
				write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- NOT FITTED"
			end if
		end do!j
		if (stars(i)%astrob) then
			write(fout,*)   '    with ',stars(i)%na,' astro data from file: ',trim(stars(i)%astrofile)
		else
			write(fout,*)   '    with NO astro data'
		end if!astrob
		if (stars(i)%rvb) then
			write(fout,*)   '    with ',stars(i)%nrv,' RV data from file: ',trim(stars(i)%rvfile)
		else
			write(fout,*)   '    with NO RV data'
		end if!astrob
		write(fout,*)""
	end do!i
	
	if (signext) then
		write(fout,*)" Computation of the next uncertainty on observables at time: ", timenext
		write(fout,*)""
	end if
	
	write(fout,*)" === PRIORS === "
	if (npriors>0) then
		write(fout,'(i3,A)') npriors, " priors considered:"
		do i=1,npriors
			write(fout,'(A,i3,A)',advance='no') " Prior n. ", i, ' on parameter '
			if (priors(i)%idx<= nparamglob) then
				write(fout,*) nameGlob(priors(i)%idx),' with uncertainty of ', priors(i)%sig
			else
					istar=(priors(i)%idx-1-nparamglob)/nparamloc+1
					write(fout,'(A,A,A,A,G10.4)') nameloc(priors(i)%idx-nparamglob-(istar-1)*nparamloc), ' from star ',trim(stars(istar)%name),&
						&' with uncertainty of ', priors(i)%sig
			end if
		end do!i
	else
		write(fout,*)" NO prior considered"
	end if
		
	
end subroutine writeOut


subroutine addParam(p,e)
  use constantsMod, only:nparamglob,nparamloc
  use modelInterface, only :globpar
  integer(id), intent(in) :: p
  real(rd), intent(in)    :: e
  integer(id) :: istar, ipar
  
  if (p<=nparamglob) then
    globpar(p)=globpar(p)+e
  else
     istar = (p-nparamglob-1)/nparamloc +1
     ipar  =  p-nparamglob-(istar-1)*nparamloc
     stars(istar)%par(ipar)=stars(istar)%par(ipar)+e
  end if
  
end subroutine addParam

subroutine opti_blocks()
	use fitMod, only: inverse_SVD
	use starsMod, only: init_block_star,star_included
	use constantsMod, only: dirout,name,nparamGlob,nglobtofit,tofitglobal
	integer(id)     :: i,j,idxFit

	real(rd), dimension(:,:), allocatable    ::  U, Uinv,Up


  allocate(U(nglobtofit,nglobtofit),Up(nglobtofit,nglobtofit),Uinv(nglobtofit,nglobtofit))
  U=0._rd
  ! initialization
  do i=1,nstars
    if (star_included(stars(i))) then
      call init_block_star(stars(i))
      U=U+stars(i)%ui
    end if
  end do!

  open(unit=67,file = trim(dirout)//'/'//trim(name)//'.opt',action='write',status='replace')

  ! all stars
  call inverse_SVD(U,Uinv)
  idxFit=1
  write(67,'(i8,2x,A3,11x)',advance='no') j, 'All'
  do i=1,nparamGlob
    if (tofitglobal(i)) then
      write(67,'(g10.5,3x)',advance='no')sqrt(Uinv(idxFit,idxFit))
      idxFit=idxFit+1
    else
      write(67,'(g10.5,3x)',advance='no')-1
    end if
  end do
  write(67,*)
  
  do j=1,nstars
    write(67,'(i8,2x,A10,4x)',advance='no') j,stars(j)%name
    if (star_included(stars(j))) then
	    Up = U - stars(j)%Ui
	    call inverse_SVD(Up,Uinv)
      idxFit=1
	    do i=1,nparamGlob
	      if (tofitglobal(i)) then
	        write(67,'(g10.5,3x)',advance='no')sqrt(Uinv(idxFit,idxFit))
	        idxFit=idxFit+1
        else
          write(67,'(g10.5,3x)',advance='no')-1
        end if
      end do
    else
      do i=1,nparamGlob
        write(67,'(g10.5,3x)',advance='no')-1
      end do
    end if
    write(67,*)
  end do!j
  close(67)
end subroutine opti_blocks
end module dynamicMod