module keplerRedshiftMod
  use kindtype
  use starsMod, only: star
  use constantsMod, only : pi, solvekepler
	implicit none
	public :: computeObservationsKeplerRedshift, getNumParamKeplerRedshift,getNameParamKeplerRedshift
	private :: createKeplerRedshift, astro, rv, orbPlane, solveKepler
	
	integer(id), private,parameter :: nglob=8
	integer(id), private,parameter :: nloc=6
	integer(id), private,parameter :: ntot=nloc+nglob	
	

	real(rd), private, parameter   :: torigin=2000._rd
	
	character(len=5), dimension(nglob), parameter, private :: nameGlob=&
				&(/"GM   ","x0   ","y0   ","vx0  ","vy0  ","vz0  ","R0   ","Red. "/)
	character(len=5), dimension(nloc), parameter, private :: nameLoc=&
				&(/"P    ","T0   ","e    ","i    ","om   ","Om   "/)
	
	type, private :: orb
		real(rd), private      :: a
		real(rd), private      :: T
		real(rd), private      :: t0
		real(rd), private      :: e
		real(rd), private      :: w
		real(rd), private      :: i
		real(rd), private      :: o
		real(rd), private      :: n	

		real(rd), private      :: Arot
		real(rd), private      :: Brot
		real(rd), private      :: Crot

		real(rd), private      :: Frot
		real(rd), private      :: Grot
		real(rd), private      :: Hrot

		real(rd), private      :: se

		real(rd), private      :: GM
		real(rd), private      :: x0
		real(rd), private      :: y0
		real(rd), private      :: vx0
		real(rd), private      :: vy0
		real(rd), private      :: vz0
		real(rd), private      :: R0
		real(rd), private      :: GRRedshift
	end type orb



contains


subroutine createKeplerRedshift(parG,parL,o)
	use constantsMod, only: GMs_yr2_au3
	real(rd), dimension(nglob), intent(in) :: parG
	real(rd), dimension(nloc), intent(in)  :: parL	
	type(orb), intent(out)                 :: o
	
	!local paramaters
	o%t=parl(1)
	o%t0=parl(2)
	o%e=parl(3)
	o%i=parl(4)*pi/180.0_rd
	o%w=parl(5)*pi/180.0_rd
	o%o=parl(6)*pi/180.0_rd
	
	! global parameters
	o%GM=parg(1)   ! in 10^6 GM_sun 
	o%x0=parg(2)   ! in mas 
	o%y0=parg(3)   ! in mas 
	o%vx0=parg(4)  ! in mas /yr
	o%vy0=parg(5)	 ! in mas / yr
	o%vz0=parg(6)  ! in km/s
	o%R0=parg(7)   ! in kpc
	o%GRRedshift=parg(8)
	
	o%n=2._rd*pi/o%T
	o%se=sqrt(1._rd-o%e**2)
	o%a=(o%GM*1.e6_rd/o%n**2*GMs_yr2_au3)**(1._rd/3._rd)! in AU
	
	o%Arot =  cos(o%w)*cos(o%O) - sin(o%w)*sin(o%O)*cos(o%i)!
	o%Brot =  cos(o%w)*sin(o%O) + sin(o%w)*cos(o%O)*cos(o%i)
	
	o%Frot = -sin(o%w)*cos(o%O) - cos(o%w)*sin(o%O)*cos(o%i)
	o%Grot = -sin(o%w)*sin(o%O) + cos(o%w)*cos(o%O)*cos(o%i)

	o%Crot = sin(o%w)*sin(o%i)
	o%Hrot = cos(o%w)*sin(o%i)	
	
end subroutine createKeplerRedshift



subroutine computeObservationsKeplerRedshift(pg,s)
	use constantsMod, only: sigNext	, timenext
	real(rd), dimension(nglob), intent(in) :: pg	
	type(star), intent(inout)  :: s
	type(orb) :: o
	integer(id) :: i

	real(rd) ::ra,dec, x,y,vx,vy
	real(rd), dimension(ntot) ::dradp,ddecdp, dxdp,dydp,dvxdp,dvydp
	
	call createKeplerRedshift(pg,s%par,o)
	if (s%astrob) then
		write(*,'(A)',advance='no') "*"
		do i=1,s%na
			call astro(o,s%tastro(i),s%ra(i),s%dec(i),s%dradp(i,:),s%ddecdp(i,:))
		end do!nastro
	end if! astro
	if (s%rvb) then
		write(*,'(A)',advance='no') "*"
		do i=1,s%nrv
			call rv(o,s%trv(i),s%rv(i),s%drvdp(i,:))
		end do!i
	end if! rv
	if(sigNext) then
		call astro(o,timenext,s%ranext,s%decnext,s%dradpnext(:),s%ddecdpnext(:))
		call rv(o,timenext,s%rvnext,s%drvdpnext(:))

		if (s%astrob) then
    	s%diffastronext(1)=s%ranext-s%ra(s%na);			s%diffastronext(2)=s%decnext-s%dec(s%na)
		else
			s%diffastronext(1)=0._rd; s%diffastronext(2)=0._rd
		end if
		if (s%rvb) then
    	s%diffRVnext = s%rvnext-s%rv(s%nrv)
		else
			s%diffRVnext = 0._rd
		end if
	end if

end subroutine computeObservationsKeplerRedshift

subroutine rv(o,t,rvo,drvdp)
	use constantsMod, only: GMs_yr2_au3, AUyr2kms,c
	type(orb), intent(in) :: o
	real(rd), intent(in)  :: t
	real(rd), intent(out) :: rvo
	real(rd), intent(out), dimension(ntot) :: drvdp
	real(rd)  :: x,y,vx,vy,r,v2,SR,GR
	real(rd), dimension(ntot)   :: dxdp,dydp,dvxdp,dvydp,drdp,dv2dp,dsrdp,dgrdp
	
	call orbPlane(o,t,x,y,dxdp,dydp,vx,vy,dvxdp,dvydp)
	rvo = (vx * o%Crot + vy * o%Hrot)*AUyr2kms + o%vz0
  write(*,*) vx,vy
	drvdp = dvxdp * o%Crot + dvydp * o%Hrot
	drvdp(nglob+4)=drvdp(nglob+4)+cos(o%i)*sin(o%w)*vx+cos(o%i)*cos(o%w)*vy ! i
	drvdp(nglob+5)=drvdp(nglob+5)+sin(o%i)*cos(o%w)*vx-sin(o%i)*sin(o%w)*vy ! omega
	
	drvdp=drvdp*AUyr2kms
	drvdp(6)=drvdp(6)+1._rd
	
	! RELATIVITY
	r    = sqrt(x**2+y**2)
	v2   = vx**2+vy**2
	SR = 0.5_rd*v2/c*AUyr2kms
	GR = o%GM*1.e6_rd/r/c *GMs_yr2_au3*AUyr2kms
	
	drdp = (x*dxdp + y * dydp)/r
	dv2dp = 2._rd*(vx*dvxdp + vy*dvydp)

	dSRdp = 0.5_rd*dv2dp/c*AUyr2kms
	dGRdp = -GR*drdp/r
	dgrdp(1) =	dgrdp(1) + gr/o%GM ! der wrt to GM

	rvo = rvo + o%GRRedshift *(sr+gr)
	drvdp = drvdp + o%GRRedshift *(dsrdp+dgrdp)
	drvdp(8) = 	drvdp(8) + sr+gr
	drvdp(nglob+4:nglob+6)=drvdp(nglob+4:nglob+6)*pi/180._rd

end subroutine rv

subroutine astro(o,t,ra,dec,dradp,ddecdp)
	type(orb), intent(in) :: o
	real(rd), intent(in)  :: t
	real(rd), intent(out) :: ra,dec
	real(rd), intent(out), dimension(ntot) :: dradp, ddecdp
	
	real(rd)  :: x,y,vx,vy
	real(rd), dimension(ntot)   :: dxdp,dydp,dvxdp,dvydp
	
	call orbPlane(o,t,x,y,dxdp,dydp,vx,vy,dvxdp,dvydp)
	
	ra = x * o%Brot + y * o%Grot
	
	dradp = dxdp * o%Brot + dydp * o%Grot
	dradp(nglob+4) = dradp(nglob+4) - x*cos(o%o)*sin(o%i)*sin(o%w) - y*cos(o%o)*sin(o%i)*cos(o%w) ! i
	dradp(nglob+5) = dradp(nglob+5) + x* (-sin(o%w)*sin(o%o) + cos(o%o)*cos(o%i)*cos(o%w))+ &
                              &	    y* (-sin(o%o)*cos(o%w) - cos(o%o)*cos(o%i)*sin(o%w)) ! omega 
	dradp(nglob+6) = dradp(nglob+6)	+ x* (cos(o%w)*cos(o%o) -sin(o%o)*cos(o%i)*sin(o%w)) + &
															&     y* (-cos(o%o)*sin(o%w)-sin(o%o)*cos(o%i)*cos(o%w))! Omega

	ra = ra/o%R0
	dradp=dradp/o%R0

	dradp(7) = dradp(7) - ra/o%r0
	
	ra=ra - o%x0 - o%vx0*(t-torigin)
	dradp(2)=dradp(2)-1._rd
	dradp(4)=dradp(4)-(t-torigin)
	
	
	dec    = x* o%Arot + y * o%Frot
		
	ddecdp = dxdp* o%Arot + dydp * o%Frot
	ddecdp(nglob+4) = ddecdp(nglob+4) + x*sin(o%o)*sin(o%i)*sin(o%w) + y*sin(o%o)*sin(o%i)*cos(o%w) ! i
	ddecdp(nglob+5) = ddecdp(nglob+5) + x* (-sin(o%w)*cos(o%o)-sin(o%o)*cos(o%i)*cos(o%w)) + &
	    											&					y* (-cos(o%o)*cos(o%w)+sin(o%o)*cos(o%i)*sin(o%w)) ! omega
	ddecdp(nglob+6) = ddecdp(nglob+6) + x* (-cos(o%w)*sin(o%o)-cos(o%o)*cos(o%i)*sin(o%w)) + &
														& 	      y* (sin(o%o)*sin(o%w)-cos(o%o)*cos(o%i)*cos(o%w))!Omega

	dec = dec/o%r0
	ddecdp = ddecdp/o%r0
	ddecdp(7) = ddecdp(7) - dec/o%r0

	dec = dec + o%y0 + o%vy0*(t-torigin)
	ddecdp(3)=	ddecdp(3) + 1._rd
	ddecdp(5)=	ddecdp(5) + (t-torigin)	

	dradp(nglob+4:nglob+6)=dradp(nglob+4:nglob+6)*pi/180._rd
	ddecdp(nglob+4:nglob+6)=ddecdp(nglob+4:nglob+6)*pi/180._rd
end subroutine astro

subroutine orbPlane(o,t,x,y,dxdp,dydp,vx,vy,dvxdp,dvydp)
	type(orb), intent(in) :: o
	real(rd), intent(in)  :: t
	real(rd), intent(out) :: x,y,vx,vy
	real(rd), intent(out), dimension(ntot) :: dxdp,dydp,dvxdp,dvydp

	real(rd), dimension(ntot) :: dadp, dEdp,dndp,dmdp,dtmpvdp
	real(rd) :: m,E,sE,cE,tmpv
	
	dadp(:)=0._rd;	dEdp(:)=0._rd;	dndp(:)=0._rd;

    dndp(nglob+1)=-o%n/o%t
	
	dadp=-2._rd*o%a / (3._rd*o%n) *dndp
	dadp(1)=dadp(1)+ o%a/ o%GM / 3._rd
	
	m=o%n*(t-o%t0)
	dmdp=dndp*(t-o%t0)
	dmdp(nglob+2) =	dmdp(nglob+2) - o%n
	call solveKepler(m,o%e,E)
	sE=sin(E);cE=cos(E)
	
	dEdp=dmdp
	dEdp(nglob+3) = dEdp(nglob+3) + sE !der wrt to e
	dEdp=dEdp/(1._rd-o%e*cE)

	x = o%a*(ce-o%e) 
	y = o%a*o%se*se

	dxdp = (cE-o%e)*dadp -o%a*se*dedp
	dxdp(nglob+3) = dxdp(nglob+3) - o%a! der wrt e
	
	dydp = y/o%a *dadp + o%a*o%se*cE*dEdp
	dydp(nglob+3) = dydp(nglob+3) - o%a*o%e*sE/o%se
	
	tmpv = o%n*o%a/(1-o%e*ce)
	dtmpvdp = (dndp*o%a + o%n*dadp)/(1._rd-o%e*ce) - o%n*o%a*o%e*sE/(1._rd-o%e*cE)**2*dEdp
	dtmpvdp(nglob+3) = dtmpvdp(nglob+3) + o%n*o%a*cE/(1._rd-o%e*cE)**2   !der wrt e
	
	vx = -tmpv*se
	vy = o%se*tmpv*ce

	dvxdp = -dtmpvdp*se - tmpv*ce*dEdp
	dvydp = dtmpvdp*o%se*ce - tmpv*o%se*se*dEdp
	dvydp(nglob+3)=dvydp(nglob+3) - tmpv*cE*o%e/o%se!der wrt e
	
end subroutine orbPlane


subroutine getNumParamKeplerRedshift(ng,nl)
	integer(id), intent(out) :: ng,nl
	ng=nglob
	nl=nloc
end subroutine getNumParamKeplerRedshift

subroutine getNameParamKeplerRedshift(ng,nl)
	character(len=5), dimension(nglob), intent(out) :: ng
	character(len=5), dimension(nloc), intent(out) :: nl
	ng=nameglob
	nl=nameloc
end subroutine getNameParamKeplerRedshift


end module keplerRedshiftMod