!> @file kindtype.f90
!! Definition of module kindtype where variable are defined

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!  Module  kindtype                                            
!> Module where the kind of the variables are defined
!>
!> Compilation: preprocessor variables needed: QUAD. QUAD=1, the compilation will consider quadruple precision for the floating numbers. 
!> If compilation with Intel compiler (IFORT), the variable IFORT should also be defined.
!!
!! @author Aurelien Hees, ORB, aurelien.hees@gmail.com
!! @date 05/2010 
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
module kindtype

!> Defining single real
integer, parameter :: rsingle  = selected_real_kind(4), rp = rsingle

!> Defining integer kind : isingle=8bits
integer, parameter :: isingle  = selected_int_kind(3), is = isingle
!> Defining integer kind : idouble=16 bits 
integer, parameter :: idouble  = selected_int_kind(6), id = idouble,iwp=id

!> Defining logical kind 
integer, parameter :: lgt  = kind(.TRUE.)

#if QUAD==1
	!> Defining real double (here quad)
   integer, parameter :: rdouble = selected_real_kind(16), rd = rdouble, wp=rd

#else    
 !> Defining real double   
        integer, parameter :: rdouble  = selected_real_kind(8), rd = rdouble, wp=rd
#endif

integer, parameter :: dp = rd
end module kindtype
