module modelInterface
  use kindtype
  use keplerMod
	use keplerredshiftmod	
	use relativisticMod		
	use fifthforceMod
	use BHMod		
	use keplerAdvanceMod
  implicit none
  
  character(LEN=100), public, save                      :: modelname  !< 
	integer, public, save                                 :: model=0     !< 1= Kepler standard
	real(rd), public, save                                :: lambda5
  real(rd),public, save :: gamext,radext

  
  character(len=5), dimension(:), allocatable, public, save :: nameLoc
	character(len=5), dimension(:), allocatable, public, save :: nameGlob

  real(rd), dimension(:), public, allocatable           :: globPar



contains

subroutine setModel(str,fin)
  use constantsMod
  implicit none
  character(len=*), intent(in) :: str
  integer(id), intent(in) :: fin
  
  integer :: fittmp,nohairtmp
  integer(id) :: i,j
  if (allocated(globPar)) deallocate(globPar)
  if (allocated(nameglob)) deallocate(nameglob)
  if (allocated(nameloc)) deallocate(nameloc)
  if (allocated(toFitGlobal)) deallocate(toFitGlobal)
  
  modelname=str
	
	SELECT CASE (trim(str))
	   CASE ('kepler')
				model=1
	      call getNumParamKepler(nparamglob,nparamloc)
				write(*,*) " Model used: Kepler"
				write(*,*) "   Number of global parameters: ",nparamglob
				write(*,*) "   Number of local parameters : ",nparamloc
				allocate(nameglob(nparamglob),nameloc(nparamloc))
				call getNameParamKepler(nameglob,nameloc)
		 CASE ('keplerRedshift')
				model=2
	      call getNumParamKeplerRedshift(nparamglob,nparamloc)
				write(*,*) " Model used: Kepler + Redshift"
				write(*,*) "   Number of global parameters: ",nparamglob
				write(*,*) "   Number of local parameters : ",nparamloc
				allocate(nameglob(nparamglob),nameloc(nparamloc))
				call getNameParamKeplerRedshift(nameglob,nameloc)				
		 CASE ('relativistic')
				model=3
	      call getNumParamRelativistic(nparamglob,nparamloc)
				write(*,*) " Model used: Relativistic"
				write(*,*) "   Number of global parameters: ",nparamglob
				write(*,*) "   Number of local parameters : ",nparamloc
				allocate(nameglob(nparamglob),nameloc(nparamloc))
				call getNameParamRelativistic(nameglob,nameloc)

      CASE ('5thforce')
		    model=4
             call getNumParam5f(nparamglob,nparamloc)
	        write(*,*) " Model used: 5th force"
		    write(*,*) "   Number of global parameters: ",nparamglob
		    write(*,*) "   Number of local parameters : ",nparamloc
		    allocate(nameglob(nparamglob),nameloc(nparamloc))
		    call getNameParam5f(nameglob,nameloc)
		
		 CASE('BH')	
		 		model=5
	      call getNumParamBH(nparamglob,nparamloc)
				write(*,*) " Model used: Blackhole "
				write(*,*) "   Number of global parameters: ",nparamglob
				write(*,*) "   Number of local parameters : ",nparamloc
				allocate(nameglob(nparamglob),nameloc(nparamloc))
				call getNameParamBH(nameglob,nameloc)
				
			CASE('keplerAdvance')	
   		 		model=6
   	      call getNumParamKeplerAdvance(nparamglob,nparamloc)
   				write(*,*) " Model used: Kepler + Redshift + Advance of periastron "
   				write(*,*) "   Number of global parameters: ",nparamglob
   				write(*,*) "   Number of local parameters : ",nparamloc
   				allocate(nameglob(nparamglob),nameloc(nparamloc))
   				call getNameParamKeplerAdvance(nameglob,nameloc)
	   CASE DEFAULT
	      write(*,*) ' ERROR: wrong type of model: ', str
				stop
	END SELECT

	nglobtofit=0
	allocate(globPar(nparamglob))
	allocate(toFitGlobal(nparamglob))
	do i=1,nparamglob
	  if (((model==3).or.(model==5).or.(model==4)).and.(i==8)) then ! extended mass
	    read(fin,*)globpar(i),gamext,radext,fittmp
    else if ((model==4).and.(i==9)) then
		  read(fin,*)globpar(i),lambda5,fittmp  
		else if ((model==5).and.(i==14)) then ! quadrupole
		  read(fin,*)globpar(i),nohairtmp,fittmp
      if (nohairtmp==1) globpar(i)=globpar(11)**2 !No HAIR    
    else
		  read(fin,*)globpar(i),fittmp
	  end if
    if ((model==5).and.(i==14)) then ! quadrupole
      toFitGlobal(i)=(fittmp==1)
      if ((fittmp==2).and.(.not.toFitGlobal(11)))  then
        write(*,*) "ERROR: Fit of Q while assuming no hair but S is not fitted "
        stop
      end if
      if (fittmp==2) call setNoHair()
    else
		  toFitGlobal(i)=(fittmp==1)
	  end if
		if (tofitglobal(i))  nglobtofit=nglobtofit+1
	end do!i
	if(allocated(idx_glob))  deallocate(idx_glob)
  allocate(idx_glob(nglobtofit))

  j=1
  do i=1,nparamglob
    if (tofitglobal(i)) then
      idx_glob(j)=i
      j=j+1
    end if
  end do

end subroutine setModel
subroutine computePartials()
	use keplermod
	use keplerredshiftmod	
	use relativisticmod
	use fifthforcemod		
	use BHMod		
	use keplerAdvanceMod	
	use starsMod, only : nstars, stars

	integer(id) :: i
	write(*,*) " ==================================== "
	write(*,*) "    COMPUTING PARTIALS  "
	do i=1,nstars
		write(*,'(A,A,2x)',advance='no')  "-   Star ",trim(stars(i)%name)
		select case (model)
			case (1)
				call computeObservationsKepler(globPar,stars(i))
			case (2)
				call computeObservationsKeplerRedshift(globPar,stars(i))				
			case (3)
				call computeObservationsRelativistic(globPar,stars(i),gamext,radext)
			case (4)
				call computeObservations5f(globPar,stars(i),gamext,radext,lambda5)	
			case (5)
				call computeObservationsBH(globPar,stars(i),gamext,radext)				
			case (6)
  			call computeObservationsKeplerAdvance(globPar,stars(i))
			case default
				write(*,*) ' ERROR: wrong type of model (computePartials): ', model
				stop
		end select
			write(*,'(A)')'*'
	end do ! i	
	write(*,*) "    PARTIALS COMPUTED  "	
	write(*,*) " ==================================== "

end subroutine computePartials
end module modelInterface