module relativisticMod
  use kindtype
  use starsMod, only: star
  use constantsMod, only : pi,solvekepler
	implicit none
	public :: computeObservationsRelativistic, getNumParamRelativistic, getNameParamRelativistic
	private :: createRelativistic,computenext,astro,rv,orbplane,eq_motion
	
	integer(id), private,parameter :: nglob=10
	integer(id), private,parameter :: nloc=6
	integer(id), private,parameter :: ntot=nloc+nglob	
	integer, private, parameter     :: nparint=7 ! 4 ci + GM +GM ext+ GR
	integer, private, parameter    :: neq=4*(nparint+1)

	real(rd), private, parameter   :: torigin=2000._rd
	real(rd), private, parameter   :: tstart=1990._rd
	
		
	character(len=5), dimension(nglob), parameter, private :: nameGlob=&
				&(/"GM   ","x0   ","y0   ","vx0  ","vy0  ","vz0  ","R0   ","Ext. ","Red. ","Rel. "/)
	character(len=5), dimension(nloc), parameter, private :: nameLoc=&
				&(/"P    ","T0   ","e    ","i    ","om   ","Om   "/)
	
	type, private :: orb
		real(rd), private      :: a
		real(rd), private      :: T
		real(rd), private      :: t0
		real(rd), private      :: e
		real(rd), private      :: w
		real(rd), private      :: i
		real(rd), private      :: o
		real(rd), private      :: n	

		real(rd), private      :: Arot
		real(rd), private      :: Brot
		real(rd), private      :: Crot

		real(rd), private      :: Frot
		real(rd), private      :: Grot
		real(rd), private      :: Hrot

		real(rd), private      :: se

		real(rd), private      :: GM
		real(rd), private      :: x0
		real(rd), private      :: y0
		real(rd), private      :: vx0
		real(rd), private      :: vy0
		real(rd), private      :: vz0
		real(rd), private      :: R0
		real(rd), private      :: GRRedshift
		real(rd), private      :: GMExt
		real(rd), private      :: GR
		real(rd), private      :: gammaext
		real(rd), private      :: R0ext
		real(rd), private      :: Bext		
	end type orb
	
	real(rd), private, save :: GM
	real(rd), private, save :: Bext
	real(rd), private, save :: gammaext
	real(rd), private, save :: GR
	real(rd), private, save :: r03g


contains


subroutine createRelativistic(parG,parL,o,gext,rext)
	use constantsMod, only: GMs_yr2_au3, pi
	real(rd), dimension(nglob), intent(in) :: parG
	real(rd), dimension(nloc), intent(in)  :: parL	
	type(orb), intent(out)                 :: o
	real(rd), intent(in)  :: rext
	real(rd), intent(in)  :: gext	
	!local paramaters
	o%t=parl(1)
	o%t0=parl(2)
	o%e=parl(3)
	o%i=parl(4)*pi/180.0_rd
	o%w=parl(5)*pi/180.0_rd
	o%o=parl(6)*pi/180.0_rd
	
	! global parameters
	o%GM=parg(1)   ! in 10^6 GM_sun 
	o%x0=parg(2)   ! in mas 
	o%y0=parg(3)   ! in mas 
	o%vx0=parg(4)  ! in mas /yr
	o%vy0=parg(5)	 ! in mas / yr
	o%vz0=parg(6)  ! in km/s
	o%R0=parg(7)   ! in kpc 
	o%GMExt=parg(8)   ! in 10^6 GM_sun 
	o%GRRedshift=parg(9)
	o%GR=parg(10)

	o%R0ext=rext*648000000._rd/pi ! from pc to AU
	o%gammaext=gext
	o%Bext=o%GMExt/o%R0ext**(3.-o%gammaext)
	
	
	
	o%n=2._rd*pi/o%T
	o%se=sqrt(1._rd-o%e**2)
	o%a=(o%GM*1.e6_rd/o%n**2*GMs_yr2_au3)**(1._rd/3._rd)! in AU

	o%Arot =  cos(o%w)*cos(o%O) - sin(o%w)*sin(o%O)*cos(o%i)!
	o%Brot =  cos(o%w)*sin(o%O) + sin(o%w)*cos(o%O)*cos(o%i)
	
	o%Frot = -sin(o%w)*cos(o%O) - cos(o%w)*sin(o%O)*cos(o%i)
	o%Grot = -sin(o%w)*sin(o%O) + cos(o%w)*cos(o%O)*cos(o%i)

	o%Crot = sin(o%w)*sin(o%i)
	o%Hrot = cos(o%w)*sin(o%i)	
	
end subroutine createRelativistic



subroutine computeObservationsRelativistic(pg,s,gext,rext)
	use constantsMod, only: sigNext	, timenext

	real(rd), dimension(nglob), intent(in) :: pg	
	type(star), intent(inout)  :: s
	real(rd), intent(in)  :: rext
	real(rd), intent(in)  :: gext
	type(orb) :: o
	integer(id) :: i

	!real(rd) :: x,y,vx,vy
	!real(rd), dimension(ntot) :: dxdp,dydp,dvxdp,dvydp
	
	call createrelativistic(pg,s%par,o,gext,rext)

	if (s%astrob) then
		write(*,'(A)',advance='no') "*"
		call astro(o,s)
	end if

	if (s%rvb) then
		write(*,'(A)',advance='no') "*"
		call rv(o,s)
	end if
  
  if (signext) then
    call computeNext(o,s)
		if (s%astrob) then
    	s%diffastronext(1)=s%ranext-s%ra(s%na);			s%diffastronext(2)=s%decnext-s%dec(s%na)
		else
			s%diffastronext(1)=0._rd; s%diffastronext(2)=0._rd
		end if
		if (s%rvb) then
    	s%diffRVnext = s%rvnext-s%rv(s%nrv)
		else
			s%diffRVnext = 0._rd
		end if
  end if
end subroutine computeObservationsRelativistic

subroutine computeNext(o,s)
  use odemod,only : integ, setupstart
	use constantsMod, only: GMs_yr2_au3, AUyr2kms,c,timenext
	type(orb), intent(in) :: o
	type(star), intent(inout) :: s
	real(rd)  :: x,y,vx,vy,r,v2,SR_redshift,GR_redshift
	real(rd), dimension(ntot)   :: dxdp,dydp,dvxdp,dvydp,drdp,dv2dp,dsrdp,dgrdp
	
	real(rd)  :: x0,y0,vx0,vy0
	real(rd), dimension(ntot)   :: dx0dp,dy0dp,dvx0dp,dvy0dp
	real(rd), dimension(4*(nparint+1)) :: ci, yint
!	integer(id) :: i,j
	
	! set constant variables
	GM=o%GM*1.e6_rd*GMs_yr2_au3
	Bext=o%bext*1.e6_rd*GMs_yr2_au3
	GR=o%GR
	gammaext=o%gammaext
	r03g=o%r0ext**(3-o%gammaext)
	!initical condition
	call orbPlane(o,tstart,x0,y0,dx0dp,dy0dp,vx0,vy0,dvx0dp,dvy0dp)
  ci=0._rd
  
	ci(1)=x0;ci(2)=y0;ci(3)=vx0;ci(4)=vy0;
	
	ci(5)=1._rd; ci(8)=1._rd;
	ci(2*nparint+9)=1._rd; ci(2*nparint+12)=1._rd;
	
	
	call setupstart(typeO=4,NE=4*(nparint+1),TSTART=tstart,YSTART=ci,TEND=timenext,ToLp=2.e-13_rd,THRESH1=1.e-20_rd)

	call integ(eq_motion,timenext,yint)!inte

	x=yint(1)
  y=yint(2)
  vx = yint(3)
  vy = yint(4)
  
  dxdp= yint(5)*dx0dp + yint(7)*dy0dp + yint(9) *dvx0dp + yint(11)*dvy0dp
  dydp= yint(6)*dx0dp + yint(8)*dy0dp + yint(10)*dvx0dp + yint(12)*dvy0dp
  dvxdp = yint(19)*dx0dp + yint(21)*dy0dp + yint(23) *dvx0dp + yint(25)*dvy0dp
  dvydp = yint(20)*dx0dp + yint(22)*dy0dp + yint(24)*dvx0dp + yint(26)*dvy0dp
    
    !der wrt to GM
    dxdp(1) = dxdp(1) + yint(13)
    dydp(1) = dydp(1) + yint(14)
    dvxdp(1) = dvxdp(1) + yint(27)
    dvydp(1) = dvydp(1) + yint(28)
    
    !der wrt to Ext. mass
    dxdp(8) = dxdp(8) + yint(15)
    dydp(8) = dydp(8) + yint(16)
    dvxdp(8) = dvxdp(8) + yint(29)
    dvydp(8) = dvydp(8) + yint(30)
    
    !der wrt to GR
    dxdp(10) = dxdp(10) + yint(17)
    dydp(10) = dydp(10) + yint(18)
	  dvxdp(10) = dvxdp(10) + yint(31)
    dvydp(10) = dvydp(10) + yint(32)
  
  !conversion
  s%ranext = x * o%Brot + y * o%Grot
	s%dradpnext = dxdp * o%Brot + dydp * o%Grot


	s%dradpnext(nglob+4) = s%dradpnext(nglob+4) - x*cos(o%o)*sin(o%i)*sin(o%w) - y*cos(o%o)*sin(o%i)*cos(o%w) ! i
	s%dradpnext(nglob+5) = s%dradpnext(nglob+5) + x* (-sin(o%w)*sin(o%o) + cos(o%o)*cos(o%i)*cos(o%w))+ &
                              &	    y* (-sin(o%o)*cos(o%w) - cos(o%o)*cos(o%i)*sin(o%w)) ! omega 
	s%dradpnext(nglob+6) = s%dradpnext(nglob+6)	+ x* (cos(o%w)*cos(o%o) -sin(o%o)*cos(o%i)*sin(o%w)) + &
															&     y* (-cos(o%o)*sin(o%w)-sin(o%o)*cos(o%i)*cos(o%w))! Omega

	s%ranext = s%ranext/o%R0
	s%dradpnext(:)=s%dradpnext(:)/o%R0
	s%dradpnext(7) = s%dradpnext(7) - s%ranext/o%r0

	s%ranext=s%ranext - o%x0 - o%vx0*(timenext-torigin)

	s%dradpnext(2)=s%dradpnext(2)-1._rd
	s%dradpnext(4)=s%dradpnext(4)-(timenext-torigin)


	s%decnext   = x* o%Arot + y * o%Frot
	s%ddecdpnext(:) = dxdp* o%Arot + dydp * o%Frot
	s%ddecdpnext(nglob+4) = s%ddecdpnext(nglob+4) + x*sin(o%o)*sin(o%i)*sin(o%w) + y*sin(o%o)*sin(o%i)*cos(o%w) ! i
	s%ddecdpnext(nglob+5) = s%ddecdpnext(nglob+5) + x* (-sin(o%w)*cos(o%o)-sin(o%o)*cos(o%i)*cos(o%w)) + &
	    											&					y* (-cos(o%o)*cos(o%w)+sin(o%o)*cos(o%i)*sin(o%w)) ! omega
	s%ddecdpnext(nglob+6) = s%ddecdpnext(nglob+6) + x* (-cos(o%w)*sin(o%o)-cos(o%o)*cos(o%i)*sin(o%w)) + &
														& 	      y* (sin(o%o)*sin(o%w)-cos(o%o)*cos(o%i)*cos(o%w))!Omega



	s%decnext = s%decnext/o%r0
	s%ddecdpnext(:) = s%ddecdpnext(:)/o%r0
	s%ddecdpnext(7) = s%ddecdpnext(7) - s%decnext/o%r0

	s%decnext = s%decnext + o%y0 + o%vy0*(timenext-torigin)
	s%ddecdpnext(3)=	s%ddecdpnext(3) + 1._rd
	s%ddecdpnext(5)=	s%ddecdpnext(5) + (timenext-torigin)	


	s%dradpnext(nglob+4:nglob+6)=s%dradpnext(nglob+4:nglob+6)*pi/180._rd
	s%ddecdpnext(nglob+4:nglob+6)=s%ddecdpnext(nglob+4:nglob+6)*pi/180._rd
	  
    s%rvnext = (vx * o%Crot + vy * o%Hrot)*AUyr2kms + o%vz0

	  s%drvdpnext(:) = dvxdp * o%Crot + dvydp * o%Hrot
	  s%drvdpnext(nglob+4)=s%drvdpnext(nglob+4)+cos(o%i)*sin(o%w)*vx+cos(o%i)*cos(o%w)*vy ! i
	  s%drvdpnext(nglob+5)=s%drvdpnext(nglob+5)+sin(o%i)*cos(o%w)*vx-sin(o%i)*sin(o%w)*vy ! omega
	
	  s%drvdpnext(:)=s%drvdpnext(:)*AUyr2kms
	  s%drvdpnext(6)=s%drvdpnext(6)+1._rd
	  
	
	  ! RELATIVITY
	  r    = sqrt(x**2+y**2)
	  v2   = vx**2+vy**2
	  SR_redshift = 0.5_rd*v2/c*AUyr2kms
	  GR_redshift = o%GM*1.e6_rd/r/c *GMs_yr2_au3*AUyr2kms
	
	  drdp = (x*dxdp + y * dydp)/r
	  dv2dp = 2._rd*(vx*dvxdp + vy*dvydp)

	  dSRdp = 0.5_rd*dv2dp/c*AUyr2kms
	  dGRdp = -GR_redshift*drdp/r
	  dgrdp(1) =	dgrdp(1) + GR_redshift/o%GM ! der wrt to GM

	  s%rvnext = s%rvnext + o%GRRedshift *(sr_redshift+gr_redshift)
	  s%drvdpnext(:) = s%drvdpnext(:) + o%GRRedshift *(dsrdp+dgrdp)
	  s%drvdpnext(9) = 	s%drvdpnext(9) + sr_redshift+gr_redshift
	  
	  s%drvdpnext(nglob+4:nglob+6)=s%drvdpnext(nglob+4:nglob+6)*pi/180._rd
end subroutine computeNext


subroutine rv(o,s)
  	use odemod,only : integ, setupstart
	use constantsMod, only: GMs_yr2_au3, AUyr2kms,c
	type(orb), intent(in) :: o
	type(star), intent(inout) :: s
	real(rd)  :: x,y,vx,vy,r,v2,SR_redshift,GR_redshift
	real(rd), dimension(ntot)   :: dxdp,dydp,dvxdp,dvydp,drdp,dv2dp,dsrdp,dgrdp
	
	real(rd)  :: x0,y0,vx0,vy0
	real(rd), dimension(ntot)   :: dx0dp,dy0dp,dvx0dp,dvy0dp
	real(rd), dimension(4*(nparint+1)) :: ci, yint
	integer(id) :: i,j

	
	! set constant variables
	GM=o%GM*1.e6_rd*GMs_yr2_au3
	Bext=o%bext*1.e6_rd*GMs_yr2_au3
	GR=o%GR
	gammaext=o%gammaext
	r03g=o%r0ext**(3-o%gammaext)
	!initical condition
	call orbPlane(o,tstart,x0,y0,dx0dp,dy0dp,vx0,vy0,dvx0dp,dvy0dp)
  ci=0._rd
  
	ci(1)=x0;ci(2)=y0;ci(3)=vx0;ci(4)=vy0;
	
	ci(5)=1._rd; ci(8)=1._rd;
	ci(2*nparint+9)=1._rd; ci(2*nparint+12)=1._rd;
	
	
	call setupstart(typeO=4,NE=4*(nparint+1),TSTART=tstart,YSTART=ci,TEND=maxval(s%trv),ToLp=2.e-13_rd,THRESH1=1.e-20_rd)

	s%drvdp=0._rd


	do i=1,s%nrv
	  call integ(eq_motion,s%trv(i),yint)!integration

    x  = yint(1)
    y  = yint(2)
    vx = yint(3)
    vy = yint(4)
    
    dxdp  = yint(5)*dx0dp + yint(7)*dy0dp + yint(9) *dvx0dp + yint(11)*dvy0dp
    dydp  = yint(6)*dx0dp + yint(8)*dy0dp + yint(10)*dvx0dp + yint(12)*dvy0dp
    dvxdp = yint(19)*dx0dp + yint(21)*dy0dp + yint(23) *dvx0dp + yint(25)*dvy0dp
    dvydp = yint(20)*dx0dp + yint(22)*dy0dp + yint(24)*dvx0dp + yint(26)*dvy0dp
    
    
    !der wrt to GM
    dxdp(1) = dxdp(1) + yint(13)
    dydp(1) = dydp(1) + yint(14)
    dvxdp(1) = dvxdp(1) + yint(27)
    dvydp(1) = dvydp(1) + yint(28)
    
    !der wrt to Ext. mass
    dxdp(8) = dxdp(8) + yint(15)
    dydp(8) = dydp(8) + yint(16)
    dvxdp(8) = dvxdp(8) + yint(29)
    dvydp(8) = dvydp(8) + yint(30)
    
    !der wrt to GR
    dxdp(10) = dxdp(10) + yint(17)
    dydp(10) = dydp(10) + yint(18)
	  dvxdp(10) = dvxdp(10) + yint(31)
    dvydp(10) = dvydp(10) + yint(32)
    
	s%rv(i) = (vx * o%Crot + vy * o%Hrot)*AUyr2kms + o%vz0

	  s%drvdp(i,:) = dvxdp * o%Crot + dvydp * o%Hrot
	  s%drvdp(i,nglob+4)=s%drvdp(i,nglob+4)+cos(o%i)*sin(o%w)*vx+cos(o%i)*cos(o%w)*vy ! i
	  s%drvdp(i,nglob+5)=s%drvdp(i,nglob+5)+sin(o%i)*cos(o%w)*vx-sin(o%i)*sin(o%w)*vy ! omega
	
	  s%drvdp(i,:)=s%drvdp(i,:)*AUyr2kms
	  s%drvdp(i,6)=s%drvdp(i,6)+1._rd
	  
!	write(*,*) i,s%trv(i), s%drvdp(i,:)
	  ! LATIVITY
	  r    = sqrt(x**2+y**2)
	  v2   = vx**2+vy**2
	  SR_redshift = 0.5_rd*v2/c*AUyr2kms
	  GR_redshift = o%GM*1.e6_rd/r/c *GMs_yr2_au3*AUyr2kms
	
	  drdp = (x*dxdp + y * dydp)/r
	  dv2dp = 2._rd*(vx*dvxdp + vy*dvydp)

	  dSRdp = 0.5_rd*dv2dp/c*AUyr2kms
	  dGRdp = -GR_redshift*drdp/r
	  dgrdp(1) =	dgrdp(1) + gr_redshift/o%GM ! der wrt to GM

	  s%rv(i) = s%rv(i) + o%GRRedshift *(sr_redshift+gr_redshift)
	  s%drvdp(i,:) = s%drvdp(i,:) + o%GRRedshift *(dsrdp+dgrdp)
	  s%drvdp(i,9) = 	s%drvdp(i,9) + sr_redshift+gr_redshift
	  
	  s%drvdp(i,nglob+4:nglob+6)=s%drvdp(i,nglob+4:nglob+6)*pi/180._rd
	  
  end do!i


end subroutine rv

subroutine astro(o,s)
  	use odemod,only : integ, setupstart
  	use constantsmod, only: GMs_yr2_au3
	type(orb), intent(in) :: o
	type(star), intent(inout) :: s

	
	real(rd)  :: x0,y0,vx0,vy0, x,y
	real(rd), dimension(ntot)   :: dx0dp,dy0dp,dvx0dp,dvy0dp
	real(rd), dimension(ntot)   :: dxdp,dydp,dvxdp,dvydp
	real(rd), dimension(4*(nparint+1)) :: ci, yint
	integer(id) :: i,j
	
	! set constant variables
	GM=o%GM*1.e6_rd*GMs_yr2_au3
	Bext=o%bext*1.e6_rd*GMs_yr2_au3
	GR=o%GR
	gammaext=o%gammaext
	r03g=o%r0ext**(3-o%gammaext)
	!initical condition
	call orbPlane(o,tstart,x0,y0,dx0dp,dy0dp,vx0,vy0,dvx0dp,dvy0dp)
  ci=0._rd
  
	ci(1)=x0;ci(2)=y0;ci(3)=vx0;ci(4)=vy0;
	
	ci(5)=1._rd; ci(8)=1._rd;
	ci(2*nparint+9)=1._rd; ci(2*nparint+12)=1._rd;
	
	
	call setupstart(typeO=4,NE=4*(nparint+1),TSTART=tstart,YSTART=ci,TEND=maxval(s%tastro),ToLp=2.e-13_rd,THRESH1=1.e-20_rd)

	s%dradp=0._rd
	s%ddecdp=0._rd
	      	      
	do i=1,s%na

	  call integ(eq_motion,s%tastro(i),yint)!integration
  
    x=yint(1)
    y=yint(2)
    dxdp= yint(5)*dx0dp + yint(7)*dy0dp + yint(9) *dvx0dp + yint(11)*dvy0dp
    dydp= yint(6)*dx0dp + yint(8)*dy0dp + yint(10)*dvx0dp + yint(12)*dvy0dp
    
    !der wrt to GM
    dxdp(1) = dxdp(1) + yint(13)
    dydp(1) = dydp(1) + yint(14)

    !der wrt to Ext. mass
    dxdp(8) = dxdp(8) + yint(15)
    dydp(8) = dydp(8) + yint(16)
    
    !der wrt to GR
    dxdp(10) = dxdp(10) + yint(17)
    dydp(10) = dydp(10) + yint(18)
    
    !conversion
    s%ra(i) = x * o%Brot + y * o%Grot
  	s%dradp(i,:) = dxdp * o%Brot + dydp * o%Grot


  	s%dradp(i,nglob+4) = s%dradp(i,nglob+4) - x*cos(o%o)*sin(o%i)*sin(o%w) - y*cos(o%o)*sin(o%i)*cos(o%w) ! i
  	s%dradp(i,nglob+5) = s%dradp(i,nglob+5) + x* (-sin(o%w)*sin(o%o) + cos(o%o)*cos(o%i)*cos(o%w))+ &
                                &	    y* (-sin(o%o)*cos(o%w) - cos(o%o)*cos(o%i)*sin(o%w)) ! omega 
  	s%dradp(i,nglob+6) = s%dradp(i,nglob+6)	+ x* (cos(o%w)*cos(o%o) -sin(o%o)*cos(o%i)*sin(o%w)) + &
  															&     y* (-cos(o%o)*sin(o%w)-sin(o%o)*cos(o%i)*cos(o%w))! Omega

  	s%ra(i) = s%ra(i)/o%R0
  	s%dradp(i,:)=s%dradp(i,:)/o%R0
  	s%dradp(i,7) = s%dradp(i,7) - s%ra(i)/o%r0

  	s%ra(i)=s%ra(i) - o%x0 - o%vx0*(s%tastro(i)-torigin)

  	s%dradp(i,2)=s%dradp(i,2)-1._rd
  	s%dradp(i,4)=s%dradp(i,4)-(s%tastro(i)-torigin)


  	s%dec(i)    = x* o%Arot + y * o%Frot
  	s%ddecdp(i,:) = dxdp* o%Arot + dydp * o%Frot
  	s%ddecdp(i,nglob+4) = s%ddecdp(i,nglob+4) + x*sin(o%o)*sin(o%i)*sin(o%w) + y*sin(o%o)*sin(o%i)*cos(o%w) ! i
  	s%ddecdp(i,nglob+5) = s%ddecdp(i,nglob+5) + x* (-sin(o%w)*cos(o%o)-sin(o%o)*cos(o%i)*cos(o%w)) + &
  	    											&					y* (-cos(o%o)*cos(o%w)+sin(o%o)*cos(o%i)*sin(o%w)) ! omega
  	s%ddecdp(i,nglob+6) = s%ddecdp(i,nglob+6) + x* (-cos(o%w)*sin(o%o)-cos(o%o)*cos(o%i)*sin(o%w)) + &
  														& 	      y* (sin(o%o)*sin(o%w)-cos(o%o)*cos(o%i)*cos(o%w))!Omega



  	s%dec(i) = s%dec(i)/o%r0
  	s%ddecdp(i,:) = s%ddecdp(i,:)/o%r0
  	s%ddecdp(i,7) = s%ddecdp(i,7) - s%dec(i)/o%r0

  	s%dec(i) = s%dec(i) + o%y0 + o%vy0*(s%tastro(i)-torigin)
  	s%ddecdp(i,3)=	s%ddecdp(i,3) + 1._rd
  	s%ddecdp(i,5)=	s%ddecdp(i,5) + (s%tastro(i)-torigin)	


  	s%dradp(i,nglob+4:nglob+6)=s%dradp(i,nglob+4:nglob+6)*pi/180._rd
  	s%ddecdp(i,nglob+4:nglob+6)=s%ddecdp(i,nglob+4:nglob+6)*pi/180._rd
  	
!write(*,*)i,s%tastro(i), s%dradp(i,:)
  end do!i
!stop
end subroutine astro

subroutine orbPlane(o,t,x,y,dxdp,dydp,vx,vy,dvxdp,dvydp)
	type(orb), intent(in) :: o
	real(rd), intent(in)  :: t
	real(rd), intent(out) :: x,y,vx,vy
	real(rd), intent(out), dimension(ntot) :: dxdp,dydp,dvxdp,dvydp

	real(rd), dimension(ntot) :: dadp, dEdp,dndp,dmdp,dtmpvdp
	real(rd) :: m,E,sE,cE,tmpv
	
	dadp(:)=0._rd;	dEdp(:)=0._rd;	dndp(:)=0._rd;

  dndp(nglob+1)=-o%n/o%t
	
	dadp=-2._rd*o%a / (3._rd*o%n) *dndp
	dadp(1)=dadp(1)+ o%a/ o%GM / 3._rd
	
	m=o%n*(t-o%t0)
	

	dmdp=dndp*(t-o%t0)
	dmdp(nglob+2) =	dmdp(nglob+2) - o%n
	call solveKepler(m,o%e,E)
	sE=sin(E);cE=cos(E)
	
		
	dEdp=dmdp
	dEdp(nglob+3) = dEdp(nglob+3) + sE !der wrt to e
	dEdp=dEdp/(1._rd-o%e*cE)

	x = o%a*(ce-o%e) 
	y = o%a*o%se*se

	dxdp = (cE-o%e)*dadp -o%a*se*dedp
	dxdp(nglob+3) = dxdp(nglob+3) - o%a! der wrt e
	
	dydp = y/o%a *dadp + o%a*o%se*cE*dEdp
	dydp(nglob+3) = dydp(nglob+3) - o%a*o%e*sE/o%se
	
	tmpv = o%n*o%a/(1-o%e*ce)
	dtmpvdp = (dndp*o%a + o%n*dadp)/(1._rd-o%e*ce) - o%n*o%a*o%e*sE/(1._rd-o%e*cE)**2*dEdp
	dtmpvdp(nglob+3) = dtmpvdp(nglob+3) + o%n*o%a*cE/(1._rd-o%e*cE)**2   !der wrt e
	
	vx = -tmpv*se
	vy = o%se*tmpv*ce

	dvxdp = -dtmpvdp*se - tmpv*ce*dEdp
	dvydp = dtmpvdp*o%se*ce - tmpv*o%se*se*dEdp
	dvydp(nglob+3)=dvydp(nglob+3) - tmpv*cE*o%e/o%se!der wrt e
	
end subroutine orbPlane


subroutine getNumParamRelativistic(ng,nl)
	integer(id), intent(out) :: ng,nl
	ng=nglob
	nl=nloc
end subroutine getNumParamRelativistic

subroutine getNameParamRelativistic(ng,nl)
	character(len=5), dimension(nglob), intent(out) :: ng
	character(len=5), dimension(nloc), intent(out) :: nl
	ng=nameglob
	nl=nameloc
end subroutine getNameParamRelativistic

subroutine eq_motion(t,x,f)
use constantsmod, only:c2,GMs_yr2_au3
  REAL(rd), DIMENSION(neq), INTENT(IN):: X
  REAL(rd), INTENT(IN):: T
  REAL(rd), DIMENSION(neq), INTENT(OUT):: F
  real(rd), dimension(2,2) :: A0,A1
  
  integer :: i,j
  real(rd) :: r,rv,v2, tmp1
  f(1:2)=x(3:4)
  
  r=sqrt(x(1)*x(1)+x(2)*x(2))
  v2=x(3)*x(3)+x(4)*x(4)
  rv=x(1)*x(3)+x(2)*x(4)
  
  
  f(3:4)=-GM*x(1:2)/r**3 & ! Newton
    & - Bext * x(1:2)*r**(-gammaext)   &! Extended mass
    & + GR*c2*GM/r**3*x(1:2)* (4._rd*GM/r-v2) + 4._rd*GR*c2*GM*x(3:4)*rv/r**3
    

! ========= PARTIALS
  f(5:5+2*nparint-1)=x(5+2*nparint:neq)

  A0=0._rd;A1=0._rd
  tmp1=3._rd*GM/r**5 + gammaext*Bext*r**(-2._rd-gammaext) - 3._rd*GR*GM*c2/r**5* (4._rd*GM/r-v2)- GR*4._rd*GM*c2*GM/r**6
  do i=1,2
    do j=1,2
      A0(i,j)=A0(i,j)+tmp1*x(i)*x(j) - 12._rd*GR*c2*GM*x(j)*x(2+i)*rv/r**5 + 4._rd*GR*c2*GM*x(2+i)*x(2+j)/r**3 
      A1(i,j)=A1(i,j)-2._rd*GR*GM*c2/r**3*x(i)*x(j+2) + 4._rd*GR*GM*c2/r**3*x(j)*x(i+2)
      if (i==j) then
        a0(i,i)=a0(i,i)-GM/r**3 -Bext*r**(-gammaext)+GR*GM*c2/r**3* (4._rd*GM/r-v2)
        A1(i,j)=A1(i,j)+4._rd*GR*GM*c2*rv/r**3
      end if
    end do !j
  end do!i
  
  do i=1,nparint
    f(5+2*nparint+2*(i-1):5+2*nparint+2*i-1)= matmul(A0,x(5+2*i-2:5+2*i-1))  + matmul(A1,x(5+2*nparint+2*(i-1):5+2*nparint+2*i-1))


    if (i==5) then ! GM
      f(5+2*nparint+2*(i-1):5+2*nparint+2*i-1)=f(5+2*nparint+2*(i-1):5+2*nparint+2*i-1)-x(1:2)*1.e6_rd*GMs_yr2_au3/r**3+ &
                  &  GR*c2*1.e6_rd*GMs_yr2_au3/r**3*x(1:2)* (8._rd*GM/r-v2) + 4._rd*GR*c2*1.e6_rd*GMs_yr2_au3*x(3:4)*rv/r**3
    end if
    
    if (i==6) then ! extended mass
      f(5+2*nparint+2*(i-1):5+2*nparint+2*i-1)=f(5+2*nparint+2*(i-1):5+2*nparint+2*i-1)&
              &-x(1:2)*r**(-gammaext)*1.e6_rd*GMs_yr2_au3/r03g
    end if!i==6
    
    if (i==7) then ! GR
      f(5+2*nparint+2*(i-1):5+2*nparint+2*i-1)=f(5+2*nparint+2*(i-1):5+2*nparint+2*i-1)+GM/r**3*c2*(4._rd*GM/r-v2)*x(1:2)&
            & +4._rd*GM/r**3*c2*rv*x(3:4)
    end if
  end do!i

end subroutine eq_motion
end module relativisticMod