module sensitivityMod
  use kindtype
	use starsMod, only: nstars, star, stars
  implicit none

	public  :: initSensitivity,itersensitivity
	private :: load_Star_include,writeNoChanges,writeAllUncertainties,writeResults


	integer, private, save                                 :: fit_type =0 !< 0 = inversion of the full matrix ; 1 partial inversion using the block structure.
	

	real(rd), private, save :: increment
	real(rd), private, save :: starting_epoch	
	real(rd), private, save :: end_epoch	
  
  integer(id), private, save                     :: n_stars_in
		
	integer(id),private, save     :: Niteration     !< number of time periods

	integer(id), parameter, private :: fout=56, foutloc=57, foutglob=58, foutgloball=59, foutstarnames=61
	real(rd), dimension(:,:), allocatable :: cov
	real(rd), dimension(:,:), allocatable :: uinv
	
	character(len=200), save :: dir_res
contains


subroutine  initSensitivity(nameIn)
  use constantsMod, only: dirout, name,signext,timenext,priors,npriors,tofitglobal,nparamglob,nparamloc, nparam
  use modelInterface, only: setModel, computePartials
!	use odemod, only:setup
	implicit none
	character(len=*), intent(in) :: nameIn
	integer :: fittmp,nohairtmp
	integer(id) :: i,j,k
	integer(id) :: itmp,istar
	logical(lgt) :: exist
	character(len=15) :: modelStr
	
	!
	!Reading the file
	!
	inquire(file=nameIn, EXIST=exist) 
	if (.NOT.exist) then
		WRITE(*,*) "ERROR : File ", trim(nameIn)," does not exist"
		STOP
	endif
	open(unit=55,file=nameIn,action='read')
	
	read(55,'(A)') dirout
#ifdef IFORT	 
	inquire(directory=trim(dirout),exist=exist)
#else	
	inquire(file=trim(dirout),exist=exist)
#endif	

	if (.not.exist) then
		write(*,*) 'Creating folder : ', trim(dirout)
		call system(' mkdir '//trim(dirout))
	end if
	!write(*,*) exist,trim(dirout)
	
	!===== 2: name of the file	
	read(55,'(A)') name	
	write(*,*) 'File:', name
	
	dir_res = trim(dirout)//'/'//trim(name)//'_res/'
#ifdef IFORT	 
  	inquire(directory=trim(dir_res),exist=exist)
#else	
  	inquire(file=trim(dir_res),exist=exist)
#endif
  if (.not.exist) then
		write(*,*) 'Creating folder : ', trim(dir_res)
		call system(' mkdir '//trim(dir_res))
	end if
  
	read(55,'(i1)') fit_type
	read(55,*) starting_epoch, end_epoch,increment
	
    niteration = ceiling((end_epoch - starting_epoch)/increment)
	
	read(55,'(A)') modelStr
	
	read(55,*) npriors
	if (npriors>0) then
		allocate(priors(npriors))
		do i=1,npriors
			read(55,*)priors(i)%idx,priors(i)%sig
		end do !i
	end if!n priors>0
	
	
	call setModel(modelstr,55)
	read(55,*) nstars
	nparam=nparamglob+nstars*nparamloc
	
	if (nstars<1) then
		write(*,*) "ERROR : number of stars should be at least 1 and it is: ",nstars
		stop
	end if
	if (npriors>0) then
		do i=1,npriors
			if (priors(i)%idx>nparam) then
				write(*,*) "ERROR: index of prior ", i, " too large: ",priors(i)%idx
				stop
			end if
			if (priors(i)%sig<=0._rd) then
				write(*,*)"ERROR: non positive sigma for prior ", i
				stop
			end if
		end do!i
	end if!npriors
	
	if (allocated(stars)) deallocate(stars)
	allocate(stars(nstars))

	write(*,*)" ============ LOADING STARS ======= "
	
	do i=1, nstars
		allocate(stars(i)%par(nparamloc))
		allocate(stars(i)%tofit(nparamloc))
		
		read(55,*)stars(i)%name
		read(55,'(A)')stars(i)%astroFile
		read(55,'(A)')stars(i)%rvFile
		stars(i)%ntofit=0
		do j=1,nparamloc
			read(55,*) stars(i)%par(j),fittmp
			stars(i)%tofit(j)=(fittmp==1)
			if (stars(i)%tofit(j))  stars(i)%ntofit=stars(i)%ntofit+1
		end do!j
		call load_Star_include(stars(i))
		allocate(stars(i)%idx_fit(stars(i)%ntofit))
    k=0
    do j=1,nparamloc
      	if (stars(i)%tofit(j)) then
      	  k=k+1
      	  stars(i)%idx_fit(k)=j
    	  end if
    end do!j
	end do! i
	close(55)
	write(*,*)" ============ STARS LOADED ======= "
	call writeinput()
	call computePartials()
end subroutine  initSensitivity

subroutine load_Star_include(s)
	use constantsMod, only : countLines,nparamloc,nparamglob,fromMJD2year
	use starsMod, only: loadstar,star_included
	type(star), intent(inout) :: s
	real(rd),dimension(2) :: tmp
	real(rd) :: tmpyr
	integer(id):: nmean
	integer(id) ::i

	call loadstar(s)
	
  if (s%astrob) then
    s%npastastro = maxloc(s%tastro,1,mask=s%tastro.le.starting_epoch)
		s%dnastro = maxloc(s%tastro,1,mask=s%tastro.le.end_epoch)- s%npastastro
  else
    s%npastastro=0
    s%dnastro = 0    
  end if
  
  if (s%rvb) then
    s%npastrv = maxloc(s%trv,1,mask=s%trv.le.starting_epoch)
		s%dnrv = maxloc(s%trv,1,mask=s%trv.le.end_epoch) - s%npastrv
  else
    s%dnrv = 0		
    s%npastrv =0
  end if

  s%included = star_included(s)
  if (s%included)     n_stars_in=n_stars_in+1
end subroutine load_Star_include

subroutine iterSensitivity()
  if (fit_type==0) then
    call iterSensitivity_fullfit
  else if (fit_type==1) then
    call iterSensitivity_block
  else
    write(*,*) "Value of fit_type parameter (line 3 from input file) not correct"
    stop
  end if
end subroutine iterSensitivity



subroutine iterSensitivity_block()
	use fitMod, only: inverse_SVD
	use constantsMod!,only: nstars,tofitglobal,nparamGlob,nparamLoc,nameglob,nameloc,fromJY2MJD
	use starsMod, only: init_block_star, inverse_star, upgrade_block,star_included
	integer(id)     :: i, j, k, dastro,drv, lastFit, kstart
	real(rd)    :: t
	logical, dimension(2) :: parmask =.true.


	real(rd), dimension(:,:), allocatable    ::  U
	
	logical :: changed
	
	real(rd), dimension(nglobtofit,nglobtofit) ::tmp,tmp4
real(rd), dimension(6,6) ::tmp2
real(rd), dimension(nglobtofit,6) ::tmp3
    
	write(*,*)" Start of the iterations for the sensitivity"

  open(unit=foutgloball,file = trim(dirout)//'/'//trim(name)//'_all.sen',action='write',status='replace')
  open(unit=foutstarnames,file = trim(dirout)//'/'//trim(name)//'_stars.sen',action='write',status='replace')
  open(unit=foutglob,file = trim(dirout)//'/'//trim(name)//'.sen',action='write',status='replace')

  allocate(U(nglobtofit,nglobtofit),Uinv(nglobtofit,nglobtofit))
  U=0._rd
  ! initialization
  do i=1,nstars
    if(stars(i)%included) then
        call init_block_star(stars(i),past=.true.)
        U=U+stars(i)%ui
    end if
  end do!
  call inverse_SVD(U,Uinv)

  !call writeResults(0,starting_epoch,parmask,nglobtofit)!parmask and  is not used here
  call writeAllUncertainties(starting_epoch,foutglob)
  call writeAllUncertainties(starting_epoch,foutgloball)


  lastFit=0
  

  do j=1,niteration
    write(*,*) 'Iteration j=', j, ' out of ',niteration
    ! 1. compute the time threshold
    t = starting_epoch+j*increment
    if (i==niteration) t =end_epoch

    ! 2. number of stars "included" and number of stars changed + update of npastastro + dnastro, ...
    changed=.false.

    do i=1,nstars
      drv=0
      dastro=0
      if (stars(i)%dnastro>0) then
        dastro = maxloc(stars(i)%tastro(stars(i)%npastastro+1:),1,mask=stars(i)%tastro(stars(i)%npastastro+1:).le.t)
        if (dastro>0) then
          if (stars(i)%included) then
            kstart = stars(i)%npastastro+1
            do k=kstart,kstart+dastro-1
              call upgrade_block(stars(i),1,k)
            end do!j
          else  
            stars(i)%dnastro=stars(i)%dnastro-dastro
            stars(i)%npastastro=stars(i)%npastastro+dastro
          end if

        end if!dastro>0
      end if  ! dnastro(i) > 0

      if (stars(i)%dnrv>0) then
        drv = maxloc(stars(i)%trv(stars(i)%npastrv+1:),1,mask=stars(i)%trv(stars(i)%npastrv +1:).le.t)
        if (drv>0) then
          if (stars(i)%included) then
            kstart = stars(i)%npastrv+1
            do k=kstart,kstart+drv-1 
              call upgrade_block(stars(i),2,k)
            end  do! k
          else
              stars(i)%dnrv=stars(i)%dnrv-drv
              stars(i)%npastrv=stars(i)%npastrv+drv
          end if
        end if!drv>0
      end if  ! dnrv(j) > 0
      if (stars(i)%included) then
        if (drv+dastro>0) then
          changed=.true.
          !  upgrade invDD and Ui
          call inverse_star(stars(i),past=.true.)
         end if
         
      else
        if (star_included(stars(i))) then
          changed=.true.
          stars(i)%included =.true.
          n_stars_in=n_stars_in+1
          call init_block_star(stars(i),past=.true.)
        end if
      end if! stars_in
    end do!i
  
    
    if (changed) then
      lastFit=j
      U=0._rd
      do i=1,nstars
        if(stars(i)%included) U=U+stars(i)%Ui
      end do!i
      ! 3. Inversion of U
      call inverse_SVD(U,Uinv)
      
      !4. writing results
      !call writeResults(j,t,parmask,nglobtofit)
      call writeAllUncertainties(t,foutglob)
    else
      call writeNoChanges(j,t,lastFit)
    end if! changed
    
    call writeAllUncertainties(t,foutgloball)
    call write_starsused(foutstarnames,j,t)
  end do!i

 close(foutglob);close(foutgloball); close(foutstarnames)
end subroutine iterSensitivity_block



subroutine iterSensitivity_fullfit()
	use fitMod, only: fitsvd
	use constantsMod!,only: nstars,tofitglobal,nparamGlob,nparamLoc,nameglob,nameloc,fromJY2MJD
	use starsMod ,  only: star_included
	integer(id)     :: i, j, dastro, drv, nobs, npartofit,idx,istar,lastFit
	real(rd)    :: t, chi2
	real(rd), dimension(:,:), allocatable :: pder
	real(rd), dimension(:), allocatable   :: omc,sig,par
	logical, dimension(:), allocatable    :: parmask
	
	logical :: changed
	
	write(*,*)" Start of the iterations for the sensitivity"

 open(unit=foutgloball,file = trim(dirout)//'/'//trim(name)//'_all.sen',action='write',status='replace')
 open(unit=foutstarnames,file = trim(dirout)//'/'//trim(name)//'_stars.sen',action='write',status='replace')
 open(unit=foutglob,file = trim(dirout)//'/'//trim(name)//'.sen',action='write',status='replace')
 lastFit=0
 
 do j=0,niteration
   write(*,*) 'Iteration j=', j, ' out of ',niteration
   ! 1. compute the time threshold
   t = starting_epoch+j*increment
   if (i==niteration) t =end_epoch

   ! 2. number of stars "included" and number of stars changed + update of npastastro + dnastro, ...
   changed=.false.
   nobs = 0

   do i=1,nstars
     drv=0
     dastro=0
     if (stars(i)%dnastro>0) then
       dastro = maxloc(stars(i)%tastro(stars(i)%npastastro+1:),1,mask=stars(i)%tastro(stars(i)%npastastro+1:).le.t)
       if (dastro>0) then
         stars(i)%dnastro=stars(i)%dnastro-dastro
         stars(i)%npastastro=stars(i)%npastastro+dastro
       end if!dastro>0
     end if  ! dnastro(i) > 0

     if (stars(i)%dnrv>0) then
       drv = maxloc(stars(i)%trv(stars(i)%npastrv+1:),1,mask=stars(i)%trv(stars(i)%npastrv +1:).le.t)
       if (drv>0) then
         stars(i)%dnrv=stars(i)%dnrv-drv
         stars(i)%npastrv=stars(i)%npastrv+drv
       end if!drv>0
     end if  ! dnrv(j) > 0
     if (stars(i)%included) then
       if (drv+dastro>0) changed=.true.
       nobs=nobs+2*stars(i)%npastastro+stars(i)%npastrv
     else
       if (star_included(stars(i))) then
         changed=.true.
         stars(i)%included =.true.
         n_stars_in=n_stars_in+1
         nobs=nobs+2*stars(i)%npastastro+stars(i)%npastrv   
       end if
     end if! stars_in
   end do!i
   npartofit = nparamGlob + n_stars_in*nparamloc
   
   if ((changed).or.(j==0)) then
     lastFit=j
     ! 3. construction of the partial derivatives matrix
     if (allocated(cov)) deallocate(cov)
     allocate(pder(nobs+npriors,npartofit),omc(nobs+npriors),sig(nobs+npriors),par(npartofit), &
           &  parmask(npartofit),cov(npartofit,npartofit))
 	  pder=0._rd;omc=0.0_rd;par=0._rd
 	  parmask(1:nparamglob)=toFitGlobal
 	  idx=1
 	  istar=0
     do i=1,nstars
       
       if(stars(i)%included) then
         istar=istar+1
 		    parmask(nparamglob+1+(istar-1)*nparamloc:nparamglob+istar*nparamloc)=stars(i)%tofit

 		  if (stars(i)%astrob) then
 			  pder(idx:idx+stars(i)%npastastro-1,1:nparamglob)=stars(i)%dradp(1:stars(i)%npastastro,1:nparamglob)
 			  pder(idx:idx+stars(i)%npastastro-1,nparamglob+(istar-1)*nparamloc+1:nparamglob+istar*nparamloc) = &
 			            & stars(i)%dradp(1:stars(i)%npastastro,nparamglob+1:)
         sig(idx:idx+stars(i)%npastastro-1)=stars(i)%sigra(1:stars(i)%npastastro)
 			  idx=idx+stars(i)%npastastro

 			  pder(idx:idx+stars(i)%npastastro-1,1:nparamglob)=stars(i)%ddecdp(1:stars(i)%npastastro,1:nparamglob)
 			  pder(idx:idx+stars(i)%npastastro-1,nparamglob+(istar-1)*nparamloc+1:nparamglob+istar*nparamloc) = &
 			        & stars(i)%ddecdp(1:stars(i)%npastastro,nparamglob+1:)
 			  sig(idx:idx+stars(i)%npastastro-1)=stars(i)%sigdec(1:stars(i)%npastastro)
 			  idx=idx+stars(i)%npastastro
 		  end if
 		  if (stars(i)%rvb)   then
 			  pder(idx:idx+stars(i)%npastrv-1,1:nparamglob)=stars(i)%drvdp(1:stars(i)%npastrv,1:nparamglob)
 			  pder(idx:idx+stars(i)%npastrv-1,nparamglob+(istar-1)*nparamloc+1:nparamglob+istar*nparamloc) = &
 			        & stars(i)%drvdp(1:stars(i)%npastrv,nparamglob+1:)
 			  sig(idx:idx+stars(i)%npastrv-1)=stars(i)%sigrv(1:stars(i)%npastrv)
 			  idx=idx+stars(i)%npastrv
 		  end if
		  end if
 	  end do
 	  if (npriors>0)then
 		  do i=1,npriors
 			  pder(idx,priors(i)%idx)=1._rd
 			  sig(idx)=priors(i)%sig
 			  idx=idx+1
 		  end do
 	  end if!priors
   
     ! 4. Computation of the sigma    
 	  call fitsvd(pder,OMC,sig,par,parmask,cov,chi2)


     ! 5. write results in "local" file
     !call writeResults(j,t,parmask,npartofit)
     call writeAllUncertainties(t,foutglob)      
     deallocate(pder,omc,sig,par,parmask)
   else
     call writeNoChanges(j,t,lastFit)
   end if! changed
   
   call writeAllUncertainties(t,foutgloball)
   call write_starsused(foutstarnames,j,t)
 end do!i

 close(foutglob);close(foutgloball); close(foutstarnames)
end subroutine iterSensitivity_fullfit

subroutine writeNoChanges(count,t,last)
  use constantsMod, only : dirout, name,extres
  integer(id), intent(in) :: count,last
  real(rd), intent(in)    :: t
  
  character(len=4) :: format_string
  character(len=4) ::idxstr
  if (count < 10) then
    format_string = "(I1)"
  elseif (count<100) then
    format_string = "(I2)"
  elseif (count<1000) then
    format_string = "(I3)"
  else
    format_string = "(I4)"
  endif

  write (idxstr,format_string) count
  open(unit=foutloc,file = trim(dir_res)//'/'//trim(name)//'_'//trim(idxstr)//extres,action='write',status='replace')
  write(foutloc,*)" Iteration: ", count
  write(foutloc,*)" Threshold time: ", t
  write(foutloc,*)" No changes with respect to simu number:",last
  write(foutloc,*)" See output file named: "

  if (last < 10) then
    format_string = "(I1)"
  elseif (last<100) then
    format_string = "(I2)"
  elseif (last<1000) then
    format_string = "(I3)"
  else
    format_string = "(I4)"
  endif
  write (idxstr,format_string) last  
  write(foutloc,*)trim(name)//'_'//trim(idxstr)//extres
  
  
  close(foutloc)
end subroutine writeNoChanges

subroutine writeAllUncertainties(t,f)
	use constantsMod
	integer(id), intent(in) :: f
	real(rd), intent(in) :: t
	integer(id) :: i,j,k
	write(f,'(1x,G12.7,5x,i4,2x)', advance='no')t,n_stars_in
	k=0
		do j=1,nparamglob
				if (tofitglobal(j)) then
				  if (fit_type==0) then
					  write(f,'(1x,G20.15,3x)', advance='no') sqrt(cov(j,j)) 
				  else 
				    k=k+1
				    write(f,'(1x,G20.15,3x)', advance='no') sqrt(uinv(k,k)) 
			    end if
				else
					write(f,'(3x,i2,9x)', advance='no')  -1
				end if
		end do!j
		if (fit_type==0) then
    j=j-1
    do i=1,nstars
      if (stars(i)%included) then
        do k=1,nparamloc
          j=j+1
				  if (stars(i)%tofit(k)) then
					  write(f,'(1x,G10.5,1x)', advance='no') sqrt(cov(j,j))  
				  else
					  write(f,'(3x,i2,7x)', advance='no')  -1
				  end if
			  end do!k
		  else
		    do k=1,nparamloc
		      write(f,'(3x,i2,7x)', advance='no')  -1
	      end do
			end if
		end do !j
	end if
    write(f,'(A)',advance='yes')
end subroutine writeAllUncertainties

subroutine write_starsused(f, count,t)
  integer(id), intent(in) :: f
  integer(id), intent(in) :: count
  real(rd), intent(in)    :: t
  integer(id) :: i,j
  character(len=4) :: format_string
  
  if (count < 10) then
        format_string = "(I1)"
  elseif (count<100) then
            format_string = "(I2)"
  elseif (count<1000) then
         format_string = "(I3)"
  else
           format_string = "(I4)"
  endif



  write(f,'(i3,3x)', advance='no') count
  write(f,'(G10.6,3x)', advance='no')  t 
  j=0
  do i=1,nstars
    if (stars(i)%included) then
		j=j+1
        write(f,'(A,3x)',advance='no') trim(stars(i)%name)
    end if
  end do!
  write(f,'(i3,3x)', advance='no') j
  write(f,*)


end subroutine write_starsused


subroutine writeResults(count,t,parmask,nparam)
  use constantsMod, only:priors,npriors,nparamloc,nparamglob,name,dirout,extres,tofitGlobal
  use modelinterface, only: nameglob,nameloc,globpar,modelname
  integer(id), intent(in) :: count
  real(rd), intent(in)    :: t
  logical, dimension(:), intent(in) :: parmask
  integer(id), intent(in) :: nparam
  character(len=4) :: format_string
  character(len=4) ::idxstr
  integer(id) :: i,j,istar,idx,ic,jc
  
  if (count < 10) then
        format_string = "(I1)"
  elseif (count<100) then
            format_string = "(I2)"
  elseif (count<1000) then
         format_string = "(I3)"
  else
           format_string = "(I4)"
  endif

  write (idxstr,format_string) count
  open(unit=foutloc,file = trim(dir_res)//'/'//trim(name)//'_'//trim(idxstr)//extres,action='write',status='replace')
  write(foutloc,*)" Iteration: ", count
  write(foutloc,*)" Threshold time: ", t 

  
  write(foutloc,*)  "  ====================================================  "
  write(foutloc,*)  "         INPUT    "
  write(foutloc,*)  "  ====================================================  "
  write(foutloc,*)   " Covariance analysis made using the model: 			", trim(modelname)
  write(foutloc,*)   "     - Global parameteres used: "
  do i=1,nparamglob
	  if (toFitGlobal(i)) then
		  write(foutloc,'(A,A,A,G10.4,A)') "     - ",nameGlob(i)," : ",globPar(i),"   -- FITTED"
	  else
		  write(foutloc,*) "     - ",nameGlob(i)," : ",globPar(i),"   -- NOT FITTED"
	  end if
  end do!i
  
  write(foutloc,*)  " Number of stars simulated: ",n_stars_in
  

  do i=1,nstars
    if (stars(i)%included) then
	    write(foutloc,*) "    ",trim(stars(i)%name)," with local parameters: "
	    do j=1,nparamloc
		    if (stars(i)%tofit(j)) then
			    write(foutloc,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- FITTED"
		    else
			    write(foutloc,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- NOT FITTED"
		    end if
	    end do!j
	    if (stars(i)%astrob) then
		    write(foutloc,*)   '    with ',stars(i)%npastastro,' astro data from file: ',trim(stars(i)%astrofile)
	    else
		    write(foutloc,*)   '    with NO astro data'
	    end if!astrob
	    if (stars(i)%rvb) then
		    write(foutloc,*)   '    with ',stars(i)%npastrv,' RV data from file: ',trim(stars(i)%rvfile)
	    else
		    write(foutloc,*)   '    with NO RV data'
	    end if!astrob
	    write(foutloc,*)""
    end if
  end do!i


  write(foutloc,*)' Priors :'
  if(npriors<=0) then 
	  write(foutloc,*)'   No priors considered'
  else
	  write(foutloc,'(i3,A)') npriors, " priors considered:"
	  do i=1,npriors
		  write(foutloc,'(A,i3,A)',advance='no') " Prior n. ", i, ' on parameter '
		  if (priors(i)%idx<= nparamglob) then
			  write(foutloc,*) nameGlob(priors(i)%idx),' with uncertainty of ', priors(i)%sig
		  else
			  istar=(priors(i)%idx-1-nparamglob)/nparamloc+1
			  write(foutloc,'(A,A,A,A,G10.4)') nameloc(priors(i)%idx-nparamglob-(istar-1)*nparamloc), ' from star ',trim(stars(istar)%name),&
				  &' with uncertainty of ', priors(i)%sig
		  end if
	  end do!i
  end if
  write(foutloc,*)  "  ====================================================  "
  write(foutloc,*)  "         RESULTS    "
  write(foutloc,*)  "  ====================================================  "
  write(foutloc,*)" Resulting uncertainties: "
  write(foutloc,*)"  == Global parameters: =="
  idx=1
  do i=1,nparamglob
    if (tofitglobal(i)) then
      if (fit_type==0) then
        write(foutloc,'(2x,A5,2x,G15.8)') nameGlob(i), sqrt(cov(i,i))
      else if (fit_type==1) then
        write(foutloc,'(2x,A5,2x,G15.8)') nameGlob(i), sqrt(Uinv(idx,idx))
        idx=idx+1
      end if
    else
      write(foutloc,'(2x,A5,2x,A)') nameGlob(i), '  -NF- '
    end if
  end do!i
if (fit_type==0) then
  write(foutloc,*)
  write(foutloc,*)"  == Local parameters: =="
  idx=nparamglob+1
  do i=1,nstars
    if (stars(i)%included) then
  		write(foutloc,*)"  Star: ",stars(i)%name
  		do j=1,nparamloc
  			if (stars(i)%tofit(j)) then
  				write(foutloc,'(2x,A5,2x,G15.8)') nameLoc(j),sqrt(cov(idx,idx))
  			else
  				write(foutloc,'(2x,A5,2x,A)') nameLoc(j),'  -NF-  '
  			end if
  			idx=idx+1
  		end do!j
  		write(foutloc,*)""
    end if!star_in(i)
  end do!i


  write(foutloc,*)""
  write(foutloc,*)  "  == Correlation matrix == "
  do i=1,nparam
    do j=1,nparam
      if(parmask(i).and.parmask(j)) write(foutloc,'(F12.6,2x)',ADVANCE='NO') cov(i,j)/sqrt(cov(i,i)*cov(j,j))
      if ((j==nparamglob).and.parmask(i)) write(foutloc,'(A)',ADVANCE='no')"    "
      if ((mod(j-nparamglob,nparamloc)==0).and.(j>nparamglob).and.parmask(i))write(foutloc,'(A)',ADVANCE='no')"    "
    end do!j
    if (parmask(i)) write(foutloc,'(A)',ADVANCE='YES')
    if (i==nparamglob)write(foutloc,*)" "
    if ((mod(i-nparamglob,nparamloc)==0).and.(i>nparamglob))write(foutloc,*)" "
  end do!i
else if (fit_type==1) then
  write(foutloc,*)""
  write(foutloc,*)  "  == Correlation matrix == "
  do i=1,nparam
    do j=1,nparam
        write(foutloc,'(F12.6,2x)',ADVANCE='NO') uinv(i,j)/sqrt(uinv(i,i)*uinv(j,j))
    end do!j
    write(foutloc,'(A)',ADVANCE='YES')
  end do
end if
  close(foutloc)
end subroutine writeResults

subroutine writeInput()
use constantsMod
use modelinterface
integer(id) :: i,j,istar
!==================writeOut
open(unit=fout,file = trim(dirout)//'/'//trim(name)//'.inp',action='write',status='replace')
write(*,*)'Output file: ',trim(dirout)//'/'//trim(name)//extres



write(fout,*)  "  ====================================================  "
write(fout,*)  "         INPUT    "
write(fout,*)  "  ====================================================  "
write(fout,*)   " Sensitivity analysis made using the model: 			", trim(modelname)
write(fout,*)   "     - Global parameteres used: "
do i=1,nparamglob
	if (toFitGlobal(i)) then
		 write(fout,'(A,A,A,G10.4,A)') "     - ",nameGlob(i)," : ",globPar(i),"   -- FITTED"
	else
		 write(fout,*) "     - ",nameGlob(i)," : ",globPar(i),"   -- NOT FITTED"
	end if
end do!i
write(fout,*)  " Number of stars simulated: ",nstars
do i=1,nstars
	write(fout,*) "    ",trim(stars(i)%name)," with local parameters: "
	do j=1,nparamloc
		if (stars(i)%tofit(j)) then
			write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- FITTED"
		else
			write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- NOT FITTED"
		end if
	end do!j
	if (stars(i)%astrob) then
		write(fout,*)   '    with ',stars(i)%na,' astro data from file: ',trim(stars(i)%astrofile)
	else
		write(fout,*)   '    with NO astro data'
	end if!astrob
	if (stars(i)%rvb) then
		write(fout,*)   '    with ',stars(i)%nrv,' RV data from file: ',trim(stars(i)%rvfile)
	else
		write(fout,*)   '    with NO RV data'
	end if!astrob
	write(fout,*)""
end do!i


write(fout,*)' Priors :'
if(npriors<=0) then 
	write(fout,*)'   No priors considered'
else
	write(fout,'(i3,A)') npriors, " priors considered:"
	do i=1,npriors
		write(fout,'(A,i3,A)',advance='no') " Prior n. ", i, ' on parameter '
		if (priors(i)%idx<= nparamglob) then
			write(fout,*) nameGlob(priors(i)%idx),' with uncertainty of ', priors(i)%sig
		else
			istar=(priors(i)%idx-1-nparamglob)/nparamloc+1
			write(fout,'(A,A,A,A,G10.4)') nameloc(priors(i)%idx-nparamglob-(istar-1)*nparamloc), ' from star ',trim(stars(istar)%name),&
				&' with uncertainty of ', priors(i)%sig
		end if
	end do!i
end if

write(fout,*)  "  ====================================================  "
write(fout,*)  "      SENSITIVITY TOOL INPUT    "
write(fout,*)  "  ====================================================  "

write(fout,*) 'Number of iteration:							', Niteration
write(fout,*) 'Starting epoh          					', starting_epoch
write(fout,*) 'Starting epoh          					', end_epoch
if (fit_type==0)then
  write(fout,*) 'Inversion of the full covariance matrix          					'
else if (fit_type==1)then
  write(fout,*) 'Partial nversion of the covariance matrix using Schur complement'	
else
  write(*,*) "Value of fit_type parameter (line 3 from input file) not correct"
  stop
end if

write(fout,*) 'Increment:                                  ', increment, 'year'

close(fout)
end subroutine writeInput
end module sensitivityMod
