module weatherMod
  use kindtype
	use starsMod, only: nstars, star, stars
  implicit none

	public   ::  initWeather,iterWeather, initcov
	private ::  list_insert,list_count, load_star_Chain,updatecov, init_random_seed,writeAllUncertainties, &
	      & write_intro_results, write_res_results

  type, private :: epoch
    integer(id), dimension(:), allocatable :: idx_stars
    integer(id), dimension(:), allocatable :: idx_epochs    
    integer(id) :: nstars=0
    real(rd)    :: t=-1
    logical :: isUsed=.false.
  end type epoch

  type, private :: list_epoch
    type(list_epoch), pointer :: next=>null()
    type(epoch)         :: ep
  end type list_epoch

	real(rd), private, save :: end_epoch	
  real(rd), private, save :: increment
		
	real(rd), private, save :: starting_epoch	
	real(rd), private, save :: ratio_loss
	integer(id),private, save     :: Niteration     !< number of time periods
	
	integer(id), private,save :: nfutastro
  integer(id), private,save :: nfutrv
  
	logical, dimension(:), private, allocatable    :: stars_in
  integer(id), private, save                     :: n_stars_in
	
	
	real(rd), dimension(:,:), allocatable, private    :: pastcov !< covariance matrix 
	real(rd), dimension(:,:), allocatable, private    :: cov !< covariance matrix 
	
	integer(id), private, save    :: npartofit !<
	integer(id), private, save :: npastobs
	
	integer(id), private, dimension(:), allocatable :: idxPar
	
	integer(id), parameter, private :: fout=56, foutloc=57, foutglob=58, foutgloball=59
	
	type(list_epoch), private, pointer :: list_astro
	type(list_epoch), private, pointer :: list_rv
	
	real(rd), private, dimension(:), allocatable :: fixed_astro
	real(rd), private, dimension(:), allocatable :: fixed_rv
	integer(id) , private, save :: nfixed_astro
	integer(id) , private, save :: nfixed_RV	
contains

integer function list_count( list )
    type(list_epoch), pointer  :: list

    type(list_epoch), pointer  :: current
    type(list_epoch), pointer  :: next

    if ( associated(list) ) then
        list_count = 1
        current => list
        do while ( associated(current%next) )
            current => current%next
            list_count = list_count + 1
        enddo
    else
        list_count = 0
    endif
end function list_count

subroutine list_insert( elem, ep )
    type(list_epoch), pointer  :: elem
    type(epoch), intent(in)    :: ep
    
    type(list_epoch), pointer :: next
    
    if (elem%ep%t>0._rd) then
      allocate(next)

      next%next => elem%next
      elem%next => next
      next%ep =  ep
    else
      elem%ep = ep
    end if
end subroutine list_insert

!


subroutine  initWeather(nameIn)
  use constantsMod, only: dirout, name,signext,timenext,priors,npriors,tofitglobal,nparamglob,nparamloc, nparam,&
        &  nglobtofit,frommjd2year
  use modelInterface, only: setModel, computePartials

	implicit none
	character(len=*), intent(in) :: nameIn
	integer :: fittmp,nohairtmp
	integer(id) :: i,j,idxtmp,idxtmp2
	integer(id) :: itmp,istar
	logical(lgt) :: exist

	character(len=15) :: modelStr
	type(list_epoch), pointer :: list

	allocate(list_astro,list_rv)

	!
	!Reading the file
	!
	inquire(file=nameIn, EXIST=exist) 
	if (.NOT.exist) then
		WRITE(*,*) "ERROR : File ", trim(nameIn)," does not exist"
		STOP
	endif
	open(unit=55,file=nameIn,action='read')
	
	read(55,'(A)') dirout
#ifdef IFORT	 
	inquire(directory=trim(dirout),exist=exist)
#else	
	inquire(file=trim(dirout),exist=exist)
#endif	

	if (.not.exist) then
		write(*,*) 'Creating folder : ', trim(dirout)
		call system(' mkdir '//trim(dirout))
	end if
	!write(*,*) exist,trim(dirout)
	
	!===== 2: name of the file	
	read(55,'(A)') name	
	write(*,*) 'File:', name
		
	read(55,*) niteration, starting_epoch,ratio_loss
	
  read(55,*) nfixed_astro
  if (nfixed_astro>0) then
    allocate(fixed_astro(nfixed_astro))
    read(55,*)fixed_astro
    call fromMJD2year(fixed_astro)
    write(*,*) fixed_astro
  end if!nastro_fixed>0
	read(55,*) nfixed_RV
   if (nfixed_RV>0) then
     allocate(fixed_RV(nfixed_RV))
     read(55,*)fixed_RV
     call fromMJD2year(fixed_RV)
   end if!nastro_fixed>0
   
	read(55,'(A)') modelStr

	read(55,*) npriors
	if (npriors>0) then
		allocate(priors(npriors))
		do i=1,npriors
			read(55,*)priors(i)%idx,priors(i)%sig
		end do !i
	end if!n priors>0
	
	call setModel(modelstr,55)
	npartofit=nglobtofit
	
	read(55,'(i2)') nstars
	nparam=nparamglob+nstars*nparamloc
	
	if(allocated(idxPar)) deallocate(idxPar)
	allocate(idxPar(nparam))
	
	if (nstars<1) then
		write(*,*) "ERROR : number of stars should be at least 1 and it is: ",nstars
		stop
	end if
	if (npriors>0) then
		do i=1,npriors
			if (priors(i)%idx>nparam) then
				write(*,*) "ERROR: index of prior ", i, " too large: ",priors(i)%idx
				stop
			end if
			if (priors(i)%sig<=0._rd) then
				write(*,*)"ERROR: non positive sigma for prior ", i
				stop
			end if
		end do!i
	end if!npriors

  if (allocated(stars)) deallocate(stars)
  if (allocated(stars_in)) deallocate(stars_in)		
	allocate(stars(nstars))
	allocate(stars_in(nstars))
	idxtmp=1
	idxtmp2=1
	do i=1,nparamglob
		if (toFitGlobal(i)) then
			idxPar(idxtmp2)=idxtmp
			idxtmp=idxtmp+1
		else
			idxPar(idxtmp2)=-1
		end if
		idxtmp2=idxtmp2+1
	end do
	npastobs=0
	do i=1, nstars
		allocate(stars(i)%par(nparamloc))
		allocate(stars(i)%tofit(nparamloc))
		
		read(55,*)stars(i)%name
		read(55,'(A)')stars(i)%astroFile
		read(55,'(A)')stars(i)%rvFile
		stars(i)%ntofit=0
		do j=1,nparamloc
			read(55,*) stars(i)%par(j),fittmp
			stars(i)%tofit(j)=(fittmp==1)
			if (stars(i)%tofit(j))  stars(i)%ntofit=stars(i)%ntofit+1
			npartofit=npartofit+fittmp
			if (stars(i)%tofit(j)) then
				idxPar(idxtmp2)=idxtmp
				idxtmp=idxtmp+1
			else
				idxPar(idxtmp2)=-1
			end if
			idxtmp2=idxtmp2+1
		end do!j
		call load_star_Chain(stars(i),i)
		npastobs = npastobs+ 2*stars(i)%npastastro + stars(i)%npastrv
	end do! i

	close(55)

  nfutastro=list_count(list_astro)
  nfutrv=list_count(list_rv)

	call writeinput()
  call computepartials()
end subroutine  initWeather


subroutine initCov()
	use constantsMod, only: npriors,priors,nparamglob,tofitglobal,nparamloc
	use fitMod, only: fitsvd,computeSig
	integer(id) :: i,idxObs,j
	real(rd), dimension(:,:), allocatable :: pder
	real(rd), dimension(:), allocatable   :: sig,omc,par
	logical, dimension(:), allocatable :: parmask
	real(rd) :: chi2
	logical :: exist
	allocate(pder(npastobs+npriors,npartofit))
	allocate(sig(npastobs+npriors),omc(npastobs+npriors),par(npartofit),parmask(npartofit))
	omc=0._rd; parmask=.true.
	
	if (allocated(cov) ) deallocate(cov)
	allocate(cov(npartofit,npartofit))
	write(*,*)" =================================== "
	write(*,*)"  Preliminary step: loading the partials"
	idxObs=1
	do i=1,nstars
		if (stars(i)%npastastro>0)then
			do j=1,nparamGlob
				if (tofitglobal(j)) then
					pder(idxObs:idxObs+stars(i)%npastastro-1,idxPar(j))=stars(i)%dradp(1:stars(i)%npastastro,j)
					pder(idxObs+stars(i)%npastastro:idxObs+2*stars(i)%npastastro-1,idxPar(j))=stars(i)%ddecdp(1:stars(i)%npastastro,j)
				end if
			end do!j			
			do j=1,nparamLoc
				if (stars(i)%tofit(j)) then
					pder(idxObs:idxObs+stars(i)%npastastro-1,idxPar(nparamglob+(i-1)*nparamloc+j))  &
							& =stars(i)%dradp(1:stars(i)%npastastro,nparamglob+j)
					pder(idxObs+stars(i)%npastastro:idxObs+2*stars(i)%npastastro-1,idxPar(nparamglob+(i-1)*nparamloc+j)) &
							& = stars(i)%ddecdp(1:stars(i)%npastastro,nparamglob+j)
				end if
			end do!j
			sig(idxObs:idxObs+stars(i)%npastastro-1)=stars(i)%sigra(1:stars(i)%npastastro)
			sig(idxObs+stars(i)%npastastro:idxObs+2*stars(i)%npastastro-1)=stars(i)%sigdec(1:stars(i)%npastastro)
			idxObs=idxObs+2*stars(i)%npastastro
		end if !astro
		
		if (stars(i)%npastrv>0) then
			do j=1,nparamGlob
				if (tofitglobal(j)) then

					pder(idxObs:idxObs+stars(i)%npastrv-1,idxPar(j))=stars(i)%drvdp(1:stars(i)%npastrv,j)
				end if
			end do!j			
			do j=1,nparamLoc
				if (stars(i)%tofit(j)) then
					pder(idxObs:idxObs+stars(i)%npastrv-1,idxPar(nparamglob+(i-1)*nparamloc+j))  &
							& =stars(i)%drvdp(1:stars(i)%npastrv,nparamglob+j)
				end if
			end do!j
			sig(idxObs:idxObs+stars(i)%npastrv-1)=stars(i)%sigrv(1:stars(i)%npastrv)
			idxObs=idxObs+stars(i)%npastrv
		end if!RV
	end do!i - nstars
	if (npriors>0) then
		do i=1,npriors
			pder(idxObs,priors(i)%idx) = 1._rd
			sig(idxObs)=priors(i)%sig
			idxObs=idxObs+1
		end do!i
	end if!nrpiors
	
	
	write(*,*)"      Inversion of the normal matrix"
	call fitsvd(pder,OMC,sig,par,parmask,cov,chi2)
	write(*,*)" Preliminary step performed successfully"
	write(*,*)" =================================== "

	if (allocated(pastcov) ) deallocate(pastcov)
	allocate(pastcov(npartofit,npartofit))
	pastcov = cov

!===============past cov in terminal!
!	do i=1,npartofit
!			write(*,'(2x,G10.5)') sqrt(cov(i,i))
!	end do

end subroutine initCov



subroutine load_star_Chain(s,idx)
	use constantsMod, only : countLines,nparamloc,nparamglob,fromMJD2year
	use starsMod, only: loadstar
	type(star), intent(inout) :: s
	integer(id), intent(in) :: idx
	real(rd),dimension(2) :: tmp
	real(rd) :: tmpyr
	integer(id):: nmean
	integer(id) ::i
	type(list_epoch), pointer :: current
	type(epoch) :: tmp_ep
	
	call loadstar(s)
	if (s%astrob) then
    s%npastastro = maxloc(s%tastro,1,mask=s%tastro.le.starting_epoch)
  else
    s%npastastro=0
  end if
  
  if (s%rvb) then
    s%npastrv = maxloc(s%trv,1,mask=s%trv.le.starting_epoch)
  else
    s%npastrv=0
  end if

  ! FILLING THE LINKED CHAINS
  ! ASTRO
  current =>list_astro
  do i=s%npastastro+1, s%na
!    write(*,*) 'Ici',s%tastro(i)
    if ((s%tastro(i)<current%ep%t).and.(current%ep%t>0)) then
      write(*,*) 'ERROR: it seems the astro file for ',trim(s%name),' is not chronologically ordered for the MJD'
      write(*,*)' at time ',s%tastro(i)
      stop
    end if
    do while(.true.)
      if (current%ep%t==s%tastro(i)) then
!        write(*,*)'Overload'
        current%ep%nstars=current%ep%nstars+1
        current%ep%idx_stars(current%ep%nstars)=idx
        current%ep%idx_epochs(current%ep%nstars)=i
        exit
        elseif (.not.associated(current%next)) then ! case where we are at the end of the list
!          write(*,*)'No next'
          tmp_ep%t=s%tastro(i)
          tmp_ep%nstars=1
!          tmp_ep%isAstro=.True.
          if (.not.allocated(tmp_ep%idx_stars)) then
            allocate(tmp_ep%idx_stars(nstars))
            tmp_ep%idx_stars=-1
          end if
          if (.not.allocated(tmp_ep%idx_epochs)) then
            allocate(tmp_ep%idx_epochs(nstars))
            tmp_ep%idx_epochs=-1
          end if
          tmp_ep%idx_stars(1)=idx
          tmp_ep%idx_epochs(1)=i
        call list_insert(current,tmp_ep)
        if(associated(current%next)) current=>current%next
        exit
        elseif (current%next%ep%t >s%tastro(i)) then !case where the next element is a time larger, we insert a new node in between
!          write(*,*)'Insert between'
          tmp_ep%t=s%tastro(i)
          tmp_ep%nstars=1
!          tmp_ep%isAstro=.True.
          if (.not.allocated(tmp_ep%idx_stars)) then
            allocate(tmp_ep%idx_stars(nstars))
            tmp_ep%idx_stars=-1
          end if
          if (.not.allocated(tmp_ep%idx_epochs)) then
            allocate(tmp_ep%idx_epochs(nstars))
            tmp_ep%idx_epochs=-1
          end if
          tmp_ep%idx_stars(1)=idx
          tmp_ep%idx_epochs(1)=i
          call list_insert(current,tmp_ep)
          exit
        else
          current=>current%next
        end if
      end do !.true.
  end do!i
  
  
  ! RV
   current =>list_rv
   do i=s%npastrv+1, s%nrv
     if ((s%trv(i)<current%ep%t).and.(current%ep%t>0)) then
       write(*,*) 'ERROR: it seems the RV file for ',trim(s%name),' is not chronologically ordered for the MJD'
       write(*,*)' at time ',s%trv(i)
       stop
     end if
     do while(.true.)
       if (current%ep%t==s%trv(i)) then
!         write(*,*)'Overload'
         current%ep%nstars=current%ep%nstars+1
         current%ep%idx_stars(current%ep%nstars)=idx
         current%ep%idx_epochs(current%ep%nstars)=i
         exit
        elseif (.not.associated(current%next)) then ! case where we are at the end of the list
!          write(*,*)'No next'
          tmp_ep%t=s%trv(i)
          tmp_ep%nstars=1
!          tmp_ep%isAstro=.False.
          if (.not.allocated(tmp_ep%idx_stars)) then
            allocate(tmp_ep%idx_stars(nstars))
            tmp_ep%idx_stars=-1
          end if
          if (.not.allocated(tmp_ep%idx_epochs)) then
            allocate(tmp_ep%idx_epochs(nstars))
            tmp_ep%idx_epochs=-1
          end if
          tmp_ep%idx_stars(1)=idx
          tmp_ep%idx_epochs(1)=i
          call list_insert(current,tmp_ep)
          if(associated(current%next)) current=>current%next
          exit
        elseif (current%next%ep%t >s%trv(i)) then !case where the next element is a time larger, we insert a new node in between
!          write(*,*)'Insert between'
          tmp_ep%t=s%trv(i)
          tmp_ep%nstars=1
!          tmp_ep%isAstro=.False.
          if (.not.allocated(tmp_ep%idx_stars)) then
            allocate(tmp_ep%idx_stars(nstars))
            tmp_ep%idx_stars=-1
          end if
          if (.not.allocated(tmp_ep%idx_epochs)) then
            allocate(tmp_ep%idx_epochs(nstars))
            tmp_ep%idx_epochs=-1
          end if
          tmp_ep%idx_stars(1)=idx
          tmp_ep%idx_epochs(1)=i
          call list_insert(current,tmp_ep)
          exit
        else
          current=>current%next
        end if
       end do !.true.
   end do!i
end subroutine load_star_Chain


subroutine iterWeather()
!	use fitMod, only: fitsvd
	use constantsMod,only: dirout,name
	integer(id)     :: i,k,nastroused,nastrolost,nrvused,nrvlost
	type(list_epoch),pointer :: current
	real(rd) :: rnd

!	real(rd)    :: t, chi2

!	real(rd), dimension(:), allocatable   :: omc,sig,par
!	logical, dimension(:), allocatable    :: parmask
	
	logical :: changed
  CALL init_random_seed() 

	write(*,*)" Start of the iterations for the weather"

  open(unit=foutgloball,file = trim(dirout)//'/'//trim(name)//'.wea',action='write',status='replace')
 ! open(unit=foutglob,file = trim(dirout)//'/'//trim(name)//'.sen',action='write',status='replace')
!  lastFit=0
  
  do i=1,niteration
    nastroused=0
    nastrolost=0
    nrvused=0
    nrvlost=0
    write(*,*) 'Iteration j=', i, ' out of ',niteration
    
    cov = pastcov
    call write_intro_Results(i)
    ! 1. Draw randomly which night are lost
    ! LOOP on ASTRO
    write(foutloc,'(A)')'   1) Nights for astro: '
    current=> list_astro
    do while (current%ep%t > 0) 
      call RANDOM_NUMBER(rnd)
      current%ep%isUsed = rnd > ratio_loss
      if (nfixed_astro>0) then
      if (count(current%ep%t==fixed_astro)>0) current%ep%isUsed = .true.
      end if
      
      if(current%ep%isUsed) then ! if the night is not lost
        nastroused=nastroused+1
        write(foutloc,'(A,F11.4,A,i3,A)',advance='no') '   Epoch: ',current%ep%t,' observed with ',current%ep%nstars, ' stars: '
        do k=1,current%ep%nstars
          write(foutloc,'(A,A)',advance='no')trim(stars(current%ep%idx_stars(k))%name),', '
          ! RIGHT ASCENSION
          call updateCov(1,current%ep%idx_stars(k),current%ep%idx_epochs(k))
        	! DECLINATION
          call updateCov(2,current%ep%idx_stars(k),current%ep%idx_epochs(k))
        end do!k
        write(foutloc,*)
      else
        write(foutloc,'(A,F11.4,A,i3,A)') '   Epoch: ',current%ep%t,' lost'
        nastrolost=nastrolost+1
      end if
      
      if (associated(current%next)) then
        current=>current%next 
      else
        exit
      end if
    end do! has next - loop on astro

    ! LOOP on RV
    write(foutloc,*)
    write(foutloc,'(A)')'   2) Nights for RV: '
    current=> list_rv
    do while (current%ep%t > 0) 
      call RANDOM_NUMBER(rnd)
      current%ep%isUsed = rnd > ratio_loss
      
      if (nfixed_RV>0) then
        if (count(current%ep%t==fixed_RV)>0) current%ep%isUsed = .true.
      end if
          
      if(current%ep%isUsed) then ! if the night is not lostv
        nrvused=nrvused+1
        write(foutloc,'(A,F11.4,A,i3,A)',advance='no') '   Epoch: ',current%ep%t,' observed with ',current%ep%nstars, ' stars: ' 
        do k=1,current%ep%nstars
          write(foutloc,'(A,A)',advance='no')trim(stars(current%ep%idx_stars(k))%name),', '
          ! RV
          call updateCov(3,current%ep%idx_stars(k),current%ep%idx_epochs(k))
        end do!k
        write(foutloc,*)
      else
        nrvlost=nrvlost+1
        write(foutloc,'(A,F11.4,A,i3,A)') '   Epoch: ',current%ep%t,' lost'
      end if
      if (associated(current%next)) then!advance in the linked chain
        current=>current%next 
      else
        exit
      end if
    end do! has next - loop on astro
    write(foutloc,*)
    call writeAllUncertainties(i,nastroused,nastrolost,nrvused,nrvlost)
    call write_res_Results()
  end do!j
close(foutgloball)
end subroutine iterWeather

subroutine updateCov(type,idxstar,idxep)
  use constantsMod, only: nparamGlob,nparamLoc,tofitglobal
  integer, intent(in) :: type !1 = RA, 2=DEC and 3=RV
  integer(id), intent(in) ::idxstar,idxep
  real(rd),dimension(:,:), allocatable :: pder,PA
  integer(id) ::j

  allocate(pder(npartofit,1),PA(npartofit,1))
  pder=0._rd;PA=0._rd;
  
  do j=1,nparamGlob
		if (tofitglobal(j)) then
		  if (type==1) then
			  pder(idxPar(j),1)   = stars(idxStar)%dradp(idxep,j) 
		  elseif(type==2) then
		    pder(idxPar(j),1)   = stars(idxStar)%ddecdp(idxep,j) 
	    elseif(type==3) then
	      pder(idxPar(j),1)   = stars(idxStar)%drvdp(idxep,j) 
      end if
		end if
	end do!j	

	do j=1,nparamLoc
		if (stars(idxStar)%tofit(j)) then
		  if (type==1) then
			  pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),1) =stars(idxStar)%dradp( idxep ,nparamglob+j)
		  else if (type==2) then
		    pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),1) =stars(idxStar)%ddecdp( idxep ,nparamglob+j)
	    else if (type==3) then
	      pder(idxPar(nparamglob+(idxStar-1)*nparamloc+j),1) =stars(idxStar)%drvdp( idxep ,nparamglob+j)
			end if! type
		end if! tofit
	end do!j
	if (type==1) then
	  pder=pder/stars(idxStar)%sigra( idxep)
  elseif(type==2) then
	  pder=pder/stars(idxStar)%sigdec( idxep)
	elseif (type==3) then
	  pder=pder/stars(idxStar)%sigrv( idxep )
  end if
	! update
	PA=matmul(cov,pder)
	cov=cov-matmul(PA,transpose(PA))/(1._rd+dot_product(pder(:,1),PA(:,1)))
  deallocate(pder,PA)
end subroutine updateCov

subroutine writeAllUncertainties(k,naused,nalost,nrvused,nrvlost)
	use constantsMod
	   use modelinterface
	integer(id), intent(in) :: k,naused,nalost,nrvused,nrvlost
	integer(id) :: i,j,idx
	write(foutgloball,'(1x,i8,5x,i5,2x,i5,2x,i5,2x,i5,2x)', advance='no')k,naused,nalost,nrvused,nrvlost
	
	idx=1
   do i=1,nparamglob
     if (tofitglobal(i)) then
         write(foutgloball,'(1x,G10.5,1x)', advance='no') sqrt(cov(idx,idx))
         idx=idx+1
     else
       write(foutgloball,'(1x,G10.5,1x)', advance='no') -1
     end if
   end do!i

   do i=1,nstars
   		do j=1,nparamloc
   			if (stars(i)%tofit(j)) then
   				write(foutgloball,'(1x,G10.5,1x)', advance='no') sqrt(cov(idx,idx))
   				idx=idx+1
   			else
   				write(foutgloball,'(1x,G10.5,1x)', advance='no') -1
   			end if

   		end do!j
   end do!i
    write(foutgloball,'(A)',advance='yes')
end subroutine writeAllUncertainties

subroutine write_intro_results(count)
   use constantsMod
   use modelinterface
    integer(id), intent(in) :: count
    character(len=4) :: format_string
    character(len=8) ::idxstr
    integer(id) :: i,j,istar,idx,ic,jc,idxi,idxj,jstar

    if (count < 10) then
          format_string = "(I1)"
    elseif (count<100) then
              format_string = "(I2)"
    elseif (count<1000) then
           format_string = "(I3)"
    elseif (count<10000) then
            format_string = "(I4)"
    elseif (count<100000) then
          format_string = "(I5)"            
    else
             format_string = "(I6)"
    endif

    write (idxstr,format_string) count
    open(unit=foutloc,file = trim(dirout)//'/'//trim(name)//'_'//trim(idxstr)//extres,action='write',status='replace')
    write(foutloc,*)" Iteration: ", count


    write(foutloc,*)  "  ====================================================  "
    write(foutloc,*)  "         INPUT    "
    write(foutloc,*)  "  ====================================================  "
    write(foutloc,*)   " Covariance analysis made using the model: 			", trim(modelname)
    write(foutloc,*)   "     - Global parameteres used: "
    do i=1,nparamglob
  	  if (toFitGlobal(i)) then
  		  write(foutloc,'(A,A,A,G10.4,A)') "     - ",nameGlob(i)," : ",globPar(i),"   -- FITTED"
  	  else
  		  write(foutloc,*) "     - ",nameGlob(i)," : ",globPar(i),"   -- NOT FITTED"
  	  end if
    end do!i

    write(foutloc,*)  " Number of stars simulated: ",n_stars_in
    do i=1,nstars
      if (stars(i)%included) then
  	    write(foutloc,*) "    ",trim(stars(i)%name)," with local parameters: "
  	    do j=1,nparamloc
  		    if (stars(i)%tofit(j)) then
  			    write(foutloc,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- FITTED"
  		    else
  			    write(foutloc,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- NOT FITTED"
  		    end if
  	    end do!j
  	    if (stars(i)%astrob) then
  		    write(foutloc,*)   '    with ',stars(i)%npastastro,' astro data from file: ',trim(stars(i)%astrofile)
  	    else
  		    write(foutloc,*)   '    with NO astro data'
  	    end if!astrob
  	    if (stars(i)%rvb) then
  		    write(foutloc,*)   '    with ',stars(i)%npastrv,' RV data from file: ',trim(stars(i)%rvfile)
  	    else
  		    write(foutloc,*)   '    with NO RV data'
  	    end if!astrob
  	    write(foutloc,*)""
      end if
    end do!i


    write(foutloc,*)' Priors :'
    if(npriors<=0) then 
  	  write(foutloc,*)'   No priors considered'
    else
  	  write(foutloc,'(i3,A)') npriors, " priors considered:"
  	  do i=1,npriors
  		  write(foutloc,'(A,i3,A)',advance='no') " Prior n. ", i, ' on parameter '
  		  if (priors(i)%idx<= nparamglob) then
  			  write(foutloc,*) nameGlob(priors(i)%idx),' with uncertainty of ', priors(i)%sig
  		  else
  			  istar=(priors(i)%idx-1-nparamglob)/nparamloc+1
  write(foutloc,'(A,A,A,A,G10.4)') nameloc(priors(i)%idx-nparamglob-(istar-1)*nparamloc), ' from star ',trim(stars(istar)%name),&
  				  &' with uncertainty of ', priors(i)%sig
  		  end if
  	  end do!i
    end if
    write(foutloc,'(A)')' ===================================================='
    write(foutloc,'(A)')'  Nights observed and lost:'

end subroutine write_intro_results

subroutine write_res_Results()
    use constantsMod, only:nparamloc,nparamglob,tofitGlobal,nglobtofit
    use modelinterface, only: nameglob,nameloc
    integer(id) :: i,j,istar,idx,ic,jc,idxi,idxj,jstar
  write(foutloc,*)  "  ====================================================  "
  write(foutloc,*)  "         RESULTS    "
  write(foutloc,*)  "  ====================================================  "
  write(foutloc,*)" Resulting uncertainties: "
  write(foutloc,*)"  == Global parameters: =="
  idx=1
  do i=1,nparamglob
    if (tofitglobal(i)) then
        write(foutloc,'(2x,A5,2x,G10.5)') nameGlob(i), sqrt(cov(idx,idx))
        idx=idx+1
    else
      write(foutloc,'(2x,A5,2x,A)') nameGlob(i), '  -NF- '
    end if
  end do!i

  write(foutloc,*)
  write(foutloc,*)"  == Local parameters: =="
 ! idx=nparamglob+1
  do i=1,nstars

  		write(foutloc,*)"  Star: ",stars(i)%name
  		do j=1,nparamloc
  			if (stars(i)%tofit(j)) then
  				write(foutloc,'(2x,A5,2x,G10.5)') nameLoc(j),sqrt(cov(idx,idx))
  				idx=idx+1
  			else
  				write(foutloc,'(2x,A5,2x,A)') nameLoc(j),'  -NF-  '
  			end if
  			
  		end do!j
  		write(foutloc,*)""
  end do!i


  write(foutloc,*)""
  write(foutloc,*)  "  == Correlation matrix == "
  istar=1
  idxi=nglobtofit
  do i=1,npartofit
    do j=1,npartofit
       jstar=1
       idxj=nglobtofit
       write(foutloc,'(F12.6,2x)',ADVANCE='NO') cov(i,j)/sqrt(cov(i,i)*cov(j,j))
       if (j==nglobtofit) write(foutloc,'(A)',ADVANCE='no')"    "
       if (j>nglobtofit) then
         if (j==idxj+stars(jstar)%ntofit) then
           write(foutloc,'(A)',ADVANCE='no')"    "
           idxj=j
           jstar=jstar+1
         end if
       end if
    end do!j
     write(foutloc,'(A)',ADVANCE='YES')
     	if (i==nglobtofit) write(foutloc,'(A)',ADVANCE='YES')
     	if (i>nglobtofit) then
     	  if (i==idxi+stars(istar)%ntofit) then
           write(foutloc,'(A)',ADVANCE='YES')
           idxi=i
           istar=istar+1
         end if
   	  end if
  end do!i

  close(foutloc)
end subroutine write_res_Results


subroutine init_random_seed()
  implicit none
  integer(id), allocatable :: seed(:)
  integer(id) :: i, n, un, istat, dt(8), pid, t(2), s
  integer(id) :: count, tms
  
  call random_seed(size = n)
  allocate(seed(n))
  ! First try if the OS provides a random number generator
  open(newunit=un, file="/dev/urandom", access="stream", &
         form="unformatted", action="read", status="old", iostat=istat)
  if (istat == 0) then
    read(un) seed
    close(un)
  else
    ! Fallback to XOR:ing the current time and pid. The PID is
    ! useful in case one launches multiple instances of the same
    ! program in parallel.
    call system_clock(count)
    if (count /= 0) then
      t = transfer(count, t)
    else
      call date_and_time(values=dt)
      tms = (dt(1) - 1970) * 365_8 * 24 * 60 * 60 * 1000 &
               + dt(2) * 31_8 * 24 * 60 * 60 * 1000 &
               + dt(3) * 24 * 60 * 60 * 60 * 1000 &
               + dt(5) * 60 * 60 * 1000 &
               + dt(6) * 60 * 1000 + dt(7) * 1000 &
               + dt(8)
      t = transfer(tms, t)
    end if
    s = ieor(t(1), t(2))
    pid = getpid() + 1099279 ! Add a prime
    s = ieor(s, pid)
    if (n >= 3) then
      seed(1) = t(1) + 36269
      seed(2) = t(2) + 72551
      seed(3) = pid
      if (n > 3) then
         seed(4:) = s + 37 * (/ (i, i = 0, n - 4) /)
      end if
    else
      seed = s + 37 * (/ (i, i = 0, n - 1 ) /)
    end if
  end if
  call random_seed(put=seed)
end subroutine init_random_seed

subroutine writeinput()
use constantsMod
use modelinterface
integer(id) :: i,j,istar
logical :: exist
!==================writeOut
inquire(file=trim(dirout)//'/'//trim(name)//extres,exist=exist)
open(unit=fout,file = trim(dirout)//'/'//trim(name)//extres,action='write',status='replace')
write(*,*)trim(dirout)//'/'//trim(name)//extres



write(fout,*)  "  ====================================================  "
write(fout,*)  "         INPUT    "
write(fout,*)  "  ====================================================  "
write(fout,*)   " Covariance analysis made using the model: 			", trim(modelname)
write(fout,*)   "     - Global parameteres used: "
do i=1,nparamglob
	if (toFitGlobal(i)) then
		 write(fout,'(A,A,A,G10.4,A)') "     - ",nameGlob(i)," : ",globPar(i),"   -- FITTED"
	else
		 write(fout,*) "     - ",nameGlob(i)," : ",globPar(i),"   -- NOT FITTED"
	end if
end do!i
write(fout,*)  " Number of stars simulated: ",nstars
do i=1,nstars
	write(fout,*) "    ",trim(stars(i)%name)," with local parameters: "
	do j=1,nparamloc
		if (stars(i)%tofit(j)) then
			write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- FITTED"
		else
			write(fout,'(A,A,A,G10.4,A)') "     - ",nameLoc(j)," : ",stars(i)%par(j),"   -- NOT FITTED"
		end if
	end do!j
	if (stars(i)%astrob) then
		write(fout,*)   '    with ',stars(i)%na,' astro data from file: ',trim(stars(i)%astrofile)
	else
		write(fout,*)   '    with NO astro data'
	end if!astrob
	if (stars(i)%rvb) then
		write(fout,*)   '    with ',stars(i)%nrv,' RV data from file: ',trim(stars(i)%rvfile)
	else
		write(fout,*)   '    with NO RV data'
	end if!astrob
	write(fout,*)""
end do!i


write(fout,*)' Priors :'
if(npriors<=0) then 
	write(fout,*)'   No priors considered'
else
	write(fout,'(i3,A)') npriors, " priors considered:"
	do i=1,npriors
		write(fout,'(A,i3,A)',advance='no') " Prior n. ", i, ' on parameter '
		if (priors(i)%idx<= nparamglob) then
			write(fout,*) nameGlob(priors(i)%idx),' with uncertainty of ', priors(i)%sig
		else
			istar=(priors(i)%idx-1-nparamglob)/nparamloc+1
			write(fout,'(A,A,A,A,G10.4)') nameloc(priors(i)%idx-nparamglob-(istar-1)*nparamloc), ' from star ',trim(stars(istar)%name),&
				&' with uncertainty of ', priors(i)%sig
		end if
	end do!i
end if

write(fout,*)  "  ====================================================  "
write(fout,*)  "      SENSITIVITY TOOL INPUT    "
write(fout,*)  "  ====================================================  "

write(fout,*) 'Number of iteration:							', Niteration
write(fout,*) 'Starting epoh          					', starting_epoch
write(fout,*) 'Ratio of night loss              ',ratio_loss
write(fout,*) 'Number of future astro nights    ',nfutastro
write(fout,*) 'Number of future spectro nights  ',nfutrv

close(fout)
end subroutine writeinput
	
end module weatherMod
