#!/usr/bin/python
import numpy as np
import os
from shutil import copy2, copyfile
import subprocess as sb
import sys
import itertools as it



newRV=[57879., 57880., 57881., 57955., 57956. , 57957. , 57997.,  58208., 58217., 58221., 58334., 58359., 58361.]
newAstro =[57878., 57954., 57998.,58284.,58367. ]

nRV=len(newRV)
nAstro=len(newAstro)
nTot=nRV+nAstro

idx = range(nTot)

sigRV=20.
nMean=8

yr2day=365.25

def fromMjd2yr(tab):
    tabout=[]
    for m in tab:
        tabout.append(2000.0 + (m - 51544.5)/yr2day)
    return tabout


yrNewAstro=fromMjd2yr(newAstro)
yrNewRV=fromMjd2yr(newRV)


def astroSig(d):
    n=min(nMean,len(d))
    ra=[]
    dec=[]
    for i in range(-n,0):
        ds = d[i].split()
        ra.append(float(ds[3]))
        dec.append(float(ds[4]))
    return np.mean(ra),np.mean(dec)
         
def readInputData(stars):
    res=[]

    for s in stars:
        print "Reading star ", s[0]
        fin = open(s[1],'r')
        print "   Astro file: ",s[1]
        astro=fin.readlines()
        fin.close()
        if len(astro[-1].split())<2:
            astro.remove(astro[-1])
        sigra,sigdec = astroSig(astro)
        print "  Creating new astro points with uncertainties: ",sigra,sigdec
        nastro=[]
        for i in range(len(newAstro)):
            nastro.append(' %8.3f   0.00000  0.00000  %7.5f  %7.5f  %13.6f \n'%(yrNewAstro[i],sigra,sigdec,newAstro[i]))

        fin = open(s[2],'r')
        print "   RV file: ",s[2]        
        rv=fin.readlines()
        fin.close()
        if len(rv[-1].split())<2:
            astro.remove(rv[-1])
        nrv=[]
        if 'S0-2' in s[0] or 'S0-2' in s[2]: #next RV
            for i in range(len(newRV)):
                nrv.append(' %8.3f   0.00000   %7.2f  %13.6f \n'%(yrNewRV[i],sigRV,newRV[i]))
        res.append([astro, rv,nastro,nrv])
    return res

def nparFromMod(model):
    if model=='relativistic':
        nparam = 10
        nlocal=6
    elif model=='kepler':
        nparam = 7
        nlocal=6
    elif model=='keplerRedshift':
        nparam = 8
        nlocal=6
    elif model=='5thforce':
        nparam=9
        nlocal=6
    return nparam,nlocal

def readInput(inFile):
    inf=open(inFile,'r')
    lines=inf.readlines()
    inf.close()
    folder=lines[0].rstrip()
    
    model = lines[3].split()[0].rstrip()
    print '    Model: ',model
    nparam,nlocal=nparFromMod(model)
    npriors = int(lines[4].split()[0])
    nstars = int(lines[nparam+5+npriors].split()[0])
    print '        Num of global parameters: ',nparam
    print '        Num of local parameters: ',nlocal
    print '        Num of stars: ',nstars
    print '        Num of priors: ',npriors
    stars=[]
    
    
    # creating the data file
    for i in range(1,nstars+1):
        stars.append([lines[6+npriors+nparam+(i-1)*(nlocal+4)].rstrip() ,  lines[7+npriors+nparam+(i-1)*(nlocal+4)].rstrip() ,  lines[8+npriors+nparam+(i-1)*(nlocal+4)].rstrip()])


    input=[folder,nparam,nlocal,npriors,nstars,lines]
    return stars,input

def createDir(n):
    dir = os.path.dirname(n)
    if not os.path.isdir(dir):
        os.makedirs(dir)

        
def createInput(id,input):
    folder = input[0]
    npriors = input[3]
    npar = input[1]
    nl = input[2]
    ns = input[4]
    lines  = input[5]
    nameIn = folder+'/'+lines[1].rstrip()+'_wea_'+str(id)
    fout = open(nameIn,'w')
    
    fout.write(lines[0])
    nameSimu=lines[1].rstrip()+ '_' +str(id)
    
    fout.write(nameSimu+'\n')
    fout.write(lines[2])
    fout.write(lines[3])
    fout.write(lines[4])#npriors

    if npriors>0:
        for j in range(npriors):
            fout.write(lines[5+j])

    for j in range(npar+1):
        fout.write(lines[5+npriors+j])

    sName=[]
    for j in range(ns):
        fout.write(lines[6+npriors+npar+j*(nl+4)])
        
        sName.append([ folder+'/'+lines[7+npriors+npar+j*(nl+4)].rstrip() + '_w' + str(id) + ' \n' , folder+'/'+lines[8+npriors+npar+j*(nl+4)].rstrip() + '_w' + str(id) + ' \n'])
        fout.write(sName[-1][0])
        fout.write(sName[-1][1])
        
        for k in range(nl+1):
            fout.write(lines[9+npriors+k+npar+j*(nl+4)])

    return nameIn,nameSimu,sName

def createData(id,comb,nameDat,data):

    for idxs in range(len(nameDat)):
        if id==1:
            createDir(nameDat[idxs][0].rstrip())
            createDir(nameDat[idxs][1].rstrip())
    
        # ASTRO
        fouta = open(nameDat[idxs][0].rstrip(),'w')
        for l in data[idxs][0]:
            fouta.write(l.rstrip()+' \n')
        for i in range(nAstro):
            if i not in comb: #take observations not in list of "lost"
                fouta.write(data[idxs][2][i])
        fouta.close()

        #RV
        foutrv = open(nameDat[idxs][1].rstrip(),'w')
        for l in data[idxs][1]:
            foutrv.write(l.rstrip()+' \n')
        if (len(data[idxs][3])>0): #is additional RV (i.e. s0-2)
            for i in range(nRV):
                if i+nAstro not in comb: #take observations not in list of "lost"
                    foutrv.write(data[idxs][3][i])
        foutrv.close()

def readOutput(name):
    fout = open(name,'r')
    isGlobal=False
    sig=[]
    for l in fout:
        if isGlobal:
            ls=l.split()
            if len(ls)<1:
                break
            if ls[1]=='-NF-' :
                sig.append('-1')
            else:
                sig.append(ls[1])
        elif '== Global' in l:
            isGlobal=True
    fout.close()
    return sig

def iterationLostObs(stars,nlost,input,starsDat):
    iter = it.combinations(idx,nlost)
    count=0
    
    createDir(input[0]+'/bla')
    foutput = open(input[0]+'/'+input[5][1].rstrip()+'.wea','w')
    

    for comb in iter:
        count=count+1
        nameIn,nameOut,nameDat=createInput(count,input)
        nameOut=input[0]+'/'+nameOut+'.res'
        createData(count,comb,nameDat,starsDat)
        sb.check_call(['./main',nameIn])
        outSig = readOutput(nameOut)

        foutput.write(str(count) + '     '+ '    '.join(outSig)+'     ')
        for i in comb:
            if i <nAstro:
                foutput.write('A %8.3f  '%yrNewAstro[i])
            else:
                foutput.write('R %8.3f  '%yrNewRV[i-nAstro])
        foutput.write('\n')
    
    foutput.close()
        

    
nLost = 6
inFile = 'input_2stars'

if len (sys.argv)>1:
    inFile=sys.argv[1]
if len (sys.argv)>2:
    nLost = int(sys.argv[2])
    

print 'Number of observations lost: ',nLost,' out of ',nTot 
print 'Input file: ', inFile
stars,input=readInput(inFile)


starsDat = readInputData(stars)
iterationLostObs(stars,nLost,input,starsDat)

